//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#ifndef BUFMGR_H
#define BUFMGR_H

#include "global.h"



class BufMgr {
	
	struct Data {
		QString fname_;
		FILE_FORMAT format_;
		bool active_;
		QString text_;
		int line_;
		int col_;
		
		Data(QString fname, FILE_FORMAT format):
			fname_(fname), format_(format), active_(true), line_(1), col_(1) {}
		Data(): fname_(""), format_(FILE_UTF8), active_(false), line_(1), col_(1) {}
		~Data() {}
	};
	
public:
	BufMgr(): aindex_(0), mindex_(0), dbgname_(0) {}
	~BufMgr() {}
	// add a new buffer
	bool add(QString fname, FILE_FORMAT format);
	// delete a buffer
	bool del(int index);
	// get active editing index
	int getActiveIndex() {
		return aindex_;
	}
	void setActiveIndex(int index) {
		if (index < mindex_)
			aindex_ = index;
	}

	bool isEmpty();
	
	// get file name
	QString getFName(int index);
	// set file name
	void setFName(QString fname, int index);
	// get file format
	FILE_FORMAT getFormat(int index);
	// get text
	QString getText(int index);
	// set active file name
	bool setActiveFName(QString fname);
	// get active file name
	QString getActiveFName();
	FILE_FORMAT getActiveFormat();
	void  setActiveFormat(FILE_FORMAT format);
	// set active text
	void setActiveText(QString text);
	// get active text
	QString& getActiveText();
	// get active line
	int getActiveLine();
	// set active line
	void setActiveLine(int line);
	// get active column
	int getActiveCol();
	// set active coulumn
	void setActiveCol(int col);

	// activate next buffer
	void setActiveNext();
	// check if any buffer has been changed
	bool anyBufChanged();
	
	// debug 
	QString roDebug();
	
	// print name of unsaved buffers
	QString printUnsaved();
	
	// check if the file exists in the list
	bool hasFile(QString fname, int *index);

private:
	Data data_[MAXBUFMGR];
	// active index
	int aindex_;
	// max index
	int mindex_;

public:
	char *dbgname_;

};

#endif
