//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************
#include "colordialog.h"
#include <qmessagebox.h>
#include <qgroupbox.h>

#ifdef USING_QT3
#include <qfiledialog.h>
#endif
#ifdef USING_QT4
#include <q3filedialog.h>
#endif

#ifdef USING_QT3
#include <qlayout.h>
#include <qlabel.h>
#include <qhbox.h>
#include <qaccel.h>
#include <qmultilineedit.h>
#endif
#ifdef USING_QT4
#include <Q3VBoxLayout>
#include <QLabel>
#include <q3hbox.h>
#include <q3accel.h>
#include <q3multilineedit.h>
#endif

#include <qpushbutton.h>
#include <qcolordialog.h>



//*****************************************************************************/
// line dialog
//*****************************************************************************/
ColorDialog::ColorDialog(): QDialog(0, "color", true) {
	c_background_ = Global::instance()->c_background;
	c_foreground_ = Global::instance()->c_foreground;
	c_highlight_ = Global::instance()->c_highlight;
	c_tag1_ = Global::instance()->c_tag1;
	c_tag2_ = Global::instance()->c_tag2;
	c_tag3_ = Global::instance()->c_tag3;
	c_tag4_ = Global::instance()->c_tag4;

	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);
	
	// background
	Q3HBox *hbox_background = new Q3HBox(this);
	hbox_background->setSpacing(10);
	label_background_ = new QLabel("Background: screen background", hbox_background);
	label_background_->setPaletteForegroundColor(c_background_);
	new QWidget(hbox_background);
	new QWidget(hbox_background);
	button_background_ = new QPushButton("browse", hbox_background);

	// foreground
	Q3HBox *hbox_foreground = new Q3HBox(this);
	hbox_foreground->setSpacing(10);
	label_foreground_ = new QLabel("Foreground: normal text", hbox_foreground);
	label_foreground_->setPaletteForegroundColor(c_foreground_);
	new QWidget(hbox_foreground);
	new QWidget(hbox_foreground);
	button_foreground_ = new QPushButton("browse", hbox_foreground);

	// highlight
	Q3HBox *hbox_highlight = new Q3HBox(this);
	hbox_highlight->setSpacing(10);
	label_highlight_ = new QLabel("Highlight: selected text", hbox_highlight);
	label_highlight_->setPaletteForegroundColor(c_highlight_);
	new QWidget(hbox_highlight);
	new QWidget(hbox_highlight);
	button_highlight_ = new QPushButton("browse", hbox_highlight);

	// tag1
	Q3HBox *hbox_tag1 = new Q3HBox(this);
	hbox_tag1->setSpacing(10);
	label_tag1_ = new QLabel("Tag1: macro processor", hbox_tag1);
	label_tag1_->setPaletteForegroundColor(c_tag1_);
	new QWidget(hbox_tag1);
	new QWidget(hbox_tag1);
	button_tag1_ = new QPushButton("browse", hbox_tag1);

	// tag2
	Q3HBox *hbox_tag2 = new Q3HBox(this);
	hbox_tag2->setSpacing(10);
	label_tag2_ = new QLabel("Tag2: \"strings\"", hbox_tag2);
	label_tag2_->setPaletteForegroundColor(c_tag2_);
	new QWidget(hbox_tag2);
	new QWidget(hbox_tag2);
	button_tag2_ = new QPushButton("browse", hbox_tag2);

	// tag3
	Q3HBox *hbox_tag3 = new Q3HBox(this);
	hbox_tag3->setSpacing(10);
	label_tag3_ = new QLabel("Tag3: comments", hbox_tag3);
	label_tag3_->setPaletteForegroundColor(c_tag3_);
	new QWidget(hbox_tag3);
	new QWidget(hbox_tag3);
	button_tag3_ = new QPushButton("browse", hbox_tag3);

	// tag4
	Q3HBox *hbox_tag4 = new Q3HBox(this);
	hbox_tag4->setSpacing(10);
	label_tag4_ = new QLabel("Tag4: keywords", hbox_tag4);
	label_tag4_->setPaletteForegroundColor(c_tag4_);
	new QWidget(hbox_tag4);
	new QWidget(hbox_tag4);
	button_tag4_ = new QPushButton("browse", hbox_tag4);

	// accept/reject
	Q3HBox *hbox_accept = new Q3HBox(this);
	hbox_accept->setSpacing(10);
	button_reset_ = new QPushButton("Reset", hbox_accept);
	// some empty space
	new QWidget(hbox_accept);
	new QWidget(hbox_accept);
	button_ok_ = new QPushButton("Ok", hbox_accept);
	button_cancel_ = new QPushButton("Cancel", hbox_accept);

	connect(button_reset_, SIGNAL(clicked()), this, SLOT(clicked_reset()));
	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
	connect(button_background_, SIGNAL(clicked()), this, SLOT(clicked_background()));
	connect(button_foreground_, SIGNAL(clicked()), this, SLOT(clicked_foreground()));
	connect(button_highlight_, SIGNAL(clicked()), this, SLOT(clicked_highlight()));
	connect(button_tag1_, SIGNAL(clicked()), this, SLOT(clicked_tag1()));
	connect(button_tag2_, SIGNAL(clicked()), this, SLOT(clicked_tag2()));
	connect(button_tag3_, SIGNAL(clicked()), this, SLOT(clicked_tag3()));
	connect(button_tag4_, SIGNAL(clicked()), this, SLOT(clicked_tag4()));
//	setSizeGripEnabled(true);
}

void ColorDialog::clicked_reset() {
	c_background_ = DEFAULT_BACKGROUND_COLOR;
	label_background_->setPaletteForegroundColor(c_background_);
	c_foreground_ = DEFAULT_FOREGROUND_COLOR;
	label_foreground_->setPaletteForegroundColor(c_foreground_);
	c_highlight_ = DEFAULT_HIGHLIGHT_COLOR;
	label_highlight_->setPaletteForegroundColor(c_highlight_);
	c_tag1_ = DEFAULT_TAG1_COLOR;
	label_tag1_->setPaletteForegroundColor(c_tag1_);
	c_tag2_ = DEFAULT_TAG2_COLOR;
	label_tag2_->setPaletteForegroundColor(c_tag2_);
	c_tag3_ = DEFAULT_TAG3_COLOR;
	label_tag3_->setPaletteForegroundColor(c_tag3_);
	c_tag4_ = DEFAULT_TAG4_COLOR;
	label_tag4_->setPaletteForegroundColor(c_tag4_);
}

void ColorDialog::clicked_background() {
	c_background_ = QColorDialog::getColor(c_background_);
	label_background_->setPaletteForegroundColor(c_background_);
}

void ColorDialog::clicked_foreground() {
	c_foreground_ = QColorDialog::getColor(c_foreground_);
	label_foreground_->setPaletteForegroundColor(c_foreground_);
}

void ColorDialog::clicked_highlight() {
	c_highlight_ = QColorDialog::getColor(c_highlight_);
	label_highlight_->setPaletteForegroundColor(c_highlight_);
}

void ColorDialog::clicked_tag1() {
	c_tag1_ = QColorDialog::getColor(c_tag1_);
	label_tag1_->setPaletteForegroundColor(c_tag1_);
}

void ColorDialog::clicked_tag2() {
	c_tag2_ = QColorDialog::getColor(c_tag2_);
	label_tag2_->setPaletteForegroundColor(c_tag2_);
}

void ColorDialog::clicked_tag3() {
	c_tag3_ = QColorDialog::getColor(c_tag3_);
	label_tag3_->setPaletteForegroundColor(c_tag3_);
}

void ColorDialog::clicked_tag4() {
	c_tag4_ = QColorDialog::getColor(c_tag4_);
	label_tag4_->setPaletteForegroundColor(c_tag4_);
}

