//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#include "ctags.h"
#include <qapplication.h>
#include <time.h>



CtagsThread::CtagsThread(QObject *parent): 
#ifdef USING_QT4
	QThread(parent),
#endif
#ifdef USING_QT3
	QThread(),
#endif
	doit_(0), done_(0), parent_(parent), pdb_(0) {
	

	msgout_ = "";
	outbuffer_ = "";
	outfile_ = "";
	active_ = false;

	
	LOG("ctags thread created");
}

CtagsThread::~CtagsThread() {
	if (pdb_)
		delete pdb_;
}

void CtagsThread::sendAlldone() {
#ifdef USING_QT4
	emit alldone();
#else
	QApplication::postEvent(parent_, new QCustomEvent(65432));
#endif
}

void CtagsThread::run() {
	LOG("ctags thread run");
	srand(time(NULL));

	loadTagFile();


	bool nodelta = true;
	while(1) {
		// reload tags command from qwerty;
		// loadTagFile will set the active_ flag
		if (msgin_ == "reload tags") {
			while (!vtag_.empty())
				vtag_.erase(vtag_.begin());
			
			loadTagFile();
			msgin_ = "";
			done_ = doit_;
		}
		
		// process ctag input if active_, there is no delta,
		// and there is something to do
		else if (active_ && nodelta && doit_ > done_) {
			LOG("ctags delta %d", doit_ - done_);
			// work on copyes of input data!
			// real input data can change due to the user pressing keys
			// during our ctags processing below!
			int inprogress = doit_;
			QString msgin_todo = msgin_;
			msgout_ = "";
			int cnt = 0;
			Tag unique;

			// try to find the keyword located in msgin
			LOG("ctag check word %s", (const char *) msgin_);
			const char *dbline = pdb_->findString((const char *) msgin_todo, true);
			if (dbline != 0) {
				int dbindex;
				char *ptr = (char *) dbline;
				ptr += strlen((const char *) msgin_todo);
				if (*ptr == ',') {
					sscanf(ptr + 1, "%d", &dbindex);
//					printf("dbindex %d, %s, %s\n", dbindex, (const char *) vtag_[dbindex].keyword_, (const char *) vtag_[dbindex].line_number_);
					for (unsigned i = (unsigned) dbindex; i < vtag_.size(); i++) {
						if (vtag_[i].keyword_ == msgin_todo) {
							unique = vtag_[i];
//							t.log();
							cnt++;
							// found entry, copy it in the result list
							msgout_ += vtag_[i].file_ + "::" + vtag_[i].line_number_ +
								   " " + vtag_[i].line_text_ + "\n";
						}
						else
							break;
					}
					
				}
				else
					printf("big problem %s\n", dbline);
			}
			
			if (cnt == 1) {
//				unique.log();
				msgout_ = unique.file_ + "::" + unique.line_number_ + ":" + " single ctag definition, see below...\n";
				
				if (last_file1_ == unique.file_) {
					msgout_ += last_text1_;
				}
				else if (last_file2_ == unique.file_) {
					msgout_ += last_text2_;
				}
				else if (last_file3_ == unique.file_) {
					msgout_ += last_text3_;
				}
				else if (last_file4_ == unique.file_) {
					msgout_ += last_text4_;
				}
				else {
					LOG("ctag load  file %s", (const char *) unique.file_);
					char *text = UTILS::read_file_malloc((const char *) unique.file_);
					if (text != 0) {
						msgout_ += text;
						int r = rand() % 4;
						if (r == 0) {
							last_file1_ = unique.file_;
							last_text1_ = text;
						}
						else if (r == 1) {
							last_file2_ = unique.file_;
							last_text2_ = text;
						}
						else if (r == 2) {
							last_file3_ = unique.file_;
							last_text3_ = text;
						}
						else {
							last_file4_ = unique.file_;
							last_text4_ = text;
						}
					
						delete[] text;
					}
				}
			}
			
			if (cnt != 0)
				sendAlldone();
			sleep(1); // some delay for qwerty to consume the information
			done_ = inprogress;
		}
		
		// process nodelta:
		//  - set nodelta if during the last one second sleep here
		//    was no key pressed in qwerty
		//  - reset nodelta otherwise
		// this functionality is intorduced to make sure we do not
		// try to process ctags while the user is moving the cursor
		else {
			
			int tmp = doit_;
			sleep(1);
			if (doit_ == tmp)
				nodelta = true;
			else
				nodelta = false;
		}
	}
}

void CtagsThread::startProcessing(QString ctagword) {
//	if (!active_)
//		return;

	if (msgin_ != ctagword) {
		msgin_ = ctagword;
		LOG("start processing %s", (const char *) ctagword);
//printf("process @%s@\n", (const char *) ctagword);
		doit_++;
	}
}

#define MAXS 1000

bool CtagsThread::readOneLine(FILE *fp, char *buf, int size) {
	int i = 0;
	bool retval = false;
	while (1) {
		*buf = getc(fp);
		if (i >= size || *buf == EOF) {
			retval = false;
			break;
		}
		if (*buf == '\n' || *buf == '\r') {
			retval = true;
			break;
		}
		buf++;
		i++;
	}
	*buf = '\0';
	return retval;
}


void CtagsThread::loadTagFile(FILE *fp) {
	char buf[MAXS + 1];
	
	int state = 0;
	char filename[MAXS + 1];
	QString qfilename;
	char text[MAXS + 1];
	char keyword[MAXS + 1];
	char line[MAXS + 1];
	while (readOneLine(fp, buf, MAXS)) {
		if (*buf == 0x0c) {
			state = 1;
			continue;
		}
		
		switch (state) {
			case 0: {
				snprintf(filename, MAXS, "Error 1: bad input tag file\n%s\n", buf);
				msgout_ = filename;
				sendAlldone();
				active_ = false;
				return;
			}
			case 1: {// we should have a file name here
				char *ptr = buf;
				while (*ptr != ',' && *ptr != '\0') {
//printf("%x, ", *ptr);
					ptr++;
				}
//printf("%x, ", *ptr);
				if (*ptr == '\0') {
					snprintf(filename, MAXS, "Error 2: bad input tag file\n%s\n", buf);
					msgout_ = filename;
					sendAlldone();
					active_ = false;
					return;
				}
				*ptr = '\0';
				strcpy(filename, buf);
				qfilename = QString(Global::unixHomeConvert(QString(filename)));
//				LOG("file %s ### %s", filename, (const char *) qfilename); 
				state = 2;
			}
			break;
			case 2: {// we should have a tag here
				char *ptr = buf;
				while (*ptr != 0x7f && *ptr != '\0') {
//printf("%x, ", *ptr);
					ptr++;
				}
//printf("%x, ", *ptr);
				if (*ptr == '\0') {
					snprintf(filename, MAXS, "Error 3: bad input tag file\n%s\n", buf);
					msgout_ = filename;
					sendAlldone();
					active_ = false;
					return;
				}
				*ptr = '\0';
				ptr++;
				strcpy(text, buf);

				char *ptr1 = ptr;
				while (*ptr != 0x01 && *ptr != '\0')
					ptr++;
				if (*ptr == '\0') {
					snprintf(filename, MAXS, "Error 4: bad input tag file\n%s\n", buf);
					msgout_ = filename;
					sendAlldone();
					active_ = false;
					return;
				}
				*ptr = '\0';
				ptr++;
				strcpy(keyword, ptr1);

				ptr1 = ptr;
				while (*ptr != ',' && *ptr != '\0')
					ptr++;
				if (*ptr == '\0') {
					snprintf(filename, MAXS, "Error 5: bad input tag file\n%s\n", buf);
					msgout_ = filename;
					sendAlldone();
					active_ = false;
					return;
				}
				*ptr = '\0';
				ptr++;
				strcpy(line, ptr1);
				ltag_.push_back(Tag(qfilename, QString(keyword), QString(text), QString(line)));
//				printf("file: %s\nkeyword: %s\ntext: %s\nline: %s\n",
//					filename, keyword, text, line);
			}
			break;
			default:
				break;
		}
	}
}



void CtagsThread::loadTagFile() {
	FILE *fp;
	
	if (pdb_)
		delete pdb_;
	pdb_ = new DbCtags;

	// clear file buffers	
	clearFileBuffers();

	// open local tag file
	fp = fopen("TAGS", "r");
	active_ = false;
	if (fp == 0) {
		msgout_ = "user tag file not found or empty";
		sendAlldone();
	}
	else {
		loadTagFile(fp);
		fclose(fp);
		msgout_ = "tag file readed";
		sendAlldone();
	}		
	
	// open glibc tag file
	if (Global::instance()->load_glibc_tags) {
		fp = fopen(DF4, "r");
		
		if (fp == 0) {
			LOG("libc tag file not found or empty");
			msgout_ = "libc tag file not found or empty";
			sendAlldone();
		}
		else {
			loadTagFile(fp);	
			fclose(fp);
			msgout_ = "tag file readed";
			sendAlldone();
		}
	}
	
	// open posixh tag file
	if (Global::instance()->load_posixh_tags) {
		fp = fopen(DF8, "r");
		
		if (fp == 0) {
			LOG("posix headers tag file not found or empty");
			msgout_ = "posixh header tag file not found or empty";
			sendAlldone();
		}
		else {
			loadTagFile(fp);	
			fclose(fp);
			msgout_ = "tag file readed";
			sendAlldone();
		}
	}
	
	// open latex tag file
	if (Global::instance()->load_latex_tags) {
		fp = fopen(DF5, "r");
		
		if (fp == 0) {
			LOG("latex tag file not found or empty");
			msgout_ = "latex tag file not found or empty";
			sendAlldone();
		}
		else {
			loadTagFile(fp);	
			fclose(fp);
			msgout_ = "tag file readed";
			sendAlldone();
		}
	}
	
	if (ltag_.empty()) {
		msgout_ = "ctags inactive";
		sendAlldone();
		active_ = false;
		return;
	}
	
	// sort the list
	ltag_.sort();
	msgout_ = "tag file sorted";
	sendAlldone();
	
	// populate the hash table (pdb_) and the vector table (vtag_);	
	list<Tag>::iterator it;
	int i = 0;
	for (it = ltag_.begin(); it != ltag_.end(); ++it, i++) {
		Tag t = *it;
		if (pdb_->findString((const char *) t.keyword_, true) == 0) {
			QString tmp = t.keyword_ + "," + QString("%1").arg(i);
			pdb_->setString((const char *) tmp, true);
		}
		vtag_.push_back(t);
	}
	
	// release the list
	while (!ltag_.empty())
		ltag_.erase(ltag_.begin());

	
	msgout_ = "ctags active";
	sendAlldone();
	active_ = true;
}

QString CtagsThread::roDebug() {
	QString retval = "ctags: ";
	retval += (active_)? "active": "inactive";
	retval += QString(", doit/done %1/").arg(doit_);
	retval += QString("%1\n").arg(done_);
	retval += "1: " + last_file1_ + "\n";
	retval += "2: " + last_file2_ + "\n";
	retval += "3: " + last_file3_ + "\n";
	retval += "4: " + last_file4_ + "\n";
	return retval;
}


//#endif // USING_QT4

