//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#ifndef DEBUG_H
#define DEBUG_H

//***********************************************************
// Global debug configuration
//***********************************************************
#ifndef NO_DEBUG // passed on the compiler command line

// comment out this line in the final code in order to
// disable C-style assertions; use ASSERT macro in your code
	#define ENABLE_ASSERT

// comment out this line in order to disable Profiler
// in the final code
//#define ENABLE_PROFILER

// comment out this line to disable log
	#define ENABLE_LOG
#endif //NO_DEBUG

//***********************************************************
// Do not modify below this line
//***********************************************************


//***********************************************************
// Assertions
//***********************************************************
#ifdef ENABLE_ASSERT
	#ifndef ASSERT
		#include <assert.h>
		#define ASSERT(X) { if (!(X)) {LOG("assert failed %s:%d", __FILE__, __LINE__); fflush(0); assert(X);}}
	#endif
#else                
	#ifndef ASSERT
		#define ASSERT(X)
	#endif
#endif

//****************************************************************
// Normal and error log - use on type of log
//****************************************************************
#ifndef ENABLE_LOG
	#define Log LogNothing
#else

//	#define Log LogPrintf
	#define Log LogFile
#endif
#include "log.h"



//****************************************************************
// Profiler handling
//****************************************************************
#ifdef ENABLE_PROFILER
	#include "profiler.h"
#else
class Profiler {
public:
	static void print() {;}
	static void release() {;}
};
	#define PROFILE(X)
#endif

//****************************************************************
// Print debug configuration
//****************************************************************

class DebugConfig {
public:
	static void print() {
#ifdef ENABLE_ASSERT
		LOG("ASSERT enabled");
#endif

#ifdef ENABLE_PROFILER
		LOG("Profiler enabled");
#endif
	}
};


#endif // DEBUG_DEBUG_H

