//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#include "dialogs.h"
#include <qmessagebox.h>
#include <qgroupbox.h>

#ifdef USING_QT4
#include <Q3VBoxLayout>
#include <q3hbox.h>
#include <q3accel.h>
#include <QLabel>
#endif
#ifdef USING_QT3
#include <qlayout.h>
#include <qhbox.h>
#include <qaccel.h>
#include <qlabel.h>
#endif

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qcombobox.h>

//*****************************************************************************/
// line dialog
//*****************************************************************************/
LineDialog::LineDialog(): QDialog(0, "Enter line number", true) {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	Q3HBox *hbox = new Q3HBox(this);
	hbox->setSpacing(10);
	label_ = new QLabel("Line: ", hbox);
	text_ = new QLineEdit(hbox);

	// accept/reject
	new QWidget(this);
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	(new QWidget(hbox2))->setMinimumWidth(80);
	(new QWidget(hbox2))->setMinimumWidth(80);
	button_ok_ = new QPushButton("Ok", hbox2);
	button_cancel_ = new QPushButton("Cancel", hbox2);

	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
}

int LineDialog::getLine() {
	int retval;

	if (text_->text().isEmpty() == true)
		retval = -1;
	else {
		QString str = text_->text();
		bool ok;
		retval = str.toInt(&ok);
		if (!ok || retval <= 0) {
			retval = -1;
			QMessageBox::warning(0,"Error", "Not a valid line number");
		}
	}
	return retval;
}

//*****************************************************************************/
// bookmark dialog
//*****************************************************************************/
BookmarkDialog::BookmarkDialog(): QDialog(0, "Enter bookmark number", true) {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	Q3HBox *hbox = new Q3HBox(this);
	hbox->setSpacing(10);
	label_ = new QLabel("Bookmark: ", hbox);
	text_ = new QLineEdit(hbox);

	// accept/reject
	new QWidget(this);
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	(new QWidget(hbox2))->setMinimumWidth(80);
	(new QWidget(hbox2))->setMinimumWidth(80);
	button_ok_ = new QPushButton("Ok", hbox2);
	button_cancel_ = new QPushButton("Cancel", hbox2);

	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
}

int BookmarkDialog::getBookmark() {
	int retval;

	if (text_->text().isEmpty() == true)
		retval = -1;
	else {
		QString str = text_->text();
		bool ok;
		retval = str.toInt(&ok);
		if (!ok || retval < 0 || retval >= MAXBOOKMARKS) {
			retval = -1;
			QMessageBox::warning(0,"Error", "Not a valid bookmark number");
		}
	}
	return retval;
}

//*****************************************************************************/
// bookmark dialog
//*****************************************************************************/
HexDialog::HexDialog(): QDialog(0, "Enter Unicode Char", true) {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	Q3HBox *hbox = new Q3HBox(this);
	hbox->setSpacing(10);
	label_ = new QLabel("Unicode character code: ", hbox);
	text_ = new QLineEdit(hbox);

	// accept/reject
	new QWidget(this);
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	(new QWidget(hbox2))->setMinimumWidth(80);
	(new QWidget(hbox2))->setMinimumWidth(80);
	button_ok_ = new QPushButton("Ok", hbox2);
	button_cancel_ = new QPushButton("Cancel", hbox2);

	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
}

int HexDialog::getHex() {
	int retval;

	if (text_->text().isEmpty() == true)
		retval = -1;
	else {
		QString str = text_->text();
		bool ok;
		retval = str.toInt(&ok, 16);
		if (!ok) {
			retval = -1;
			QMessageBox::warning(0,"Error", "Not a valid byte code");
		}
	}
	return retval;
}


//*****************************************************************************/
// www dialog
//*****************************************************************************/
WwwDialog::WwwDialog(): QDialog(0, "www location", true) {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label_ = new QLabel("Please enter the URL of a file you want to download", hbox1);

	Q3HBox *hbox = new Q3HBox(this);
	hbox->setSpacing(10);
	text_ = new QLineEdit(hbox);

	Q3HBox *hbox2 = new Q3HBox(this);
	(new QWidget(hbox2))->setMinimumWidth(80);
	(new QWidget(hbox2))->setMinimumWidth(80);
	button_ = new QPushButton("Ok", hbox2);

	connect(button_, SIGNAL(clicked()), SLOT(accept()));
}

QString WwwDialog::getWord() {
	QString retval;

	if (text_->text().isEmpty() == true)
		retval = "";
	else {
		retval = text_->text();
	}
	return retval;
}

//*****************************************************************************/
// status dialog
//*****************************************************************************/
StatusDialog::StatusDialog(QFont &fnt, char *msg): QDialog(0, "File status", true) {
	ASSERT(msg != 0);
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	Q3HBox *hbox = new Q3HBox(this);

	QString qstr = QString::fromUtf8(msg);

	label_ = new QLabel(qstr, hbox);
	label_->setFont(fnt);

	new QWidget(this);
	Q3HBox *hbox1 = new Q3HBox(this);
	new QWidget(hbox1);
	new QWidget(hbox1);
	button_ = new QPushButton("Ok", hbox1);
	new QWidget(hbox1);

	connect(button_, SIGNAL(clicked()), SLOT(accept()));
}



//*****************************************************************************/
// basic unicode dialog
//*****************************************************************************/
BasicUnicodeDialog::BasicUnicodeDialog(QString title): QDialog(0, title, true) {
	text_ = 0;

	Q3Accel *a = new Q3Accel(this);		  // create accels for myWindow
	a->connectItem(a->insertItem(Qt::ALT+Qt::Key_A), this, SLOT(unicode_a()));
	a->connectItem(a->insertItem(Qt::SHIFT+Qt::ALT+Qt::Key_A), this, SLOT(unicode_A()));
	a->connectItem(a->insertItem(Qt::ALT+Qt::Key_S), this, SLOT(unicode_s()));
	a->connectItem(a->insertItem(Qt::SHIFT+Qt::ALT+Qt::Key_S), this, SLOT(unicode_S()));
	a->connectItem(a->insertItem(Qt::ALT+Qt::Key_Q), this, SLOT(unicode_q()));
	a->connectItem(a->insertItem(Qt::SHIFT+Qt::ALT+Qt::Key_Q), this, SLOT(unicode_Q()));
	a->connectItem(a->insertItem(Qt::ALT+Qt::Key_T), this, SLOT(unicode_t()));
	a->connectItem(a->insertItem(Qt::SHIFT+Qt::ALT+Qt::Key_T), this, SLOT(unicode_T()));
	a->connectItem(a->insertItem(Qt::ALT+Qt::Key_I), this, SLOT(unicode_i()));
	a->connectItem(a->insertItem(Qt::SHIFT+Qt::ALT+Qt::Key_I), this, SLOT(unicode_I()));
}

QString BasicUnicodeDialog::getWord() {
	if (text_ == 0)
		return "";

	QString retval;

	if (text_->text().isEmpty() == true)
		retval = "";
	else {
		retval = text_->text();
	}
	return retval;
}


void BasicUnicodeDialog::unicode_a() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("ă");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_A() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("Ă");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_s() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("ş");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_S() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("Ş");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_q() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("â");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_Q() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("Â");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_t() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("ţ");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_T() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("Ţ");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_i() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("î");
	text_->insert(str);
	text_->cursorForward(false);
}

void BasicUnicodeDialog::unicode_I() {
	if (!hasFocus())
		return;
	if (text_ == 0)
		return;
	QString str = QString::fromUtf8("Î");
	text_->insert(str);
	text_->cursorForward(false);
}



//*****************************************************************************/
// basic unicode combo dialog
//*****************************************************************************/
BasicUnicodeComboDialog::BasicUnicodeComboDialog(QString title): BasicUnicodeDialog(title) {
	combotext_ = 0;

}

QString BasicUnicodeComboDialog::getWord() {
	if (combotext_ == 0)
		return "";

	QString retval;

	if (combotext_->currentText().isEmpty() == true)
		retval = "";
	else {
		retval = combotext_->currentText();
	}
	return retval;
}







//*****************************************************************************/
// search dialog
//*****************************************************************************/
SearchDialog::SearchDialog(): BasicUnicodeComboDialog("Enter search word") {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	// target word
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label_ = new QLabel("Target: ", hbox1);
	text_ = new QLineEdit(hbox1);
	combotext_ = new QComboBox(TRUE, hbox1);
	combotext_->setLineEdit(text_);

	Q3HBox *hbox3 = new Q3HBox(this);
	ignore_case_ = new QCheckBox("Ignore case", hbox3);

	Q3HBox *hbox4 = new Q3HBox(this);
	whole_ = new QCheckBox("Whole words", hbox4);

	Q3HBox *hbox5 = new Q3HBox(this);
	backwards_ = new QCheckBox("Search Backwards", hbox5);

	// accept/reject
	new QWidget(this);
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	(new QWidget(hbox2))->setMinimumWidth(80);
	(new QWidget(hbox2))->setMinimumWidth(80);
//	new QWidget(hbox2);
	button_ok_ = new QPushButton("Ok", hbox2);
	button_cancel_ = new QPushButton("Cancel", hbox2);


	connect(button_ok_, SIGNAL(clicked()), SLOT(clicked_ok()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
}

bool SearchDialog::getIgnoreCase() {
	return ignore_case_->isChecked();
}

bool SearchDialog::getWhole() {
	return whole_->isChecked();
}

bool SearchDialog::getBackwards() {
	return backwards_->isChecked();
}


void SearchDialog::clicked_ok() {
	QString word = getWord();
	// check if the word is not already in the list
	bool found = false;
	for (int i = 0; i < combotext_->count(); i++) {
		if (combotext_->text(i) == word) {
			found = true;
			break;		
		}
	}
	if (!found)
		combotext_->insertItem(word, 0);
	
	accept();
}

//*****************************************************************************/
// grep dialog
//*****************************************************************************/
GrepDialog::GrepDialog(): BasicUnicodeComboDialog("Enter search word") {
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	// target word
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label_ = new QLabel("Target: ", hbox1);
	text_ = new QLineEdit(hbox1);
	combotext_ = new QComboBox(TRUE, hbox1);
	combotext_->setLineEdit(text_);


	Q3HBox *hbox11 = new Q3HBox(this);
	hbox11->setSpacing(10);
	label2_ = new QLabel("Starting directory: ", hbox11);
	sdirectory_ = new QLineEdit(hbox11);
	button_dir_ = new QPushButton("...", hbox11);
	sdirectory_->setText("./");

	Q3HBox *hbox12 = new Q3HBox(this);
	hbox12->setSpacing(10);
	label3_ = new QLabel("File pattern: ", hbox12);
	fpattern_ = new QLineEdit(hbox12);
	fpattern_->setText("*.*");

	Q3HBox *hbox13 = new Q3HBox(this);
	hbox13->setSpacing(10);
	label4_ = new QLabel("Exclude directories: ", hbox13);
	edirectory_ = new QLineEdit(hbox13);
	edirectory_->setText("");

	Q3HBox *hbox3 = new Q3HBox(this);
	ignore_case_ = new QCheckBox("Ignore case", hbox3);

	Q3HBox *hbox4 = new Q3HBox(this);
	whole_ = new QCheckBox("Whole words", hbox4);

	Q3HBox *hbox5 = new Q3HBox(this);
	recursive_ = new QCheckBox("Recursive", hbox5);

	// accept/reject
	new QWidget(this);
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	(new QWidget(hbox2))->setMinimumWidth(80);
	(new QWidget(hbox2))->setMinimumWidth(80);
	button_ok_ = new QPushButton("Ok", hbox2);
	button_cancel_ = new QPushButton("Cancel", hbox2);


//	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_ok_, SIGNAL(clicked()), SLOT(clicked_ok()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
	connect(button_dir_, SIGNAL(clicked()), this, SLOT(clicked_dir()));
}

bool GrepDialog::getIgnoreCase() {
	return ignore_case_->isChecked();
}

bool GrepDialog::getWhole() {
	return whole_->isChecked();
}

bool GrepDialog::getRecursive() {
	return recursive_->isChecked();
}

QString GrepDialog::getSdirectory() {
	return sdirectory_->text();
}

QString GrepDialog::getFpattern() {
	return fpattern_->text();
}

QString GrepDialog::getEdirectory() {
	return edirectory_->text();
}

void GrepDialog::clicked_dir() {
	QString s = QFileDialog::getExistingDirectory("./", this, "get existing directory", "Choose a directory", TRUE );
	if (s.isNull() || s.isEmpty()) {
		return;
	}
	sdirectory_->setText(s);
}

void GrepDialog::clicked_ok() {
	QString word = getWord();
	// check if the word is not already in the list
	bool found = false;
	for (int i = 0; i < combotext_->count(); i++) {
		if (combotext_->text(i) == word) {
			found = true;
			break;		
		}
	}
	if (!found)
		combotext_->insertItem(word, 0);
	
	accept();
}

//*****************************************************************************/
// replace dialog
//*****************************************************************************/
ReplaceDialog::ReplaceDialog(QString tofind, QString replacement):
	BasicUnicodeComboDialog("Replace") {
	
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);

	// target word
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label1_ = new QLabel("Target: ", hbox1);
	text1_ = new QLineEdit(hbox1);
	text1_->setText(tofind);
	combotext1_ = new QComboBox(TRUE, hbox1);
	combotext1_->setLineEdit(text1_);

	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	label2_ = new QLabel("Replace with: ", hbox2);
	text2_ = new QLineEdit(hbox2);
	text2_->setText(replacement);
	combotext2_ = new QComboBox(TRUE, hbox2);
	combotext2_->setLineEdit(text2_);

	// accept/reject
	new QWidget(this);
	Q3HBox *hbox3 = new Q3HBox(this);
	hbox3->setSpacing(10);
	button_findnext_ = new QPushButton("Find Next", hbox3);
	button_replace_ = new QPushButton("Replace", hbox3);
	button_replaceall_ = new QPushButton("Replace All", hbox3);
	new QWidget(hbox3);
	button_cancel_ = new QPushButton("Cancel", hbox3);


	connect(button_findnext_, SIGNAL(clicked()), this, SLOT(clicked_findnext()));
	connect(button_replace_, SIGNAL(clicked()), this, SLOT(clicked_replace()));
	connect(button_replaceall_, SIGNAL(clicked()), this, SLOT(clicked_replaceall()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(clicked_cancel()));
//	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));

	text_ = 0;
	combotext_ = 0;
	action_ = ACTION_NONE;
}

bool ReplaceDialog::hasFocus() {
	if (text1_->hasFocus()) {
		text_ = text1_;
		combotext_ = combotext1_;
	}
	else if (text2_->hasFocus()) {
		text_ = text2_;
		combotext_ = combotext2_;
	}
	else {
		text_ = 0;
		combotext_ = 0;
	}

	return true;	
}

void ReplaceDialog::printAction() {
	if (action_ == ACTION_FINDNEXT)
		LOG("ReplaceDialog::ACTION_FINDNEXT");
	else if (action_ == ACTION_REPLACE)
		LOG("ReplaceDialog::ACTION_REPLACE");
	else if (action_ == ACTION_REPLACEALL)
		LOG("ReplaceDialog::ACTION_REPLACEALL");
	else if (action_ == ACTION_CANCEL)
		LOG("ReplaceDialog::ACTION_CANCEL");
	else
		LOG("ReplaceDialog::ACTION_NONE");
	
}
void ReplaceDialog::clicked_findnext() {
	action_ = ACTION_FINDNEXT;
}

void ReplaceDialog::clicked_replace() {
	action_ = ACTION_REPLACE;
}

void ReplaceDialog::clicked_replaceall() {
	action_ = ACTION_REPLACEALL;
}

void ReplaceDialog::clicked_cancel() {
	action_ = ACTION_CANCEL;
}

void ReplaceDialog::getOut() {

	QString str = getWordToFind();
	// check if the word is not already in the list
	bool found = false;
	for (int i = 0; i < combotext1_->count(); i++) {
		if (combotext1_->text(i) == str) {
			found = true;
			break;		
		}
	}
	if (!found) 
		combotext1_->insertItem(str, 0);

	str = getReplacement();
	found = false;
	for (int i = 0; i < combotext2_->count(); i++) {
		if (combotext2_->text(i) == str) {
			found = true;
			break;		
		}
	}
	if (!found)
		combotext2_->insertItem(str, 0);


	text_ = 0;
	combotext_ = 0;
	action_ = ACTION_NONE;
	accept();
}

void ReplaceDialog::closeEvent(QCloseEvent *evt) {
	(void) evt;
	action_ = ACTION_CANCEL;
}

QString ReplaceDialog::getWordToFind() {
	if (text1_ == 0)
		return "";

	QString retval;

	if (text1_->text().isEmpty() == true)
		retval = "";
	else {
		retval = text1_->text();
	}



	return retval;
}

QString ReplaceDialog::getReplacement() {
	if (text2_ == 0)
		return "";

	QString retval;

	if (text2_->text().isEmpty() == true)
		retval = "";
	else {
		retval = text2_->text();
	}

	return retval;
}
