//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#ifndef DIALOGS_H
#define DIALOGS_H

#include "global.h"
#include <qdialog.h>

// forward declarations
class QLabel;
class QPushButton;
class QLineEdit;
class QCheckBox;
class QComboBox;

class StatusDialog: public QDialog {
public:
	StatusDialog(QFont &fnt, char *msg);
private:
	QLabel *label_;
	QPushButton *button_;
};



class WwwDialog: public QDialog {
public:
	WwwDialog();
	QString getWord();
private:
	QLabel *label_;
	QLineEdit *text_;
	QPushButton *button_;
};


class LineDialog: public QDialog {
public:
	LineDialog();
	int getLine();
private:
	QLabel *label_;
	QLineEdit *text_;
	QPushButton *button_ok_;
	QPushButton *button_cancel_;
};

class BookmarkDialog: public QDialog {
public:
	BookmarkDialog();
	int getBookmark();
private:
	QLabel *label_;
	QLineEdit *text_;
	QPushButton *button_ok_;
	QPushButton *button_cancel_;
};

class HexDialog: public QDialog {
public:
	HexDialog();
	int getHex();
private:
	QLabel *label_;
	QLineEdit *text_;
	QPushButton *button_ok_;
	QPushButton *button_cancel_;
};

class BasicUnicodeDialog: public QDialog {
	Q_OBJECT

public:
	BasicUnicodeDialog(QString title);
	virtual QString getWord();

private slots:
	void unicode_a();
	void unicode_A();
	void unicode_s();
	void unicode_S();
	void unicode_q();
	void unicode_Q();
	void unicode_t();
	void unicode_T();
	void unicode_i();
	void unicode_I();
protected:
	virtual bool hasFocus() = 0;
protected:
	QLineEdit *text_;
};

class BasicUnicodeComboDialog: public BasicUnicodeDialog {
	Q_OBJECT
public:
	BasicUnicodeComboDialog(QString title);
	QString getWord();

protected:
	QComboBox *combotext_;
};




class SearchDialog: public BasicUnicodeComboDialog {
	Q_OBJECT
public:
	SearchDialog();
	bool getIgnoreCase();
	bool getWhole();
	bool getBackwards();

private slots:
	void clicked_ok();
private:
	bool hasFocus() {
		return true;
	}
private:
	QLabel *label_;
	QPushButton *button_ok_;
	QPushButton *button_cancel_;
	QCheckBox *ignore_case_;
	QCheckBox *whole_;
	QCheckBox *backwards_;
};

class GrepDialog: public BasicUnicodeComboDialog {
	Q_OBJECT
public:
	GrepDialog();
	bool getIgnoreCase();
	bool getWhole();
	bool getRecursive();
	QString getSdirectory();
	QString getFpattern();
	QString getEdirectory();

private slots:
	void clicked_dir();
	void clicked_ok();


private:
	bool hasFocus() {
		return true;
	}
private:
	QLabel *label_;
	QLabel *label2_;
	QLabel *label3_;
	QLabel *label4_;
	QPushButton *button_ok_;
	QPushButton *button_cancel_;
	QPushButton *button_dir_;
	QCheckBox *ignore_case_;
	QCheckBox *whole_;
	QCheckBox *recursive_;
	QLineEdit *sdirectory_;
	QLineEdit *fpattern_;
	QLineEdit *edirectory_;
};

class ReplaceDialog: public BasicUnicodeComboDialog {
	Q_OBJECT

public:
	typedef enum {
		ACTION_NONE = 0,
		ACTION_FINDNEXT,
		ACTION_REPLACE,
		ACTION_REPLACEALL,
		ACTION_CANCEL
	} ReplaceAction;

	ReplaceDialog(QString tofind, QString replacement);

	// action functions
	volatile ReplaceAction getAction() {
		return action_;
	}
	void reset() {
		action_ = ACTION_NONE;
	}
	void printAction();
	
	// retrieve dialog data
	QString getWordToFind();
	QString getReplacement();
	void getOut();		


private slots:
	void clicked_findnext();
	void clicked_replace();
	void clicked_replaceall();
	void clicked_cancel();

private:
	bool hasFocus();
protected:
	virtual void closeEvent(QCloseEvent *);
private:
	QLabel *label1_;
	QLabel *label2_;
	QLineEdit *text1_;
	QLineEdit *text2_;
	QComboBox *combotext1_;
	QComboBox *combotext2_;
	QPushButton *button_findnext_;
	QPushButton *button_replace_;
	QPushButton *button_replaceall_;
	QPushButton *button_cancel_;
	volatile ReplaceAction action_;
};


#endif // DIALOGS_H


