//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#include "diffdialog.h"
#include "busycursor.h"
#include <qmessagebox.h>
#include <qgroupbox.h>

#ifdef USING_QT3
#include <qfiledialog.h>
#endif
#ifdef USING_QT4
#include <q3filedialog.h>
#endif


#ifdef USING_QT3
#include <qlayout.h>
#include <qlabel.h>
#include <qhbox.h>
#include <qaccel.h>
#include <qmultilineedit.h>
#endif
#ifdef USING_QT4
#include <Q3VBoxLayout>
#include <QLabel>
#include <q3hbox.h>
#include <q3accel.h>
#include <q3multilineedit.h>
#endif

#include <qlineedit.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qtextcodec.h>

extern QString utf82latex(QString txt);

//*****************************************************************************/
// diff dialog
//*****************************************************************************/
DiffDialog::DiffDialog(): QDialog(0, "diff", true) {

	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);
	
	
	// file1
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	label1_ = new QLabel("File 1: ", hbox1);
	file1_ = new QLineEdit(hbox1);
	button_browse1_ = new QPushButton("browse", hbox1);

	// file2
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	label2_ = new QLabel("File 2: ", hbox2);
	file2_ = new QLineEdit(hbox2);
	button_browse2_ = new QPushButton("browse", hbox2);

	// accept/reject
	new QWidget(this);
	Q3HBox *hbox3 = new Q3HBox(this);
	hbox3->setSpacing(10);
	// some space
	(new QWidget(hbox3))->setMinimumWidth(80);
	(new QWidget(hbox3))->setMinimumWidth(80);
	button_ok_ = new QPushButton("Ok", hbox3);
	button_cancel_ = new QPushButton("Cancel", hbox3);

	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
	connect(button_browse1_, SIGNAL(clicked()), this, SLOT(clicked_browse1()));
	connect(button_browse2_, SIGNAL(clicked()), this, SLOT(clicked_browse2()));
}

QString DiffDialog::getFile1() {
	if (file1_ == 0)
		return "";

	QString retval;

	if (file1_->text().isEmpty() == true)
		retval = "";
	else {
		retval = file1_->text();
	}
	return retval;
}

QString DiffDialog::getFile2() {
	if (file2_ == 0)
		return "";

	QString retval;

	if (file2_->text().isEmpty() == true)
		retval = "";
	else {
		retval = file2_->text();
	}
	return retval;
}

void DiffDialog::clicked_browse1() {
	QString file = Q3FileDialog::getOpenFileName(Global::instance()->diff_dir1, QString::null, this );
	if (!file.isNull()) {
		QFileInfo info(file);
		Global::instance()->diff_dir1 = info.absFilePath();
	}
	file1_->setText(file);
}

void DiffDialog::clicked_browse2() {
	QString file = Q3FileDialog::getOpenFileName(Global::instance()->diff_dir2, QString::null, this );
	if (!file.isNull()) {
		QFileInfo info(file);
		Global::instance()->diff_dir2 = info.absFilePath();
	}
	file2_->setText(file);
}

static inline bool isRegularLine(QCharRef ch) {
	if (ch != QChar(0x01) && ch != QChar(0x02))
		return true;
	return false;
}

DiffWorksDialog::DiffWorksDialog(QFont &fnt, QString file1, QString file2): QDialog(0, "diffworks", true) {
LOG("file1 %s, file2 %s", (const char *) file1, (const char *) file2);
	Global *g = Global::instance();
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);
	
	// file1
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	f1_ = new QLabel("1: " + file1, hbox1);

	// file2
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	f2_ = new QLabel("2: " + file2, hbox2);

	// text box
	Q3HBox *hbox3 = new Q3HBox(this);
	hbox3->setSpacing(10);
	text_ = new Q3MultiLineEdit(hbox3);
	syntax_ = getSyntax(".diff", text_);
	// config text
	text_->setFont(fnt);
	text_->setWordWrap(Q3TextEdit::NoWrap);
	text_->setReadOnly(TRUE);
	// colors
	text_->setPaletteBackgroundColor(g->c_background);
	text_->setPaletteForegroundColor(g->c_foreground);
	QPalette qp = text_->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, g->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, g->c_highlight);
	text_->setPalette(qp);
	// do diff
	BCursor busy;
	QString result = doDiff((const char *) file1, (const char *) file2);
	text_->setText(QString::fromUtf8(result));
	resize(Global::instance()->diff_x, Global::instance()->diff_y);

	// accept/reject
	Q3HBox *hbox4 = new Q3HBox(this);
	hbox4->setSpacing(10);
	if (result == "Files do not differ!") {
		// only ok button is shown
		new QWidget(hbox4);
		new QWidget(hbox4);
		new QWidget(hbox4);
		new QWidget(hbox4);
		button_ok_ = new QPushButton("Ok", hbox4);
		connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
		return;
	}
	// else show next/previous/ok
	button_next_ = new QPushButton("Next", hbox4);
	button_previous_ = new QPushButton("Previous", hbox4);
	// some space
	(new QWidget(hbox4))->setMinimumWidth(80);
	(new QWidget(hbox4))->setMinimumWidth(80);
	button_ok_ = new QPushButton("Ok", hbox4);
	connect(button_ok_, SIGNAL(clicked()), SLOT(accept()));
	connect(button_next_, SIGNAL(clicked()), this, SLOT(clicked_next()));
	connect(button_previous_, SIGNAL(clicked()), this, SLOT(clicked_previous()));

	// set initial selection
	int line;
	int col;
	text_->getCursorPosition(&line, &col);
	QString str = text_->textLine(line);
	if (isRegularLine(str.at(0)))
		clicked_next();
	else {
		int start = line;
		int end = start + 1;
		while (!text_->textLine(end).isNull() &&
		       !text_->textLine(end).isEmpty() &&
		       !isRegularLine(text_->textLine(end).at(0)))
			end++;
		
		text_->setSelection(start, 0, end, 0);
	}
}

void DiffWorksDialog::clicked_next() {
	int line;
	int col;
	BCursor busy;
	text_->getCursorPosition(&line, &col);
	QString str = "";
	bool found = false;
	if (line != 0)
		line++;
	while (1) {
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			return;		
		str = text_->textLine(line);
		if (!isRegularLine(str.at(0))) {
			found = true;
			break;
		}
		line++;
	}
	
	int start = line;
	int end = line;
	while (found) {
		line++;
		end = line;
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			break;		
		str = text_->textLine(line);
		if (str.isNull() || str.isEmpty())
			break;
		if (isRegularLine(str.at(0)))
			break;
	}
	
	text_->setSelection(start, 0, end, 0);
	update();
}

void DiffWorksDialog::clicked_previous() {
	int line;
	int col;
	BCursor busy;
	text_->getCursorPosition(&line, &col);
	if (!text_->atEnd())
		line--;
	QString str = text_->textLine(line);
	while (line >= 0 && !isRegularLine(str.at(0))) {
		line--;
		str = text_->textLine(line);
	}
		
	bool found = false;
	while (line >= 0) {
		line--;
		text_->setCursorPosition(line, 0, false);
		str = text_->textLine(line);
		if (!isRegularLine(str.at(0))) {
			found = true;
			break;
		}
	}
	
	int start = line;
	int end = line;
	while (found && line >= 0) {
		line--;
		end = line;
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			return;		
		str = text_->textLine(line);
		if (str.isNull() || str.isEmpty())
			break;
		if (isRegularLine(str.at(0)))
			break;
	}
	text_->setSelection(end + 1, 0, start + 1, 0);
	update();
}

void DiffWorksDialog::resizeEvent( QResizeEvent * )
{
	LOG("DiffWorksDialog::resizeEvent sizes %d %d", width(), height());
//	if (e_ && m_ )
//		e_->setGeometry( 0, m_->height(), width(), height() - m_->height());
	Global::instance()->diff_x = width();
	Global::instance()->diff_y = height();
}

MergeWorksDialog::MergeWorksDialog(QFont &fnt, QString file1, QString file2): QDialog(0, "diffworks", true) {
LOG("file1 %s, file2 %s", (const char *) file1, (const char *) file2);
	saved_ = false;
	Global *g = Global::instance();
	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);
	
	// file1
	Q3HBox *hbox1 = new Q3HBox(this);
	hbox1->setSpacing(10);
	f1_ = new QLabel("1: " + file1, hbox1);

	// file2
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	f2_ = new QLabel("2: " + file2, hbox2);

	// text box
	Q3HBox *hbox3 = new Q3HBox(this);
	hbox3->setSpacing(10);
	text_ = new Q3MultiLineEdit(hbox3);
	syntax_ = getSyntax(".diff", text_);
	// configure text
	text_->setFont(fnt);
	text_->setWordWrap(Q3TextEdit::NoWrap);
	// colors
	text_->setPaletteBackgroundColor(g->c_background);
	text_->setPaletteForegroundColor(g->c_foreground);
	QPalette qp = text_->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, g->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, g->c_highlight);
	text_->setPalette(qp);
	connect(text_, SIGNAL(textChanged()), this, SLOT(textChanged()));
	// diff
	BCursor busy;
	QString result = doDiff((const char *) file1, (const char *) file2);
	text_->setText(QString::fromUtf8(result));
	resize(Global::instance()->diff_x, Global::instance()->diff_y);



	// accept/reject
	Q3HBox *hbox4 = new Q3HBox(this);
	hbox4->setSpacing(10);
	if (result == "Files do not differ!") {
		new QWidget(hbox4);
		new QWidget(hbox4);
		new QWidget(hbox4);
		new QWidget(hbox4);
		new QWidget(hbox4);
		new QWidget(hbox4);
		button_cancel_ = new QPushButton("Cancel", hbox4);
		connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
		return;
	}
	// else show all buttons
	button_next_ = new QPushButton("Next", hbox4);
	button_previous_ = new QPushButton("Previous", hbox4);
	// some space
	new QWidget(hbox4);
	button_one_ = new QPushButton("1", hbox4);
	button_two_ = new QPushButton("2", hbox4);
	button_save_ = new QPushButton("Save", hbox4);
	// some space
	new QWidget(hbox4);
	button_ok_ = new QPushButton("Ok", hbox4);
	button_cancel_ = new QPushButton("Cancel", hbox4);

	connect(button_ok_, SIGNAL(clicked()), SLOT(clicked_ok())); //accept()
	connect(button_cancel_, SIGNAL(clicked()), SLOT(reject()));
	connect(button_next_, SIGNAL(clicked()), this, SLOT(clicked_next()));
	connect(button_previous_, SIGNAL(clicked()), this, SLOT(clicked_previous()));
	connect(button_save_, SIGNAL(clicked()), this, SLOT(clicked_save()));
	connect(button_one_, SIGNAL(clicked()), SLOT(clicked_one()));
	connect(button_two_, SIGNAL(clicked()), SLOT(clicked_two()));

	
	// set initial selection
	int line;
	int col;
	text_->getCursorPosition(&line, &col);
	QString str = text_->textLine(line);
	if (isRegularLine(str.at(0)))
		clicked_next();
	else {
		int start = line;
		int end = start + 1;
		while (!text_->textLine(end).isNull() &&
		       !text_->textLine(end).isEmpty() &&
		       !isRegularLine(text_->textLine(end).at(0)))
			end++;
		
		text_->setSelection(start, 0, end, 0);
	}	
}

void MergeWorksDialog::textChanged() {
	saved_ = false;
}

bool MergeWorksDialog::isMoreToMerge() {
	QString str1 = QChar(0x01);
	QString str2 = QChar(0x02);
	int index = 0;
	int para = 0;
	bool found = text_->find(str1, true, false, true, &index, &para);
	if (found) {
		QMessageBox::warning(this,"Merge Info", QString("There is more to merge!\n"));
		text_->setCursorPosition(index, para, false);
		text_->setFocus();
		return true;
	}
	found = text_->find(str2, true, false, true, &index, &para);
	if (found) {
		QMessageBox::warning(this,"Merge Info", QString("There is more to merge!\n"));
		text_->setCursorPosition(index, para, false);
		text_->setFocus();
		return true;
	}
	return false;

}
void MergeWorksDialog::clicked_one() {
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	BCursor busy;
	
	text_->getSelection(&para_from, &index_from, &para_to, &index_to);
	if (para_from == -1 || index_from == -1 || para_to == -1 || index_to == -1) {
		return;
	}

	for (int i = para_from; i < para_to; i++) {
		QString line = text_->textLine(i);
		if (line.startsWith(QChar(0x01))) {
			line = line.mid(4);
			text_->setSelection(i, 0, i + 1, 0);
			text_->del();
			text_->insert(line + QString("\n"));
		}
		else if (line.startsWith(QChar(0x02))) {
			text_->setSelection(i, 0, i + 1, 0);
			text_->del();
			i--;
			para_to--;
		}
	}
	
	clicked_next();
}

void MergeWorksDialog::clicked_two() {
	int para_from;
	int index_from;
	int para_to;
	int index_to;
	BCursor busy;
	
	text_->getSelection(&para_from, &index_from, &para_to, &index_to);
	if (para_from == -1 || index_from == -1 || para_to == -1 || index_to == -1) {
		return;
	}

	for (int i = para_from; i < para_to; i++) {
		QString line = text_->textLine(i);
		if (line.startsWith(QChar(0x02))) {
			line = line.mid(4);
			text_->setSelection(i, 0, i + 1, 0);
			text_->del();
			text_->insert(line + QString("\n"));
		}
		else if (line.startsWith(QChar(0x01))) {
			text_->setSelection(i, 0, i + 1, 0);
			text_->del();
			i--;
			para_to--;
		}
	}
	
	clicked_next();
}

void MergeWorksDialog::clicked_save() {
	if (isMoreToMerge())
		return;
	
//	BCursor busy;
	saved_ = true;
	QString fileName = Q3FileDialog::getSaveFileName( QString::null, QString::null, this );
	BCursor busy;
	if (fileName.isEmpty()) {
		QMessageBox::warning(this,"I/O Error", QString("The file could not be opened.\n\n"));
		return;
	}

	FILE_FORMAT format = FILE_UTF8; 
	QFile f( fileName );
#ifdef USING_QT3
	if (!f.open(IO_WriteOnly )) {
#endif
#ifdef USING_QT4
	if (!f.open( QIODevice::WriteOnly )) {
#endif

		QMessageBox::warning(this,"I/O Error", QString("The file could not be opened.\n\n") + fileName);
		return;
	}
	QTextStream t(&f);
	LOG("save common format %d", format);

	switch (format) {
		case FILE_UTF8:
			t.setEncoding( QTextStream::UnicodeUTF8 );
			LOG("Editor::saveCommon utf8");
			break;
		case FILE_CP1250: {
				// lucian: qt has some problems with this conversion!!!                
				QTextCodec *codec = QTextCodec::codecForName("CP1250");
				t.setCodec(codec);
				LOG("Editor::saveCommon cp1250");
			}
			break;
		case FILE_ISO8859_2: {
				QTextCodec *codec = QTextCodec::codecForName("ISO8859-2");
				t.setCodec(codec);
				LOG("Editor::saveCommon iso8859-2");
			}
			break;
		case FILE_ISO8859_16: {
				QTextCodec *codec = QTextCodec::codecForName("ISO8859-16");
				t.setCodec(codec);
				LOG("Editor::saveCommon iso8859-16");
			}
			break;
		default:
			ASSERT(0);
			return;
	}

	QString tmp;
	
	// if this is a latex file save it in latex/ascii format
	if (fileName.endsWith(".tex"))
		tmp = utf82latex(text_->text());
	else
		tmp = text_->text();
	t << tmp;
	f.close();
}

void MergeWorksDialog::clicked_ok() {
	if (isMoreToMerge())
		return;
	if (!saved_) {
		QMessageBox::warning(this,"Merge Info", QString("Merge result was not saved!\n"));
		return;
	}

	accept();
}

void MergeWorksDialog::clicked_next() {
	int line;
	int col;
	BCursor busy;
	text_->getCursorPosition(&line, &col);
	QString str = "";
	bool found = false;
	if (!line != 0)
		line++;
	while (1) {
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			return;		
		str = text_->textLine(line);
		if (!isRegularLine(str.at(0))) {
			found = true;
			break;
		}
		line++;
	}
	
	int start = line;
	int end = line;
	while (found) {
		line++;
		end = line;
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			break;		
		str = text_->textLine(line);
		if (str.isNull() || str.isEmpty())
			break;
		if (isRegularLine(str.at(0)))
			break;
	}
	
	text_->setSelection(start, 0, end, 0);
	update();
}

void MergeWorksDialog::clicked_previous() {
	int line;
	int col;
	BCursor busy;
	text_->getCursorPosition(&line, &col);
	if (!text_->atEnd())
		line--;
	QString str = text_->textLine(line);
	while (line >= 0 && !isRegularLine(str.at(0))) {
		line--;
		str = text_->textLine(line);
	}
		
	bool found = false;
	while (line >= 0) {
		line--;
		text_->setCursorPosition(line, 0, false);
		str = text_->textLine(line);
		if (!isRegularLine(str.at(0))) {
			found = true;
			break;
		}
	}
	
	int start = line;
	int end = line;
	while (found && line >= 0) {
		line--;
		end = line;
		text_->setCursorPosition(line, 0, false);
		if (text_->atEnd())
			return;		
		str = text_->textLine(line);
		if (str.isNull() || str.isEmpty())
			break;
		if (isRegularLine(str.at(0)))
			break;
	}
	text_->setSelection(end + 1, 0, start + 1, 0);
	update();
}

void MergeWorksDialog::resizeEvent( QResizeEvent * )
{
	LOG("MergeWorksDialog::resizeEvent sizes %d %d", width(), height());
//	if (e_ && m_ )
//		e_->setGeometry( 0, m_->height(), width(), height() - m_->height());
	Global::instance()->diff_x = width();
	Global::instance()->diff_y = height();
}
