//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************

#ifndef EDITOR_H
#define EDITOR_H

// import data directory location, freebsd flag, etc
#include "../../dictfiles.h"

// strcasestr is missing form Cygwin
char *strcasestr(const char *haystack, const char *needle);

// import options from global config file generated by autotools:
// - no printer support
//         #define ROSPELL_NO_PRINTER 
#include "../../config.h"


//*************************************************************************
// some compile-time options
//*************************************************************************
// maximum number of bookmarks
#define MAXBOOKMARKS 10

// maximum number of buffers handled by BufMgr
#define MAXBUFMGR    40


// enable standard C library ctags support
#define ENABLE_STDLIBC_CTAGS

// enable POSIX headers ctags support
#define ENABLE_POSIXH_CTAGS

// enable latex ctags support
#define ENABLE_LATEX_CTAGS


//*************************************************************************
// default color scheme
//*************************************************************************
#define DEFAULT_BACKGROUND_COLOR QColor(0, 0, 0)
#define DEFAULT_FOREGROUND_COLOR QColor(255, 255, 255)
#define DEFAULT_HIGHLIGHT_COLOR  QColor(200, 0, 0)
// xml tag and C/C++ preproc
#define DEFAULT_TAG1_COLOR       QColor(255, 25, 25)
// C/C++ strings
#define DEFAULT_TAG2_COLOR       QColor(200, 255, 150)
// C/C++ comments
#define DEFAULT_TAG3_COLOR       QColor(150, 150, 255)
// C/C++ keywords
#define DEFAULT_TAG4_COLOR       QColor(75, 200, 75)


//*************************************************************************
// QT3 or QT4 selection
//*************************************************************************
//#define USING_QT3
//#define USING_QT4
// the selection is made from makefile on g++ command line

// qt4 to qt3 translations
#ifdef USING_QT3
#define Q3VBox QVBox
#define Q3Accel QAccel
#define Q3PopupMenu QPopupMenu
#define Q3SyntaxHighlighter QSyntaxHighlighter
#define Q3MultiLineEdit QMultiLineEdit
#define Q3VBoxLayout QVBoxLayout
#define Q3HBox QHBox
#define Q3TextEdit QTextEdit
#define Q3FileDialog QFileDialog
#define Q3CString QCString
#define Q3PaintDeviceMetrics QPaintDeviceMetrics 
#endif


#endif
