//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************
#include "fformat.h"
#include "debug.h"
#include <string.h>
#include <qtextcodec.h>

typedef struct {
	FILE_FORMAT format;
	const char *str;
	const char *qt_codec;
} FormatDesc, *pFormatDesc;

static FormatDesc format_description[] = {
	{ FILE_UTF8, "--utf8", "UTF8" },
	{ FILE_ISO8859_1, "--iso8859-1", "ISO8859-1" },
	{ FILE_ISO8859_2, "--iso8859-2", "ISO8859-2" },
	{ FILE_ISO8859_3, "--iso8859-3", "ISO8859-3" },
	{ FILE_ISO8859_4, "--iso8859-4", "ISO8859-4" },
	{ FILE_ISO8859_5, "--iso8859-5", "ISO8859-5" },
	{ FILE_ISO8859_6, "--iso8859-6", "ISO8859-6" },
	{ FILE_ISO8859_7, "--iso8859-7", "ISO8859-7" },
	{ FILE_ISO8859_8, "--iso8859-8", "ISO8859-8" },
	{ FILE_ISO8859_9, "--iso8859-9", "ISO8859-9" },
	{ FILE_ISO8859_10, "--iso8859-10", "ISO8859-10" },
	{ FILE_ISO8859_11, "--iso8859-11", "ISO8859-11" },
	{ FILE_ISO8859_12, "--iso8859-12", "ISO8859-12" },
	{ FILE_ISO8859_13, "--iso8859-13", "ISO8859-13" },
	{ FILE_ISO8859_14, "--iso8859-14", "ISO8859-14" },
	{ FILE_ISO8859_15, "--iso8859-15", "ISO8859-15" },
	{ FILE_ISO8859_16, "--iso8859-16", "ISO8859-16" },
	{ FILE_CP1250, "--cp1250", "CP1250" },
	{ FILE_FORMAT_MAX, "  unknown format", "UNKNOWN" }
};

FILE_FORMAT ff_parse_arg(const char *arg) {
	int i = 0;
	while (format_description[i].format < FILE_FORMAT_MAX) {
		if (strcmp(arg, format_description[i].str) == 0)
			return format_description[i].format;
		i++;
	}

	if (strcmp(arg, "--latin1") == 0)
		return FILE_ISO8859_1;
	
	if (strcmp(arg, "--latin2") == 0)
		return FILE_ISO8859_2;
	
	if (strcmp(arg, "--latin3") == 0)
		return FILE_ISO8859_3;
	
	if (strcmp(arg, "--latin4") == 0)
		return FILE_ISO8859_4;
	
	if (strcmp(arg, "--cyrillic") == 0)
		return FILE_ISO8859_5;
	
	if (strcmp(arg, "--arabic") == 0)
		return FILE_ISO8859_6;
	
	if (strcmp(arg, "--greek") == 0)
		return FILE_ISO8859_7;
	
	if (strcmp(arg, "--hebrew") == 0)
		return FILE_ISO8859_8;
	
	if (strcmp(arg, "--latin5") == 0)
		return FILE_ISO8859_9;
	
	if (strcmp(arg, "--latin6") == 0)
		return FILE_ISO8859_10;
	
	return FILE_FORMAT_MAX;
}

const char *ff_str(FILE_FORMAT format) {
	int i = 0;
	while (format_description[i].format < FILE_FORMAT_MAX) {
		if (format_description[i].format == format)
			return format_description[i].qt_codec;
		i++;
	}
	ASSERT(format_description[i].format == FILE_FORMAT_MAX);
	return format_description[i].qt_codec;
}

QString ff_toUtf8(const char *str, FILE_FORMAT format) {
	QString retval;

	if (format == FILE_UTF8) {
printf("ff_toUtf8 UTF8\n");	
		retval = QString::fromUtf8(str);
		return retval;
	}

	if (format == FILE_FORMAT_MAX) {
		ASSERT(0);
		return "UNKNOWN";
	}
	
	QTextCodec *codec = QTextCodec::codecForName(ff_str(format));
printf("ff_to_Utf8 %s\n", ff_str(format));
//	QTextCodec *codec = QTextCodec::codecForName("ISO8859-2");
	QTextDecoder *decoder = codec->makeDecoder();
	retval = decoder->toUnicode(str, strlen(str));
	return retval;
}

void ff_populateList(QStringList &lst) {
	int i = 0;
	while (format_description[i].format < FILE_FORMAT_MAX) {
		lst << format_description[i].qt_codec;
		i++;
	}
}

FILE_FORMAT ff_format(const char *codec) {
	int i = 0;
	while (strcmp(format_description[i].qt_codec, codec) != 0)
		i++;
	return (FILE_FORMAT) i;
}
