//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************
#include "filevisitor.h"
#include "utils.h"
#include "qtutils.h"
#include <qdir.h>
#include <qstringlist.h>
#include <qregexp.h>

QString FVisitorDirfiles::visit(QFileInfo *fi) {
	return fi->fileName() + "\n";
}



char *FVisitorGrep::getLine(char *str) {
	while (*str != '\n' && *str != '\0')
		str++;
	
	if (*str == '\0') {
		return str;
	}
		
	*str = '\0';
	str++;
	return str;
}

QString FVisitorGrep::visit(QFileInfo *fi) {
//printf("%s\n", (const char *) fi->absFilePath());fflush(0);
	if (fi->isDir())
		return "";
	QString retval = "";
	char *fstr = UTILS::read_file_malloc((const char *) fi->absFilePath());
	if (fstr == 0)
		return "";
	char *ptr1 = fstr;
	char *ptr2 = fstr;
	int n = 1;
	while (*ptr1 != '\0') {
		ptr2 = getLine(ptr1);
		QString qs = QString(ptr1);
		int found = QTUTILS::find(qs, tofind_, 0, cs_, whole_);
		if (found != -1) {
			while (*ptr1 == ' ' || *ptr1 == '\t')
				ptr1++;
			QString txt = fi->absFilePath() + QString(":%1: ").arg(n) +QString("\n\t") + QString(ptr1) + QString("\n");
			retval += txt;
		}
		n++;
		ptr1 = ptr2;
	}
	
	delete [] fstr;
	return retval;
}





QString FVBase::walk(FVisitor *pfv) {
	ASSERT(pfv != 0);
	QString retval = "";
	QDir *thisDir = new QDir(start_dir_);
	const QFileInfoList * files = thisDir->entryInfoList();
	if ( files ) {
		QStringList lst(QStringList::split( ",", file_filter_));
		QStringList lst_dir(QStringList::split( ",", exclude_dir_filter_));

		QFileInfoListIterator it(*files);
		QFileInfo *fi;
		while( (fi=it.current()) != 0 ) {
			++it;
			if ( fi->fileName() == ".")
				continue;
			if ( fi->fileName() == "..") {
				retval += pfv->visit(fi);
				continue;
			}

			if (fi->isDir() && recursive_) {
				QStringList::Iterator it3 = lst_dir.begin();
				bool found = false;
				for ( ; it3 != lst_dir.end(); ++it3 ) {
					QString exp = *it3;
					QRegExp rx(exp);
					rx.setWildcard(TRUE);
					if (rx.exactMatch(fi->fileName())) {
						found = true;
						break;
					}
				}
				if (!found) {
					QString tmp = start_dir_;
					start_dir_ = start_dir_ + fi->fileName() + "/";
					retval += walk(pfv);
					start_dir_ = tmp;
					continue;
				}
			}
					
			QStringList::Iterator it2 = lst.begin();
			bool found = false;
			for ( ; it2 != lst.end(); ++it2 ) {
				QString exp = *it2;
//printf("%s\n", (const char *) exp1);
				// regex on dircombo
				QRegExp rx(exp);
				rx.setWildcard(TRUE);
				if (rx.exactMatch(fi->fileName()) || fi->isDir()) {
					found = true;
					break;
				}
			}
			if (found) 
				retval += pfv->visit(fi);
		}
	}
	delete thisDir;
	return retval;
}

