//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************
#include "global.h"
#include "utils.h"

QString Global::getCmd(const char *name) {
	ASSERT(name != 0);
	// check for command
	QString cmd1 = QString("which ") + name;
	char *cmd2 = UTILS::run_prg_malloc((const char *) cmd1);
	if (strstr(cmd2, name) == 0) {
		delete[] cmd2;
		return "";
	}
	
	return QString(name);
}

void Global::readCfg() {
	char *fname = getCfgFname();
	LOG("Global::readCfg from %s", fname);
	char *cfg = UTILS::read_file_malloc(fname);
	if (cfg == 0) {
		char *pmkdir = UTILS::run_prg_malloc("mkdir ~/.rospell");
		delete[] pmkdir;
		pmkdir = UTILS::run_prg_malloc("mkdir ~/.rospell/backup");
		delete[] fname;
		LOG("Global::getCfg - starting with default values");
		return;
	}

	if (strstr(cfg, "fixed_dict") != 0)
		fixeddict = true;

	char *ptr;
	if ((ptr = strstr(cfg, "xsize")) != 0) {
		int n = getNumber(ptr);
		if (n > 0)
			xsize = n;
	}
	if ((ptr = strstr(cfg, "ysize")) != 0) {
		int n = getNumber(ptr);
		if (n > 0)
			ysize = n;
	}
	if ((ptr = strstr(cfg, "diff_x")) != 0) {
		int n = getNumber(ptr);
		if (n > 0)
			diff_x = n;
	}
	if ((ptr = strstr(cfg, "diff_y")) != 0) {
		int n = getNumber(ptr);
		if (n > 0)
			diff_y = n;
	}
#ifdef ENABLE_STDLIBC_CTAGS
	if ((ptr = strstr(cfg, "load_glibc_tags")) != 0) {
		load_glibc_tags = true;
	}
#endif
#ifdef ENABLE_POSIXH_CTAGS
	if ((ptr = strstr(cfg, "load_posixh_tags")) != 0) {
		load_posixh_tags = true;
	}
#endif
#ifdef ENABLE_LATEX_CTAGS
	if ((ptr = strstr(cfg, "load_latex_tags")) != 0) {
		load_latex_tags = true;
	}
#endif
	
	if ((ptr = strstr(cfg, "generate_local_tags")) != 0) {
		generate_local_tags = true;
	}

	if ((ptr = strstr(cfg, "lang_ro")) != 0) {
		lang_ro = true;
		lang_en = false;
	}
	else if ((ptr = strstr(cfg, "lang_en")) != 0) {
		lang_ro = false;
		lang_en = true;
	}
	
	if ((ptr = strstr(cfg, "background_color")) != 0) {
		ptr += strlen("background_color") + 1;
		int r;
		int g;
		int b;
		sscanf(ptr, "%d %d %d", &r, &g, &b);
		c_background = QColor(r, g, b);
	}
	if ((ptr = strstr(cfg, "foreground_color")) != 0) {
		ptr += strlen("foreground_color") + 1;
		int r;
		int g;
		int b;
		sscanf(ptr, "%d %d %d", &r, &g, &b);
		c_foreground = QColor(r, g, b);
	}
	if ((ptr = strstr(cfg, "highlight_color")) != 0) {
		ptr += strlen("highlight_color") + 1;
		int r;
		int g;
		int b;
		sscanf(ptr, "%d %d %d", &r, &g, &b);
		c_highlight = QColor(r, g, b);
	}
	if ((ptr = strstr(cfg, "tag1_color")) != 0) {
		ptr += strlen("tag1_color") + 1;
		int r;
		int g;
		int b;
		sscanf(ptr, "%d %d %d", &r, &g, &b);
		c_tag1 = QColor(r, g, b);
	}
	if ((ptr = strstr(cfg, "tag2_color")) != 0) {
		ptr += strlen("tag2_color") + 1;
		int r;
		int g;
		int b;
		sscanf(ptr, "%d %d %d", &r, &g, &b);
		c_tag2 = QColor(r, g, b);
	}
	if ((ptr = strstr(cfg, "tag3_color")) != 0) {
		ptr += strlen("tag3_color") + 1;
		int r;
		int g;
		int b;
		sscanf(ptr, "%d %d %d", &r, &g, &b);
		c_tag3 = QColor(r, g, b);
	}
	if ((ptr = strstr(cfg, "tag4_color")) != 0) {
		ptr += strlen("tag4_color") + 1;
		int r;
		int g;
		int b;
		sscanf(ptr, "%d %d %d", &r, &g, &b);
		c_tag4 = QColor(r, g, b);
	}
	// beauty
	if ((ptr = strstr(cfg, "beauty_bcl")) != 0)
		beauty_bcl = true;
	else
		beauty_bcl = false;
	if ((ptr = strstr(cfg, "beauty_tbcl")) != 0)
		beauty_tbcl = true;
	else
		beauty_tbcl = false;
	if ((ptr = strstr(cfg, "beauty_tab")) != 0)
		beauty_tab = true;
	else
		beauty_tab = false;


	delete[] fname;
	delete[] cfg;
}

void Global::saveCfg() {
	char *fname = getCfgFname();
	LOG("Global::saveCfg in %s", fname);
	char *cfg = new char[10000];
	memset(cfg, 0, 10000);
	strcpy(cfg, "rospell configuration\n");

	if (fixeddict == true)
		strcat(cfg, "fixed_dict\n");
#ifdef    ENABLE_STDLIBC_CTAGS
	if (load_glibc_tags == true)
		strcat(cfg, "load_glibc_tags\n");
#endif
#ifdef    ENABLE_POSIXH_CTAGS
	if (load_posixh_tags == true)
		strcat(cfg, "load_posixh_tags\n");
#endif
#ifdef    ENABLE_LATEX_CTAGS
	if (load_latex_tags == true)
		strcat(cfg, "load_latex_tags\n");
#endif
	if (generate_local_tags == true)
		strcat(cfg, "generate_local_tags\n");

	if (xsize > 0) {
		char tmp[100];
		sprintf(tmp, "xsize %d\n", xsize);
		strcat(cfg, tmp);
	}
	if (ysize > 0) {
		char tmp[100];
		sprintf(tmp, "ysize %d\n", ysize);
		strcat(cfg, tmp);
	}
	if (diff_x > 0) {
		char tmp[100];
		sprintf(tmp, "diff_x %d\n", diff_x);
		strcat(cfg, tmp);
	}
	if (diff_y > 0) {
		char tmp[100];
		sprintf(tmp, "diff_y %d\n", diff_y);
		strcat(cfg, tmp);
	}
	if (lang_ro)
		strcat(cfg, "lang_ro\n");
	else if (lang_en)
		strcat(cfg, "lang_en\n");

	char tmpstr[500];
	sprintf(tmpstr, "background_color %d %d %d\n", c_background.red(), c_background.green(), c_background.blue());
	strcat(cfg, tmpstr);
	sprintf(tmpstr, "foreground_color %d %d %d\n", c_foreground.red(), c_foreground.green(), c_foreground.blue());
	strcat(cfg, tmpstr);
	sprintf(tmpstr, "highlight_color %d %d %d\n", c_highlight.red(), c_highlight.green(), c_highlight.blue());
	strcat(cfg, tmpstr);
	sprintf(tmpstr, "tag1_color %d %d %d\n", c_tag1.red(), c_tag1.green(), c_tag1.blue());
	strcat(cfg, tmpstr);
	sprintf(tmpstr, "tag2_color %d %d %d\n", c_tag2.red(), c_tag2.green(), c_tag2.blue());
	strcat(cfg, tmpstr);
	sprintf(tmpstr, "tag3_color %d %d %d\n", c_tag3.red(), c_tag3.green(), c_tag3.blue());
	strcat(cfg, tmpstr);
	sprintf(tmpstr, "tag4_color %d %d %d\n", c_tag4.red(), c_tag4.green(), c_tag4.blue());
	strcat(cfg, tmpstr);

	if (beauty_bcl == true)
		strcat(cfg, "beauty_bcl\n");
	if (beauty_tbcl == true)
		strcat(cfg, "beauty_tbcl\n");
	if (beauty_tab == true)
		strcat(cfg, "beauty_tab\n");

	UTILS::write_file(fname, cfg);

	delete[] fname;
	delete[] cfg;
}

void Global::readAliasFile() {
	// read alias file
	char *fname = getAliasFname();
	LOG("Global::readAliasFile from %s", fname);
	char *alstr = UTILS::read_file_malloc(fname);
	if (alstr != 0) {
		QString ftext = alstr;
		int i = 0;
		while (1) {
			QString alias = ftext.section('\n', i, i);
			if (alias.isEmpty())
				break;
			i++;
			QString replacement = ftext.section('\n', i, i);
			if (replacement.isEmpty())
				break;
			i++;
			alist.push_back(Global::Alias(alias, replacement));
		}
		delete[] alstr;
		delete[] fname;
	}
//	alist_.push_back(Alias(QString("#in"), QString("#include ")));
//	alist_.push_back(Alias(QString("#de"), QString("#define ")));
//	alist_.push_back(Alias(QString("#en"), QString("#endif")));
//	alist_.push_back(Alias(QString("#el"), QString("#else")));
//	alist_.push_back(Alias(QString("(cc"), QString("(const char *)")));
}

void Global::saveAliasFile() {
	list<Alias>::iterator it;
	QString str = "";
	for (it = alist.begin(); it != alist.end(); ++it) {
		Global::Alias al = *it;
		str += al.alias + "\n" + al.replace + "\n";
	}

	char *fname = getAliasFname();
	LOG("Global::saveAliasFile in %s", fname);
	UTILS::write_file(fname, (const char *) str);
	delete[] fname;
}

int Global::getNumber(char *ptr) {
	ASSERT(ptr != 0);
	while (*ptr != ' ') {
		if (*ptr == '\0')
			return 0;
		ptr++;
	}

	int retval;
	sscanf(ptr, "%d", &retval);
	LOG("Global::getNumber %d", retval);
	return retval;
}

char *Global::getCfgFname() {
	char *homedir = getenv("HOME");
	if (homedir == 0) {
		fprintf(stderr, "Error: HOME environment variable nu a fost g?sit?\n");
		LOG("home env var not found, exit program");
		exit(1);
	}

	unsigned len = strlen(homedir) + strlen("/.rospell/config");
	char *fname = new char[len + 1];
	memset(fname, '\0', len + 1);
	strcpy(fname, homedir);
	strcat(fname, "/.rospell/config");
	return fname;
}

char *Global::getAliasFname() {
	char *homedir = getenv("HOME");
	if (homedir == 0) {
		fprintf(stderr, "Error: HOME environment variable nu a fost g?sit?\n");
		LOG("home env var not found, exit program");
		exit(1);
	}

	unsigned len = strlen(homedir) + strlen("/.rospell/alias");
	char *fname = new char[len + 1];
	memset(fname, '\0', len + 1);
	strcpy(fname, homedir);
	strcat(fname, "/.rospell/alias");
	return fname;
}

// which command gives a different output on freebsd and linux
// - on linux the output goes to stderr
bool Global::checkWhich(const char *command) {
	ASSERT(command != 0);
	int len = strlen(command) + strlen("which ") + 1; // +1 for '\0'
	char *cmd = new char[len];
	strcpy(cmd, "which ");
	strcat(cmd, command);
//		ASSERT(strlen(cmd) == (len - 1));
	
	bool retval = true;
	char *output = UTILS::run_prg_malloc(cmd);
	if (output == 0 || strstr(output, " no ") != 0) {
		QMessageBox::warning(0, command, "command not installed on your system");
		retval = false;
	}
	
	delete[] cmd;
	if (output != 0)
		delete[] output;
	return retval;
}

char *Global::getBackupFname(const char *fname2backup) {
	ASSERT(fname2backup != 0);
	char *homedir = getenv("HOME");
	if (homedir == 0) {
		fprintf(stderr, "Error: HOME environment variable nu a fost g?sit?\n");
		LOG("home env var not found, exit program");
		exit(1);
	}

	unsigned len = strlen(homedir) + strlen("/.rospell/backup/") + strlen(fname2backup);
	char *fname = new char[len + 1];
	memset(fname, '\0', len + 1);
	strcpy(fname, homedir);
	strcat(fname, "/.rospell/backup/");
	strcat(fname, fname2backup);
	return fname;
}


QString Global::unixHomeConvert(const QString &fname) {
	if (!fname.startsWith("~"))
		return fname;
	char *homedir = getenv("HOME");
	if (homedir == 0) {
		fprintf(stderr, "Error: HOME environment variable nu a fost g?sit?\n");
		LOG("home env var not found, exit program");
		exit(1);
	}
	QString retval = QString(homedir) + fname.mid(1);
	return retval;
}


