//*********************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//*********************************************************************
// User configuration read and saved back to ~/.rospell directory
//*********************************************************************

#ifndef GLOBAL_H
#define GLOBAL_H
#include "singleton.h"
#include "utils.h"
#include "editor.h"
#include <qcolor.h>
#include <qstring.h>
#include <qmessagebox.h>

#include <list>
using namespace std;


#include "utf8.h"
#include "fformat.h"

class Q3SyntaxHighlighter;
class Q3TextEdit;
Q3SyntaxHighlighter *getSyntax(QString fname, Q3TextEdit *textEdit);
QString doDiff(const char *f1, const char *f2);
class SearchDialog;
class GrepDialog;
class ReplaceDialog;
extern void clearGlobal();



class Global: public Singleton<Global> {
public:
	struct Alias {
		QString alias;
		QString reverse;
		QString replace;

		Alias(QString key, QString rep) {
			alias = key;
			replace = rep;
			update();
		}

		void update() {
			QString tmp = alias.upper();
			reverse = "";
			int len = tmp.length();
			for (int i = 0; i < len; i++) {
				reverse.prepend(tmp.at(i));
			}
		}
	};

public:
	// alias list
	list<Alias> alist;
	// command line params
	bool addallmaster;
	bool addallmasterenglish;
	bool addalluser;
	bool dumpall;
	// support for adding words in the fixed dictionary
	bool fixeddict;
	// screen sizes
	int xsize;
	int ysize;
	// colors
	QColor c_background;
	QColor c_foreground;
	QColor c_highlight;
	QColor c_tag1;
	QColor c_tag2;
	QColor c_tag3;
	QColor c_tag4;
	// load standard C library tags
	bool load_glibc_tags;
	// load POSIX headers tags
	bool load_posixh_tags;
	// load latex tags
	bool load_latex_tags;
	// generate local tags
	bool generate_local_tags; 
	// tags cmd - will not get saved in config file
	QString tags_cmd;
	// diff window sizes
	int diff_x;
	int diff_y;
	// language support
	bool lang_ro;
	bool lang_en;
	// beautifier support
	bool beauty_bcl;
	bool beauty_tbcl;
	bool beauty_tab;
	SearchDialog *sd;
	GrepDialog *gd;
	ReplaceDialog *rd;
	QString make_cmd_;
	QString diff_dir1;
	QString diff_dir2;


public:
	// global defaults
	Global(): addallmaster(false), addallmasterenglish(true), addalluser(false), dumpall(false), fixeddict(false),
		xsize(400), ysize(400),
		c_background(DEFAULT_BACKGROUND_COLOR),
		c_foreground(DEFAULT_FOREGROUND_COLOR),
		c_highlight(DEFAULT_HIGHLIGHT_COLOR),
		c_tag1(DEFAULT_TAG1_COLOR),
		c_tag2(DEFAULT_TAG2_COLOR),
		c_tag3(DEFAULT_TAG3_COLOR),
		c_tag4(DEFAULT_TAG4_COLOR),
		load_glibc_tags(false),
		load_posixh_tags(false),
		load_latex_tags(false),
		generate_local_tags(false),
		tags_cmd(QString("")),
		diff_x(600), diff_y(400),
		lang_ro(true),
		lang_en(false),
		beauty_bcl(true),
		beauty_tbcl(true),
		beauty_tab(true),
		sd(0),
		gd(0),
		rd(0),
		make_cmd_(QString("")),
		diff_dir1(QString::null),
		diff_dir2(QString::null) {
		
		readCfg();
		readAliasFile();
	}
	
	~Global() {
		saveCfg();
		saveAliasFile();
		clearGlobal();
	}

	static QString getCmd(const char *name);

private:
	void readCfg();
	void saveCfg();
	void readAliasFile();
	void saveAliasFile();

private:
	int getNumber(char *ptr);
	char *getCfgFname();
	char *getAliasFname();
	
public:
	// which command gives a different output on freebsd and linux
	// - on linux the output goes to stderr
	static bool checkWhich(const char *command);
	static char *getBackupFname(const char *fname2backup);
	static QString unixHomeConvert(const QString &fname);
};

#endif

