//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#include "latex.h"
#include "qwerty.h"
#include "briefedit.h"

static char *latex_keywords[] = {
	"abbrv",
	"abovedisplayshortskip",
	"abovedisplayskip",
	"abstract",
	"abstract",
	"abstractname",
	"acute",
	"addcontentsline",
	"address",
	"addtime",
	"addtocontents",
	"addtocounter",
	"addtolength",
	"addvspace",
	"align",
	"alph",
	"Alph",
	"alpha",
	"amsmath",
	"amsthm",
	"and",
	"appendix",
	"appendixname",
	"arabic",
	"array",
	"arraycolsep",
	"arrayrulewidth",
	"arraystretch",
	"article",
	"author",
	"a4paper",
	"a5paper",
	"backmatter",
	"bar",
	"bar",
	"baselineskip",
	"baselinestretch",
	"batchmode",
	"begin",
	"belowdisplayshortskip",
	"belowdisplayskip",
	"bezier",
	"bf",
	"bfseries",
	"bibindent",
	"bibitem",
	"bibliography",
	"bibliographystyle",
	"bibname",
	"big",
	"Big",
	"Bigg",
	"bigg",
	"Biggl",
	"biggl",
	"Biggm",
	"biggm",
	"Biggr",
	"biggr",
	"Bigl",
	"bigl",
	"bigm",
	"Bigm",
	"bigr",
	"Bigr",
	"bigskip",
	"bigskipamount",
	"binom",
	"blg",
	"boldmath",
	"boldsymbol",
	"book",
	"botfigrule",
	"bottmofraction",
	"bottomnumber",
	"boxedminipage",
	"bp",
	"breve",
	"b5paper",
	"calc",
	"calc",
	"caption",
	"caption2",
	"capt-of",
	"cases",
	"cc",
	"ccaption",
	"ccname",
	"cdotscenter",
	"centering",
	"cercle",
	"cfrac",
	"changebar",
	"chapter",
	"chapterbib",
	"chaptername",
	"check",
	"cite",
	"cleardoublepage",
	"clearpage",
	"cline",
	"clock",
	"closing",
	"cm",
	"COLON",
	"columnsep",
	"columnseprule",
	"columnwidth",
	"contentsline",
	"contentsname",
	"copyright",
	"dag",
	"dashbox",
	"date",
	"dbinom",
	"dblfigure",
	"dblfloatpage",
	"dblfloatsep",
	"dbltextfloatsep",
	"dbltopfraction",
	"dbltopnumber",
	"dcolumn",
	"dd",
	"ddag",
	"ddot",
	"ddots",
	"DeclareMathOperator",
	"depth",
	"description",
	"dfrac",
	"displaylimits",
	"displaymath",
	"displaystyle",
	"document",
	"documentclass",
	"dot",
	"dotfill",
	"doublerulesep",
	"downbracefill",
	"draft",
	"dropping",
	"dywiz",
	"em",
	"emph",
	"empty",
	"encl",
	"enclname",
	"end",
	"endfloat",
	"enlargethispage",
	"enskip",
	"enspace",
	"ensuremath",
	"enumerate",
	"enumi",
	"enumii",
	"enumiii",
	"enumiv",
	"eqnarray",
	"equation",
	"errorstopmode",
	"eucal",
	"eufrak",
	"evensidemargin",
	"everyship",
	"ex",
	"executivepaper",
	"expdlist",
	"extracolsep",
	"extramark",
	"fancybox",
	"fancyhdr",
	"fbox",
	"fboxrule",
	"fboxsep",
	"figure",
	"figurename",
	"file",
	"filecontents",
	"final",
	"flafter",
	"fleqn",
	"floatflt",
	"floatpagefraction",
	"floatsep",
	"flushbottom",
	"flushleft",
	"flushright",
	"fnpara",
	"fnsymbol",
	"fn2end",
	"fontenc",
	"footheight",
	"footmisc",
	"footnote",
	"footnotemark",
	"footnoterule",
	"footnotesep",
	"footnotesize",
	"footnotetext",
	"footnpag",
	"footskip",
	"frac",
	"frame",
	"framebox",
	"frenchspacing",
	"frontmatter",
	"ftnright",
	"fussy",
	"gather",
	"genfrac",
	"geometry",
	"glossary",
	"glossaryentry",
	"graphicx",
	"graphpaper",
	"grave",
	"hat",
	"hbox",
	"headheihgt",
	"headings",
	"headsep",
	"height",
	"helvet",
	"hfill",
	"hhline",
	"hline",
	"hrulefill",
	"hspace",
	"huge",
	"Huge",
	"HUGE",
	"hyperref",
	"hyphenation",
	"ifthen",
	"in",
	"include",
	"includeonly",
	"indent",
	"indentfirst",
	"index",
	"indexentry",
	"indexname",
	"indexspace",
	"input",
	"inputenc",
	"intertext",
	"intextsep",
	"invisible",
	"it",
	"item",
	"itemindent",
	"itemize",
	"itemsep",
	"itshape",
	"jot",
	"kill",
	"label",
	"labelenumi",
	"labelenumii",
	"labelenumiii",
	"labelenumiv",
	"labelitemi",
	"labelitemii",
	"labelitemiii",
	"labelitemiv",
	"labelsep",
	"labelwidth",
	"landscape",
	"large",
	"LARGE",
	"Large",
	"LaTeX",
	"LaTeXe",
	"latexsym",
	"ldots",
	"left",
	"leftarrowfill",
	"lefteqn",
	"leftmargin",
	"leftmargini",
	"leftmarginii",
	"leftmarginiii",
	"leftmarginiv",
	"leftmarginv",
	"leftmarginvi",
	"leftmark",
	"legalpaper",
	"leq",
	"leqno",
	"letter",
	"letterpaper",
	"letterspace",
	"lhead",
	"limits",
	"line",
	"linebreak",
	"linethickness",
	"linewidth",
	"list",
	"listfigurename",
	"listfiles",
	"listoffigures",
	"listoftables",
	"listparindent",
	"location",
	"longtable",
	"lq",
	"lrbox",
	"lscape",
	"mainmatter",
	"makeatletter",
	"makeatother",
	"makebox",
	"makeglossary",
	"makeidx",
	"makeindex",
	"makelabel",
	"maketitle",
	"manyfoot",
	"marginpar",
	"marginparpush",
	"marginparsep",
	"marginparwidth",
	"markboth",
	"markleft",
	"markright",
	"math",
	"mathbb",
	"mathbf",
	"mathbin",
	"mathcal",
	"mathclose",
	"mathfrak",
	"mathindent",
	"mathit",
	"mathnormal",
	"mathop",
	"mathopen",
	"mathord",
	"mathpunct",
	"mathrel",
	"mathrm",
	"mathscr",
	"mathsf",
	"mathstrut",
	"mathtt",
	"mathversion",
	"mbox",
	"mdseries",
	"medmuskip",
	"medskip",
	"medskipamount",
	"minipage",
	"minitoc",
	"minus",
	"mkern",
	"mm",
	"moreverbatim",
	"mpfootnote",
	"mu",
	"multicol",
	"multicolumn",
	"multilanguage",
	"multiput",
	"multirow",
	"myheadings",
	"nabla",
	"name",
	"NeedsTeXFormat",
	"newcommand",
	"newcounter",
	"newenvironment",
	"newfont",
	"newlength",
	"newline",
	"newpage",
	"newsavebox",
	"newtheorem",
	"nocite",
	"nofiles",
	"noindent",
	"nolimits",
	"nolinebreak",
	"nomathsymbols",
	"nonfrenchspacing",
	"nonumber",
	"nopagebreak",
	"normalfont",
	"normalsize",
	"not",
	"notag",
	"note",
	"notitlepage",
	"nu",
	"numberline",
	"numline",
	"numprint",
	"oddsidemargin",
	"oldstyle",
	"onecolumn",
	"oneside",
	"onlynotes",
	"onlyslides",
	"openany",
	"openbib",
	"opening",
	"openright",
	"operatorname",
	"oval",
	"overbrace",
	"overlay",
	"overleftarrow",
	"overline",
	"overrightarrow",
	"page",
	"pagebreak",
	"pagenumbering",
	"pageref",
	"pagestyle",
	"paperheight",
	"paperwidth",
	"par",
	"paragraph",
	"parbox",
	"parbox",
	"parindent",
	"parsep",
	"parskip",
	"part",
	"partial",
	"partname",
	"partopsep",
	"pauza",
	"pc",
	"phi",
	"pi",
	"picture",
	"plain",
	"PLdateending",
	"plmath",
	"PLSlash",
	"plus",
	"pmb",
	"pmod",
	"polski",
	"polski",
	"poptabs",
	"pounds",
	"ppauza",
	"prefixing",
	"printindex",
	"protect",
	"providecommand",
	"ps",
	"pt",
	"pushtabs",
	"put",
	"qbezier",
	"qbeziermax",
	"qquad",
	"quad",
	"quotation",
	"quote",
	"raggedbottom",
	"raggedleft",
	"raggedright",
	"ragged2e",
	"raisebox",
	"ratio",
	"real",
	"ref",
	"refname",
	"refstepcounter",
	"relsize",
	"renewcommand",
	"renewenvironment",
	"report",
	"reversemarginpar",
	"rhead",
	"right",
	"rightarrowfill",
	"rightmargin",
	"rightmark",
	"rm",
	"rmfamily",
	"roman",
	"Roman",
	"rotate",
	"rotating",
	"rq",
	"rule",
	"samepage",
	"savebox",
	"sb",
	"sbox",
	"sc",
	"scriptscriptstyle",
	"scriptsize",
	"scriptstyle",
	"scrollmode",
	"scshape",
	"secnumdepth",
	"section",
	"sectionmark",
	"see",
	"seename",
	"selectfont",
	"selectlanguage",
	"setcounter",
	"setlength",
	"settime",
	"settodepth",
	"settoheight",
	"settowidth",
	"sf",
	"sffamily",
	"shadethm",
	"shadow",
	"shapepar",
	"shortstack",
	"showlabels",
	"sidecap",
	"signature",
	"sin",
	"sl",
	"slide",
	"slides",
	"sloppy",
	"sloppybar",
	"slshape",
	"small",
	"smallskip",
	"smallskipamount",
	"soul",
	"sp",
	"space",
	"sqrt",
	"ss",
	"SS",
	"stackrel",
	"startbreaks",
	"stepcounter",
	"stop",
	"stopbreaks",
	"stretch",
	"strut",
	"subfigure",
	"subfigure",
	"subitem",
	"subparagraph",
	"subsection",
	"subsubitem",
	"subsubsection",
	"sum",
	"supressfloats",
	"symbol",
	"symbol",
	"tabbing",
	"tabcolsep",
	"table",
	"tablename",
	"tableofcontents",
	"tabular",
	"tabularx",
	"tag",
	"tan",
	"tbinom",
	"telephone",
	"TeX",
	"textbf",
	"textbullet",
	"textcircled",
	"textcompwordmark",
	"textemdash",
	"textendash",
	"textexclamdown",
	"textfloatsep",
	"textfraction",
	"textheight",
	"textit",
	"textmd",
	"textnormal",
	"textperiodcenter",
	"textquestiondown",
	"textquotedblleft",
	"textquotedblright",
	"textquoteleft",
	"textquoteright",
	"textrm",
	"textsc",
	"textsf",
	"textsl",
	"textstyle",
	"textsuperscript",
	"texttt",
	"textup",
	"textvisiblespace",
	"textwidth",
	"tfrac",
	"thanks",
	"the",
	"thebibliography",
	"theindex",
	"theorem",
	"thepage",
	"thesection",
	"theta",
	"thicklines",
	"thickmuskip",
	"thinlines",
	"thispagestyle",
	"tilde",
	"time",
	"times",
	"tiny",
	"title",
	"titlepage",
	"tocdepth",
	"today",
	"topfigrule",
	"topfraction",
	"topmargin",
	"topmargin",
	"topmargin",
	"topsep",
	"topskip",
	"topskip",
	"totalheight",
	"totalnumber",
	"trivlist",
	"tt",
	"ttfamily",
	"twocolumn",
	"twocolumn",
	"twoside",
	"typein",
	"typein",
	"typeout",
	"typeout",
	"ulem",
	"ulem",
	"unboldmath",
	"underbrace",
	"underline",
	"unsort",
	"unsrt",
	"upbracefill",
	"upshape",
	"upshape",
	"usebox",
	"usebox",
	"usecounter",
	"usefont",
	"usepackage",
	"value",
	"vbox",
	"vdots",
	"vec",
	"vector",
	"verb",
	"verb",
	"verbatim",
	"verse",
	"vfill",
	"visible",
	"vline",
	"vmargin",
	"voffset",
	"vspace",
	"widehat",
	"widetilde",
	"width",
	"wrapfig",
	"xleftarrow",
	"xrightarrow",
	"threeparttable",
	"addprefix",
	"addunit",
	"ampere",
	"amperemetresecond",
	"amperepermetre",
	"amperepermetrenp",
	"amperepersquaremetre",
	"amperepersquaremetrenp",
	"angstrom",
	"arad",
	"arcminute",
	"arcsecond",
	"are",
	"atomicmass",
	"atto",
	"attod",
	"barn",
	"bbar",
	"becquerel",
	"becquerelbase",
	"bel",
	"candela",
	"candelapersquaremetre",
	"candelapersquaremetrenp",
	"celsius",
	"Celsius",
	"celsiusbase",
	"centi",
	"centid",
	"coulomb",
	"coulombbase",
	"coulombpercubicmetre",
	"coulombpercubicmetrenp",
	"coulombperkilogram",
	"coulombperkilogramnp",
	"coulombpermol",
	"coulombpermolnp",
	"coulombpersquaremetre",
	"coulombpersquaremetrenp",
	"cubed",
	"cubic",
	"cubicmetre",
	"cubicmetreperkilogram",
	"cubicmetrepersecond",
	"curie",
	"dday",
	"deca",
	"decad",
	"deci",
	"decid",
	"degree",
	"degreecelsius",
	"deka",
	"dekad",
	"derbecquerel",
	"dercelsius",
	"dercoulomb",
	"derfarad",
	"dergray",
	"derhenry",
	"derhertz",
	"derjoule",
	"derkatal",
	"derlumen",
	"derlux",
	"dernewton",
	"derohm",
	"derpascal",
	"derradian",
	"dersiemens",
	"dersievert",
	"dersteradian",
	"dertesla",
	"dervolt",
	"derwatt",
	"derweber",
	"electronvolt",
	"exa",
	"exad",
	"farad",
	"faradbase",
	"faradpermetre",
	"faradpermetrenp",
	"femto",
	"femtod",
	"fourth",
	"gal",
	"giga",
	"gigad",
	"gram",
	"graybase",
	"graypersecond",
	"graypersecondnp",
	"hectare",
	"hecto",
	"hectod",
	"henry",
	"henrybase",
	"henrypermetre",
	"henrypermetrenp",
	"hertz",
	"hertzbase",
	"hour",
	"joule",
	"joulebase",
	"joulepercubicmetre",
	"joulepercubicmetrenp",
	"jouleperkelvin",
	"jouleperkelvinnp",
	"jouleperkilogram",
	"jouleperkilogramkelvin",
	"jouleperkilogramkelvinnp",
	"jouleperkilogramnp",
	"joulepermole",
	"joulepermolekelvin",
	"joulepermolekelvinnp",
	"joulepermolenp",
	"joulepersquaremetre",
	"joulepersquaremetrenp",
	"joulepertesla",
	"jouleperteslanp",
	"katal",
	"katalbase",
	"katalpercubicmetre",
	"katalpercubicmetrenp",
	"kelvin",
	"kilo",
	"kilod",
	"kilogram",
	"kilogrammetrepersecond",
	"kilogrammetrepersecondnp",
	"kilogrammetrepersquaresecond",
	"kilogrammetrepersquaresecondnp",
	"kilogrampercubicmetre",
	"kilogrampercubicmetrecoulomb",
	"kilogrampercubicmetrecoulombnp",
	"kilogrampercubicmetrenp",
	"kilogramperkilomole",
	"kilogramperkilomolenp",
	"kilogrampermetre",
	"kilogrampermetrenp",
	"kilogrampersecond",
	"kilogrampersecondcubicmetre",
	"kilogrampersecondcubicmetrenp",
	"kilogrampersecondnp",
	"kilogrampersquaremetre",
	"kilogrampersquaremetrenp",
	"kilogrampersquaremetresecond",
	"kilogrampersquaremetresecondnp",
	"kilogramsquaremetre",
	"kilogramsquaremetrenp",
	"kilogramsquaremetrepersecond",
	"kilogramsquaremetrepersecondnp",
	"kilowatthour",
	"liter",
	"litre",
	"lumen",
	"lumenbase",
	"lux",
	"luxbase",
	"mega",
	"megad",
	"meter",
	"metre",
	"metrepersecond",
	"metrepersecondnp",
	"metrepersquaresecond",
	"metrepersquaresecondnp",
	"micro",
	"microd",
	"milli",
	"millid",
	"minute",
	"mole",
	"molepercubicmetre",
	"molepercubicmetrenp",
	"nano",
	"nanod",
	"neper",
	"newton",
	"newtonbase",
	"newtonmetre",
	"newtonpercubicmetre",
	"newtonpercubicmetrenp",
	"newtonperkilogram",
	"newtonperkilogramnp",
	"newtonpermetre",
	"newtonpermetrenp",
	"newtonpersquaremetre",
	"newtonpersquaremetrenp",
	"NoAMS",
	"no@qsk",
	"ohm",
	"ohmbase",
	"ohmmetre",
	"one",
	"paminute",
	"pascal",
	"pascalbase",
	"pascalsecond",
	"pasecond",
	"per",
	"period@active",
	"persquaremetresecond",
	"persquaremetresecondnp",
	"peta",
	"petad",
	"pico",
	"picod",
	"power",
	"@qsk",
	"quantityskip",
	"rad",
	"radian",
	"radianbase",
	"radianpersecond",
	"radianpersecondnp",
	"radianpersquaresecond",
	"radianpersquaresecondnp",
	"reciprocal",
	"rem",
	"roentgen",
	"rp",
	"rpcubed",
	"rpcubic",
	"rpcubicmetreperkilogram",
	"rpcubicmetrepersecond",
	"rperminute",
	"rpersecond",
	"rpfourth",
	"rpsquare",
	"rpsquared",
	"rpsquaremetreperkilogram",
	"second",
	"siemens",
	"siemensbase",
	"sievert",
	"sievertbase",
	"square",
	"squared",
	"squaremetre",
	"squaremetrepercubicmetre",
	"squaremetrepercubicmetrenp",
	"squaremetrepercubicsecond",
	"squaremetrepercubicsecondnp",
	"squaremetreperkilogram",
	"squaremetrepernewtonsecond",
	"squaremetrepernewtonsecondnp",
	"squaremetrepersecond",
	"squaremetrepersecondnp",
	"squaremetrepersquaresecond",
	"squaremetrepersquaresecondnp",
	"steradian",
	"steradianbase",
	"tera",
	"terad",
	"tesla",
	"teslabase",
	"ton",
	"tonne",
	"unit",
	"unitskip",
	"usk",
	"volt",
	"voltbase",
	"voltpermetre",
	"voltpermetrenp",
	"watt",
	"wattbase",
	"wattpercubicmetre",
	"wattpercubicmetrenp",
	"wattperkilogram",
	"wattperkilogramnp",
	"wattpermetrekelvin",
	"wattpermetrekelvinnp",
	"wattpersquaremetre",
	"wattpersquaremetrenp",
	"wattpersquaremetresteradian",
	"wattpersquaremetresteradiannp",
	"weber",
	"weberbase",
	"yocto",
	"yoctod",
	"yotta",
	"yottad",
	"zepto",
	"zeptod",
	"zetta",
	"zettad",
	0		// null terminated
};

void loadLatexKeywords(QString fname, Dictionaries *pdict) {
	static bool done = false;
	ASSERT(pdict != 0);
	
	if (done)
		return;
	
	LOG("loadLatexKeywords for file %s", (const char *) fname);

	for (int i = 0; latex_keywords[i] != 0; i++)
		pdict->getIgnore()->setString(latex_keywords[i]);
	done = true;
}

void Editor::addLatextMenu() {
	ASSERT(m_ != 0);

	Q3PopupMenu * latex_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR( latex_ );
	m_->insertItem( "&LaTeX", latex_);

	Q3PopupMenu *templates = new Q3PopupMenu(latex_);
	templates->insertItem( "Article", this, SLOT(latexT1()));
	templates->insertItem( "Report", this, SLOT(latexT2()));
	templates->insertItem( "Letter", this, SLOT(latexT3()));
	templates->insertItem( "USENIX Article", this, SLOT(latexT4()));
	latex_->insertItem( "Templates" ,  templates);

	Q3PopupMenu *fs = new Q3PopupMenu(latex_);
	fs->insertItem( "tiny", this, SLOT(latexFS1()));
	fs->insertItem( "scriptsize", this, SLOT(latexFS2()));
	fs->insertItem( "footnote", this, SLOT(latexFS3()));
	fs->insertItem( "small", this, SLOT(latexFS4()));
	fs->insertItem( "normal", this, SLOT(latexFS5()));
	fs->insertItem( "large", this, SLOT(latexFS6()));
	fs->insertItem( "Large", this, SLOT(latexFS7()));
	fs->insertItem( "LARGE", this, SLOT(latexFS8()));
	fs->insertItem( "huge", this, SLOT(latexFS9()));
	fs->insertItem( "Huge", this, SLOT(latexFS10()));
	latex_->insertItem( "Font Size" ,  fs);

	Q3PopupMenu *ft = new Q3PopupMenu(latex_);
	ft->insertItem( "normal", this, SLOT(latexFT5()));
	ft->insertItem( "bold", this, SLOT(latexFT2()), Qt::CTRL+Qt::Key_B);
	ft->insertItem( "italic", this, SLOT(latexFT3()), Qt::CTRL+Qt::Key_I);
	ft->insertItem( "underline", this, SLOT(latexFT1()));
	ft->insertItem( "typewriter", this, SLOT(latexFT4()), Qt::CTRL+Qt::Key_T);
	latex_->insertItem( "Font Type" ,  ft);

	latex_->insertSeparator();
	latex_->insertItem( "Chapter", this, SLOT(latexF3()));
	latex_->insertItem( "Section", this, SLOT(latexF9()));
	latex_->insertItem( "Subsection", this, SLOT(latexF10()));

	latex_->insertSeparator();
	latex_->insertItem( "Bulleted List", this, SLOT(latexF1()));
	latex_->insertItem( "Item List", this, SLOT(latexF6()));
	latex_->insertItem( "Table", this, SLOT(latexF2()));
	latex_->insertItem( "Source Code", this, SLOT(latexF4()));
	latex_->insertItem( "Equation", this, SLOT(latexF5()));
	latex_->insertItem( "Picture", this, SLOT(latexF11()));
	latex_->insertItem( "End Note", this, SLOT(latexF12()));

	latex_->insertSeparator();
	latex_->insertItem( "New Page", this, SLOT(latexF7()));
	latex_->insertItem( "Indent", this, SLOT(latexF8()));
}

// article template based on kile.sourceforge.net
void Editor::latexT1() {
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = ""
"\\documentclass[a4paper,10pt]{article}\n"
"\n"
"\n"
"%opening\n"
"\\title{A Simple LaTeX Article}\n"
"\\author{Unknown Author}\n"
"\n"
"\\begin{document}\n"
"\n"
"\\maketitle\n"
"\n"
"\\begin{abstract}\n"
"This is a very short abstract.\n"
"\\end{abstract}\n"
"\n"
"\\section{First Section}\n"
"This is the first section. Enter your text here.\n"
"\\section{Second Section}\n"
"This is the second section. Enter your text here.\n"
"\\end{document}\n";

	e_->insertAt(str, para, index, false);
}


// report template
void Editor::latexT2() {
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = ""
"\\documentclass[a4paper,10pt]{report}\n"
"\n"
"\n"
"% Title Page\n"
"\\title{A Simple LaTeX Report}\n"
"\\author{Unknown Author}\n"
"\n"
"\n"
"\\begin{document}\n"
"\\maketitle\n"
"\n"
"\\begin{abstract}\n"
"This is a short abstract.\n"
"\\end{abstract}\n"
"\n"
"\\chapter{\\TeX}\n"
"\\section{Introduction}\n"
"\\subsection{History}\n"
"\n"
"\\chapter{\\LaTeX}\n"
"\\section{Introduction}\n"
"\\subsection{Usage}\n"
"\n"
"\\begin{thebibliography}{9}\n"
"  % type bibliography here\n"
"\\end{thebibliography}\n"
"\n"
"\\end{document}\n";

	e_->insertAt(str, para, index, false);
}

// letter template
void Editor::latexT3() {
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = ""
"\\documentclass[a4paper,10pt]{letter}\n"
"\n"
"\n"
"% If you want headings on subsequent pages,\n"
"% remove the ``%'' on the next line:\n"
"% \\pagestyle{headings}\n"
"\n"
"\\signature{your name}\n"
"\\address{\n"
"Your name\\\\\n"
"your address1\\\\\n"
"your address2\\\\\n"
"your address3\\\\\n"
"}\n"
"\n"
"\\begin{document}\n"
"\\begin{letter}{\n"
"Receivers name\\\\\n"
"Receivers address1\\\\\n"
"Receivers address2\\\\\n"
"Receivers address3\\\\\n"
"}\n"
"\n"
"\\opening{Regarding}\n"
"\n"
"\n"
"\\closing{Syncerely,}\n"
"\n"
"%add enclosure listing here\n"
"%\\encl{}\n"
"\n"
"\\end{letter}\n"
"\\end{document}\n";

	e_->insertAt(str, para, index, false);
} 

// USENIX article template - grab it from data install directory
void Editor::latexT4() {
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	
	// read the two files
	char *str1 = UTILS::read_file_malloc(DF6); // usenix.sty
	char *str2 = UTILS::read_file_malloc(DF7); // usenix.tex
	// log an error if necessary
	if (str1 == 0 || str2 == 0) {
		QMessageBox::warning(this,"I/O Info", QString("Cannot find usenix.* files in data instalation directory!\n"));
		msgScreen("USENIX template not loaded\n", 10000);
		return;
	}
	
	// save the style file
	bool retval = UTILS::write_file("./usenix.sty", str1);
	if (!retval) {
		QMessageBox::warning(this,"I/O Info", QString("Cannot write usenix.sty file! Probabely your directory is read only!\n"));
		msgScreen("USENIX template not loaded\n", 10000);
		delete str1;
		delete str2;
		return;
	}

	// load tex template
	QString str = str2;
	e_->insertAt(str, para, index, false);
	delete str1;
	delete str2;
}




QString latex2utf8(QString txt) {
	Q3CString utxt = txt.utf8();
	const char *ctext = utxt;

	char *ptr1 = UTILS::buffer_sed((char *) ctext, "\\u{a}", "ă");
	char *ptr2 = UTILS::buffer_sed(ptr1, "\\u{A}", "Ă");
	delete ptr1;
	char *ptr3 = UTILS::buffer_sed(ptr2, "\\^{i}", "î");
	delete ptr2;
	char *ptr4 = UTILS::buffer_sed(ptr3, "\\^{I}", "Î");
	delete ptr3;
	char *ptr5 = UTILS::buffer_sed(ptr4, "\\c{s}", "ş");
	delete ptr4;
	char *ptr6 = UTILS::buffer_sed(ptr5, "\\c{S}", "Ş");
	delete ptr5;
	char *ptr7 = UTILS::buffer_sed(ptr6, "\\c{t}", "ţ");
	delete ptr6;
	char *ptr8 = UTILS::buffer_sed(ptr7, "\\c{T}", "Ţ");
	delete ptr7;
	char *ptr9 = UTILS::buffer_sed(ptr8, "\\^{a}", "â");
	delete ptr8;
	char *ptr10 = UTILS::buffer_sed(ptr9, "\\^{A}", "Â");
	delete ptr9;

	QString retval = QString::fromUtf8(ptr10);
	delete ptr10;
	
	return retval;
}

QString utf82latex(QString txt) {
	Q3CString utxt = txt.utf8();
	const char *ctext = utxt;
	char *ptr1 = UTILS::buffer_sed((char *) ctext, "ă", "\\u{a}");
	char *ptr2 = UTILS::buffer_sed(ptr1, "Ă", "\\u{A}");
	delete ptr1;
	char *ptr3 = UTILS::buffer_sed(ptr2, "î", "\\^{i}");
	delete ptr2;
	char *ptr4 = UTILS::buffer_sed(ptr3, "Î", "\\^{I}");
	delete ptr3;
	char *ptr5 = UTILS::buffer_sed(ptr4, "ş", "\\c{s}");
	delete ptr4;
	char *ptr6 = UTILS::buffer_sed(ptr5, "Ş", "\\c{S}");
	delete ptr5;
	char *ptr7 = UTILS::buffer_sed(ptr6, "ţ", "\\c{t}");
	delete ptr6;
	char *ptr8 = UTILS::buffer_sed(ptr7, "Ţ", "\\c{T}");
	delete ptr7;
	char *ptr9 = UTILS::buffer_sed(ptr8, "â", "\\^{a}");
	delete ptr8;
	char *ptr10 = UTILS::buffer_sed(ptr9, "Â", "\\^{A}");
	delete ptr9;
	
	QString retval = QString::fromUtf8(ptr10);
	delete ptr10;
	
	return retval;
}

// table definition
void Editor::latexF2() {
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = ""
"\\begin{tabular}{l|l}\n"
"\t\\hline \\emph{quality} & \\emph{opposed to} \\\\\n"
"\t\\hline\\hline good & bad\\\\\n"
"\t\\hline happy & sad\\\\\n"
"\t\\hline\\hline\n"
"\\end{tabular}\n";

	e_->insertAt(str, para, index, false);
}

// bullet list
void Editor::latexF1() {
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = ""
"\\begin{itemize}\n"
"\t\\item red\n"
"\t\\item green\n"
"\t\\item blue\n"
"\\end{itemize}\n";

	e_->insertAt(str, para, index, false);
}

// picture
void Editor::latexF11() {
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = ""
"\\begin{figure}\n"
"[hbp]   \\centering\n"
"        \\includegraphics[width=200pt]{picture.eps}\n"
"        \\caption{Avidmux editor}\n"
"        \\label{Fig1:}\n"
"\\end{figure}\n";

	e_->insertAt(str, para, index, false);
}

// chapter
void Editor::latexF3() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\chapter{" + text + "}";
	e_->insertAt(str, para, index, false);
}
// section
void Editor::latexF9() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\section{" + text + "}";
	e_->insertAt(str, para, index, false);
}

// subsection
void Editor::latexF10() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\subsection{" + text + "}";
	e_->insertAt(str, para, index, false);
}

// source code
void Editor::latexF4() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = ""
"{\\tt \\scriptsize\n"
"\\begin{verbatim}\n"
	+ text + "\n"
"\\end{verbatim}\n"
"}\n";
	e_->insertAt(str, para, index, false);
}

// equation
void Editor::latexF5() {
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = ""
"\\begin{equation}\n"
"math formula\n"
"\\end{equation}\n";

	e_->insertAt(str, para, index, false);
}

// item list
void Editor::latexF6() {
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = ""
"\\begin{enumerate}\n"
"\t\\item red\n"
"\t\\item green\n"
"\t\\item blue\n"
"\\end{enumerate}\n";

	e_->insertAt(str, para, index, false);
}

// new page
void Editor::latexF7() {
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\newpage";

	e_->insertAt(str, para, index, false);
}

// indent
void Editor::latexF8() {
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\indent";

	e_->insertAt(str, para, index, false);
}
// end note
void Editor::latexF12() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\endnote{" + text + "}";
	e_->insertAt(str, para, index, false);
}

// font sizes
void Editor::latexFS1() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\tiny{" + text + "}";
	e_->insertAt(str, para, index, false);
}
void Editor::latexFS2() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\scriptsize{" + text + "}";
	e_->insertAt(str, para, index, false);
}
void Editor::latexFS3() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\footnotesize{" + text + "}";
	e_->insertAt(str, para, index, false);
}
void Editor::latexFS4() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\small{" + text + "}";
	e_->insertAt(str, para, index, false);
}
void Editor::latexFS5() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\normalsize{" + text + "}";
	e_->insertAt(str, para, index, false);
}
void Editor::latexFS6() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\large{" + text + "}";
	e_->insertAt(str, para, index, false);
}
void Editor::latexFS7() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\Large{" + text + "}";
	e_->insertAt(str, para, index, false);
}
void Editor::latexFS8() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\LARGE{" + text + "}";
	e_->insertAt(str, para, index, false);
}
void Editor::latexFS9() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\huge{" + text + "}";
	e_->insertAt(str, para, index, false);
}
void Editor::latexFS10() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\Huge{" + text + "}";
	e_->insertAt(str, para, index, false);
}

// font underline
void Editor::latexFT1() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\underline{" + text + "}";
	e_->insertAt(str, para, index, false);
}
// font type bold
void Editor::latexFT2() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\textbf{" + text + "}";
	e_->insertAt(str, para, index, false);
}
// font type italic
void Editor::latexFT3() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\textit{" + text + "}";
	e_->insertAt(str, para, index, false);
}
// font typewriter
void Editor::latexFT4() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\texttt{" + text + "}";
	e_->insertAt(str, para, index, false);
}
// font normal
void Editor::latexFT5() {
	QString text = e_->markedText();
	e_->cut();
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString str = "\\textnormal{" + text + "}";
	e_->insertAt(str, para, index, false);
}
