//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#ifndef LOG_H
#define LOG_H

#ifndef ASSERT
	#include <assert.h>
	#define ASSERT assert
#endif

#include <stdio.h>
#include <stdarg.h>	// using v-functions
#include <stdlib.h>	// using exit
//#include <string.h>
//#include <unistd.h>
//#include <sys/types.h>

class LogNothing {
public:
	//**********************************************************
	// printf functions
	//**********************************************************
	static void print_error(const char *fmt, ...) {
		(void) fmt;
	}
	static void print(const char *fmt, ...) {
		(void) fmt;
	}
};



class LogPrintf {
public:
	//**********************************************************
	// printf functions
	//**********************************************************
	static void print_error(const char *fmt, ...) {
		printf("Error:");

		va_list args;
		va_start (args,fmt);
		vprintf(fmt, args);	//use vsprintf to print to a string
		va_end (args);

		printf("\n");
		fflush(0);
	}
	static void print(const char *fmt, ...) {
		printf("Log:");

		va_list args;
		va_start (args,fmt);
		vprintf(fmt, args);	//use vsprintf to print to a string
		va_end (args);

		printf("\n");
		fflush(0);
	}
};

class LogFile {
public:
	//**********************************************************
	// printf functions
	//**********************************************************
	static void print_error(const char *fmt, ...) {
		FILE *file = getFile();
		fprintf(file, "Error:");

		va_list args;
		va_start (args,fmt);
		vfprintf(file, fmt, args);	//use vsprintf to print to a string
		va_end (args);

		fprintf(file, "\n");
		fflush(0);
	}
	static void print(const char *fmt, ...) {
		FILE *file = getFile();
		fprintf(file, "Log:");

		va_list args;
		va_start (args,fmt);
		vfprintf(file, fmt, args);	//use vsprintf to print to a string
		va_end (args);

		fprintf(file, "\n");
		fflush(0);
	}
private:
	static void release() {
		getFile(true);
	}

	static FILE *getFile(bool del = false) {
		static FILE *pinstance = 0;

		if (del == false && pinstance == 0) {
			pinstance = fopen("debug.log", "w");
			if (pinstance == NULL) {
				printf("Error: Cannot open debug.log\n");
				exit(1);
			}
		} else if (del == true && pinstance == 0); // do nothing
		else if (del == false && pinstance != 0); // do nothing
		else {
			fclose(pinstance);
		}

//		ASSERT(pinstance != 0);
		return pinstance;
	}
};


#define LOG Log::print
#define LOG_ERROR Log::print_error


#endif //LOG_H
