//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#include <qapplication.h>
#include <qdesktopwidget.h>
#include "spell.h"
#include "qwerty.h"
#include "dialogs.h"

void clearGlobal() {
	if (Global::instance()->sd != 0)
		delete Global::instance()->sd;
	if (Global::instance()->gd != 0)
		delete Global::instance()->gd;
}

static void usage() {
	printf("NAME\n");
	printf("   rospell - UTF-8 programming text editor and speller.\n");
	printf("\n");
	printf("SYNOPSYS\n");
	printf("   rospell [options] filename\n");
	printf("\n");
	printf("DESCRIPTION\n");
	printf("   rospell is a UTF-8 text editor for programmers and general use. \n");
	printf("   It features syntax highlighting, a code beautifier for C and C++, and support for\n");
	printf("   ctags, grep, diff, and merge. It also includes a speller for English and Romanian.\n");
	printf("\n");
	printf("   To start the spelling from the current cursor position press Start Spelling. Use\n");
	printf("   ignore to jump over a word, or Add to add it to the local dictionary\n");
	printf("   in ~/.rospell/dictionary.user.\n");
	printf("\n");
	printf("   rospell supports UTF-8, ISO8859, and CP1250 character sets.\n");
	printf("   If the character set is not specified on the command line,\n");
	printf("   UTF-8 is assumed.\n");
	printf("\n");
	printf("   rospell uses ALT key to enter Romanian special characters as follows:\n");
	printf("\n");
	printf("       ALT-a, ALT-s, ALT-q, ALT-t, ALT-i\n");
	printf("\n");
	printf("OPTIONS\n");
	printf("   -v: installed version\n");
	printf("\n");
	printf("   -h, -?, --help: this help screen.\n");
	printf("\n");
	printf("   --utf8: use utf8 character set (default).\n");
	printf("\n");
	printf("   --iso8859-1, --latin1: use ISO8859-1 character set.\n");
	printf("\n");
	printf("   --iso8859-2, --latin2: use ISO8859-2 character set.\n");
	printf("\n");
	printf("   --iso8859-3, --latin3: use ISO8859-3 character set.\n");
	printf("\n");
	printf("   --iso8859-4, --latin4: use ISO8859-4 character set.\n");
	printf("\n");
	printf("   --iso8859-5, --cyrillic: use ISO8859-5 character set.\n");
	printf("\n");
	printf("   --iso8859-6, --arabic: use ISO8859-6 character set.\n");
	printf("\n");
	printf("   --iso8859-7, --greek: use ISO8859-7 character set.\n");
	printf("\n");
	printf("   --iso8859-8, --hebrew: use ISO8859-8 character set.\n");
	printf("\n");
	printf("   --iso8859-9, --latin5: use ISO8859-9 character set.\n");
	printf("\n");
	printf("   --iso8859-10, --latin6: use ISO8859-10 character set.\n");
	printf("\n");
	printf("   --iso8859-11: use ISO8859-11 character set.\n");
	printf("\n");
	printf("   --iso8859-12: use ISO8859-12 character set.\n");
	printf("\n");
	printf("   --iso8859-13: use ISO8859-13 character set.\n");
	printf("\n");
	printf("   --iso8859-14: use ISO8859-14 character set.\n");
	printf("\n");
	printf("   --iso8859-15: use ISO8859-15 character set.\n");
	printf("\n");
	printf("   --iso8859-16: use ISO8859-16 character set.\n");
	printf("\n");
	printf("   --cp1250: use CP1250 (Windows) character set.\n");
	printf("\n");
	printf("   (the following options are only available for UTF-8 character set)\n");
	printf("\n");
	printf("   -m: add all words to Romanian master dictionary\n");
	printf("\n");
	printf("   -e: add all words to English master dictionary\n");
	printf("\n");
	printf("   -u: add all words to the local user dictionary\n");
	printf("\n");
	printf("   -p: print all new words\n");
	printf("\n");
	printf("EXAMPLE\n");
	printf("   $ rospell file*.txt\n");
	printf("   - start the editor using UTF-8 character set\n");
	printf("\n");
	printf("   $ rospell --iso8859-2 file*.txt\n");
	printf("   - start the editor using ISO8859-2 character set\n");
	printf("\n");
	printf("   $ rospell -p file*.txt\n");
	printf("   - generate a list of new words form file*.txt\n");
	printf("\n");
	printf("AUTHOR\n");
	printf("   http://rospell.sourceforge.net\n");
	printf("\n");
}

static Editor *start(const char *fname, FILE_FORMAT format) {
	LOG("start");
	Editor *e = new Editor;
	e->setDefaultFormat(format);
	if (fname == 0) {
		LOG("start with no file");
		e->setFname(0, format);
	} else {
		e->load(fname);
	}

	bool isSmall =  qApp->desktop()->size().width() < 450
				 || qApp->desktop()->size().height() < 450;

	if (isSmall) {
		e->showMaximized();
	} else {
		LOG("main: set editor sizes %d %d", Global::instance()->xsize, Global::instance()->ysize);
//		e->resize(Global::instance()->xsize, Global::instance()->ysize);
		e->show();
	}

	return e;
}

int main( int argc, char **argv )
{
	Action *aptr = 0;
	int start_arg = 1;
	LOG("argc %d", argc);
	LOG("argv %s %s", argv[0], argv[1]);

	FILE_FORMAT format = FILE_UTF8;

	if (argc > 1) {
		FILE_FORMAT f;
		if ((f = ff_parse_arg(argv[1])) != FILE_FORMAT_MAX) {
			LOG("main: option %s", argv[1]);
			format = f;
			start_arg = 2;
		}
		else if (strcmp(argv[1], "-m") == 0) { // Romanian master dict
			aptr = new ActionAddAllMaster();
			Global::instance()->addallmaster = true;
			LOG("Global::addallmaster = true");
			start_arg = 2;
			if (argc <= 2) {
				usage();
				return 1;
			}
		} else if (strcmp(argv[1], "-e") == 0) { // English master dict
			aptr = new ActionAddAllMasterEnglish();
			Global::instance()->addallmasterenglish = true;
			LOG("Global::addallmasterenglish = true");
			start_arg = 2;
			if (argc <= 2) {
				usage();
				return 1;
			}
		} else if (strcmp(argv[1], "-u") == 0) {
			aptr = new ActionAddAllUser();
			Global::instance()->addalluser = true;
			start_arg = 2;
			if (argc <= 2) {
				usage();
				return 1;
			}
		} else if (strcmp(argv[1], "-p") == 0) {
			aptr = new ActionDumpAll();
			Global::instance()->dumpall = true;
			start_arg = 2;
			if (argc <= 2) {
				usage();
				return 1;
			}
		} else if (strcmp(argv[1], "-h") == 0 ||
				 strcmp(argv[1], "-?") == 0 ||
				 strcmp(argv[1], "--help") == 0) {
			usage();
			return 0;
		} else if (strcmp(argv[1], "-v") == 0) {
			printf("rospell version %s\n", VERSION);
			return 0;
		}
	}

	if (aptr != 0) {
		// addallmaster (-m) and addallmasterenglish (-e) will not load the master dictionary
		// dumpall (-p) will load the master dictionary
		// addalluser (-u) will load the master dictionary
		bool readmaster = !(Global::instance()->addallmaster || Global::instance()->addallmasterenglish) ||
				   Global::instance()->dumpall ||
				   Global::instance()->addalluser;
		Dictionaries *pdict = new Dictionaries(readmaster);
	
		if (!Global::instance()->addallmaster && !Global::instance()->addallmasterenglish) {
			readHomeDir(pdict);
		}
	
		for (int i = start_arg; i < argc; i++) {
			char *fstr = UTILS::read_file_malloc(argv[i]);
			if (fstr == 0)
				continue;
			fprintf(stderr, "Process file %s\n", argv[i]);
			LOG("Process file %s", argv[i]);
			char *ptmp = spell(pdict, aptr, fstr);
			delete[] ptmp;
		}
		delete aptr;
	
		if (Global::instance()->addallmaster || Global::instance()->addallmasterenglish) {
			pdict->saveMaster();
		}
		delete pdict;

		Stats::instance()->print();
		Stats::instance(true);
		if (Global::instance()->addallmaster ||
		    Global::instance()->addallmasterenglish ||
		    Global::instance()->addalluser ||
		    Global::instance()->dumpall) {
			return 0;
		}
	}
	QApplication a( argc, argv);

	int i;
	Editor *e = 0;
	if (argc == start_arg) {
		e = start(0, format);
	}
	else {
		e = start(argv[start_arg], format);
		start_arg++;
		for (i= start_arg; i<argc; i++) {
			e->load(argv[i]);
		}
	}
	if (e == 0) // just in case!
		return 1;
	a.connect( &a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()) );

	int retval = a.exec();
	delete e;
	Global::instance(true);
	Stats::instance(true);
	return retval;
//    return a.exec();
}

