//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#include "qwerty.h"
#include "briefedit.h"
#include "dialogs.h"
#include "qtutils.h"
#include "busycursor.h"
#include "toc.h"
#include "bufmgr.h"
#include "qtutils.h"
#include "filevisitor.h"
#include <qapplication.h>
#include <qinputdialog.h>
#include <qfiledialog.h>



// also called from BriefEdit class!
bool Editor::advanceSearchList(QString tofind, bool ignore_case, bool whole) {
	// save search params for searchAgain functionality
	search_ = tofind;
	ignore_case_ = ignore_case;
	whole_ = whole;
	backwards_ = false;

	int para;
	int index;
	e_->getCursorPosition(&para, &index);
//	int old_para = para;
//	int old_index = index;
	int lines = e_->numLines();

	if (para >= lines)
		return false;
	QString search_result = "";
	while (para < lines) {
		QString line = e_->textLine(para);
//		index = line.find(tofind, index, !ignore_case);
		index = QTUTILS::find(line, tofind, index, !ignore_case, whole);
		if (index != -1) {
			char str[50];
			sprintf(str, "%d: ", para + 1);
			search_result += str + line + "\n";
		}
		index = 0;
		para++;
	}
	
	
	tw_search_->setText(search_result);
	tw_->setCurrentPage(1); //lucian: hardcoded

	return true;	
}

bool Editor::advanceSearchListRev(QString tofind, bool ignore_case, bool whole) {
	// save search params for searchAgain functionality
	search_ = tofind;
	ignore_case_ = ignore_case;
	whole_ = whole;
	backwards_ = true;

	int para;
	int index;
	e_->getCursorPosition(&para, &index);
//	int old_para = para;
//	int old_index = index;

	if (para == 0 && index == 0)
		return false;
	QString search_result = "";
	bool update_index = false;
	while (para >= 0) {
		QString line = e_->textLine(para);
		if (update_index)
			index = line.length();
		index = QTUTILS::findRev(line, tofind, index, !ignore_case, whole);
		if (index != -1) {
			char str[50];
			sprintf(str, "%d: ", para + 1);
			search_result += str + line + "\n";
		}
		update_index = true;
		
		para--;
	}
	
	
	tw_search_->setText(search_result);
	tw_->setCurrentPage(1); //lucian: hardcoded

	return true;	
}



// also called from BriefEdit class!
bool Editor::advanceSearch(QString tofind, bool ignore_case, bool whole) {
	// save search params for searchAgain functionality
	search_ = tofind;
	ignore_case_ = ignore_case;
	whole_ = whole;
	backwards_ = false;
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	int lines = e_->numLines();

	if (para >= lines)
		return false;

	while (para < lines) {
		QString line = e_->textLine(para);
		index = QTUTILS::find(line, tofind, index, !ignore_case, whole);
		if (index != -1) {
			e_->setCursorPosition(para, index);
			e_->setSelection(para, index, para, index + tofind.length());
			e_->getCursorPosition(&para, &index);
			cursorPosChanged(para, index);
			e_->update();
			return true;
		}
		index = 0;
		para++;
	}
	return false;	
}

bool Editor::advanceSearchRev(QString tofind, bool ignore_case, bool whole) {
	// save search params for searchAgain functionality
	search_ = tofind;
	ignore_case_ = ignore_case;
	whole_ = whole;
	backwards_ = true;
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	
	index--;
	if (index <= 0) {
		para--;
		QString line = e_->textLine(para);
		index = line.length();
	}		
	if (para == 0 && index == 0) {
		return false;
	}
	
	bool update_index = false;
	while (para >= 0) {
		QString line = e_->textLine(para);
		if (update_index)
			index = line.length();
		index = QTUTILS::findRev(line, tofind, index, !ignore_case, whole);
		if (index != -1) {
			e_->setCursorPosition(para, index + tofind.length());
			e_->setSelection(para, index + tofind.length(), para, index);
			e_->getCursorPosition(&para, &index);
			cursorPosChanged(para, index);
			e_->update();
			return true;
		}
		update_index = true;
		para--;
	}
	e_->getCursorPosition(&para, &index);
	e_->setSelection(para, index, para, index);
	return false;	
}


void Editor::replace() {
	cleanMWords();
	QString tofind = "";
	QString replacement = "";
	ReplaceDialog *rd = 0;

	int para;
	int index;

	while (1) {
		e_->getCursorPosition(&para, &index);

		if (rd == 0) {
			rd = Global::instance()->rd;
			if (rd == 0) {
				rd = new ReplaceDialog(tofind, replacement);
				rd->setModal(false);
				rd->setCaption("Replace");
				Global::instance()->rd = rd;
			}
			rd->show();
		}

		tofind = rd->getWordToFind();
		replacement = rd->getReplacement();
		qApp->processEvents();
		volatile ReplaceDialog::ReplaceAction raction = rd->getAction();
		if (raction == ReplaceDialog::ACTION_FINDNEXT) {
			rd->printAction();
			rd->reset();

			if (!advanceSearch(tofind)) {
				msgScreen("End of file");
				break;
			}
		}
		else if (raction == ReplaceDialog::ACTION_REPLACE) {
			rd->printAction();
			rd->reset();
			e_->del();
			e_->insertAt(replacement, para, index - tofind.length(), false);
			e_->setCursorPosition(para, index - tofind.length() + replacement.length(), false);

			if (!advanceSearch(tofind)) {
				msgScreen("End of file");
				break;
			}
		}
		else if (raction == ReplaceDialog::ACTION_REPLACEALL) {
			rd->printAction();

			while (advanceSearch(tofind)) {
				e_->getCursorPosition(&para, &index);
				e_->del();
				e_->insertAt(replacement, para, index - tofind.length(), false);
				e_->setCursorPosition(para, index - tofind.length() + replacement.length(), false);
			}

			msgScreen("End of file");
			break;
		}
		else if (raction == ReplaceDialog::ACTION_CANCEL) {
			rd->printAction();
			break;
		}
	}
	rd->getOut();
}


void Editor::search() {
	cleanMWords();
	SearchDialog *sd = Global::instance()->sd;
	if (sd == 0) {
		sd = new SearchDialog();
		sd->setCaption("Enter search word");
		Global::instance()->sd = sd;
	}
	if (!sd->exec()) {
		return;
	}

	QString sword = sd->getWord();
	search_ = sword;
	ignore_case_ = sd->getIgnoreCase();
	whole_ = sd->getWhole();
	backwards_ = sd->getBackwards();
	
	if (sword == "") {
		msgScreen("no word entered");
		return;
	}
	Q3CString utext = sword.utf8();
	const char *cstr = utext;
	LOG("Editor::search %s", cstr);

	if (!backwards_) {
		advanceSearchList(sword, ignore_case_, whole_);		
		if (!advanceSearch(sword, ignore_case_, whole_)) {
			msgScreen("End of file");
		}
	}
	else {
		advanceSearchListRev(sword, ignore_case_, whole_);		
		if (!advanceSearchRev(sword, ignore_case_, whole_)) {
			msgScreen("Start of file");
		}
	}
}

void Editor::searchAgain() {
	cleanMWords();
	if (search_ == "")
		return;
	Q3CString utext = search_.utf8();
	const char *cstr = utext;
	LOG("Editor::searchAgain %s", cstr);

	if (!backwards_) {
		if (!advanceSearch(search_, ignore_case_, whole_)) {
			msgScreen("End of file");
		}
	}
	else {
		if (!advanceSearchRev(search_, ignore_case_, whole_)) {
			msgScreen("Start of file");
		}
	}
}

void Editor::grep() {
	LOG("qwerty grep");
	cleanMWords();


	GrepDialog *gd = Global::instance()->gd;
	if (gd == 0) {
		gd = new GrepDialog();
		gd->setCaption("Grep");
		Global::instance()->gd = gd;
	}
	if (!gd->exec()) {
		return;
	}

	QString sword = gd->getWord();
	if (sword.isEmpty())
		return;
	bool ignore_case = gd->getIgnoreCase();
	bool whole = gd->getWhole();
	bool recursive = gd->getRecursive();
	QString sdirectory = gd->getSdirectory();
	QString fpattern = gd->getFpattern();
	QString edirectory = gd->getEdirectory();


	
	BCursor busy;
	FVBase fvb(sdirectory, fpattern, edirectory, recursive);
	FVisitorGrep visitor(sword, !ignore_case, whole);
	QString txt = fvb.walk(&visitor);
	
	tw_search_->setText(txt);
	tw_->setCurrentPage(1); //lucian: hardcoded
	msgScreen("done", 2000);
#if 0	
	// check for grep presence
	if (!Global::checkWhich("grep"))
		return;

	// get grep string
 	bool ok;
	QString text = QInputDialog::getText("grep", "Enter grep string:", QLineEdit::Normal, QString::null, &ok, this );
	if ( ok && !text.isEmpty() );
	else {
		return;
	}
 		
	
	QString s = QFileDialog::getExistingDirectory("./", this, "get existing directory", "Choose a directory", TRUE );
	if (s.isNull() || s.isEmpty()) {
		return;
	}
	
	tw_search_->setText("");
	msgScreen("grep, please wait...");
	BCursor busy;
	
	// grab utf8 reprezentation
	Q3CString utext = text.utf8();
	const char *cstr = utext;
	
	QTUTILS::run_program(QString("grep -n -R ") + QString(cstr) + QString(" ") + s, tw_search_, "\ngrep done\n");
	tw_->setCurrentPage(1); //lucian: hardcoded
	msgScreen("done", 2000);
#endif
}

// state0 - start new text
//
// state0 ---;---> state1 - we have a full expression "int a;" or "int a = 7;"; also working for "typedef unsigned UINT32;"
//
// state0 ---{----> state2 - starting expression "int a = {"
// state2 ---{----> state2 - do {} matching
// state2 ---}----> state3 
// state3 ---;----> state4 - we have a full expression "int a = {};"; also working for "typedef struct list {struct list *next; int val;} LIST, *pLIST;"
//
// state0 ---(----> state5 - starting to match a function
// state5 ---)----> state6
// state6 ---;----> state0 - this was just a function declaration "int f();"; do not add it to results
// state6 ---{----> state7
// state7 ---{----> state7 - do {} matching
// state7 ---}----> state8 - we have a full ANSI function definition
// state6 ---any char but not ;---->state10 - we are dealing here with a non-ANSI function definition
// state10 ---{---> state11
// state11 ---{---> state11 - do {} matching
// state11 ---}---> state12 - we have a non-ANSI function definition
//
// state0 ---#---> state9 - we do not care for macros
// state9 ---\---> state9 - advance one line
// state9 ---end of string---->state 0
void Editor::toc() {
	cleanMWords();
	TocParser *parser = 0;
	
	// find the current filename
	QString fname = b_->getActiveFName();
	
	
	if (fname.isNull() || fname.isEmpty())
		return;
	
	// parser class factory
	if (fname.endsWith(".c") ||
	    fname.endsWith(".cpp") ||
	    fname.endsWith(".h") ||
	    fname.endsWith(".cc") ||
	    fname.endsWith(".js")) {
	
		LOG("use C/C++ parser");
		parser = new TocC();
	}
#if 0
	else if (fname.endsWith(".java")) {
		LOG("use java syntax highlighter");
		return new CBasedSyntaxHighlighter(textEdit, keyw_java);
	}
	else if (fname.endsWith(".js")) {
		LOG("use javascript syntax highlighter");
		return new CBasedSyntaxHighlighter(textEdit, keyw_javascript);
	}
	else if (fname.endsWith(".cs")) {
		LOG("use csharp syntax highlighter");
		return new CBasedSyntaxHighlighter(textEdit, keyw_csharp);
	}
	else if (fname.endsWith(".ph3") ||
	         fname.endsWith(".ph4") ||
	         fname.endsWith(".php") ||
	         fname.endsWith(".phps")) {
	
		LOG("use PHP syntax highlighter");
		return new CBasedSyntaxHighlighter(textEdit, keyw_php);
	}
	else if (fname.endsWith(".S") ||
	         fname.endsWith(".asm")) {
	
		LOG("use asm syntax highlighter");
		return new AsmSyntaxHighlighter(textEdit);
	}
	else if (fname.endsWith(".pro")) {
		LOG("use qmake syntax highlighter");
		return new ScriptBasedSyntaxHighlighter(textEdit, keyw_qmake);
	}
	else if (fname.endsWith(".pl")) {
		LOG("use perl syntax highlighter");
		return new ScriptBasedSyntaxHighlighter(textEdit, keyw_perl);
	}
	else if (fname.endsWith(".py")) {
		LOG("use python syntax highlighter");
		return new ScriptBasedSyntaxHighlighter(textEdit, keyw_python);
	}
	else if (fname.endsWith(".sh")) {
		LOG("use bash/shell syntax highlighter");
		return new ScriptBasedSyntaxHighlighter(textEdit, keyw_shell);
	}
	else if (fname.endsWith(".diff")) {
		LOG("use diff syntax highlighter");
		return new DiffSyntaxHighlighter(textEdit);
	}
#endif
	else if (fname.endsWith(".tex") || fname.endsWith(".sty")) {
		LOG("use latex parser");
		parser = new TocLatex();
	}
	
	if (parser != 0) {
		// parse the file
		QString parse_result = parser->parse(e_);
		tw_search_->setText(parse_result);
		tw_->setCurrentPage(1); //lucian: hardcoded
	}
	else {
		msgScreen("TOC not supported for this file type!", 5000);
	}
}
