//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************
//** $Id: qt/qwerty.cpp   3.3.4   edited May 27 2003 $
//**
//** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
//**
//** This file is part of an example program for Qt.  This example
//** program may be used, distributed and modified without limitation.
//**
//*****************************************************************************/
// rospell user interface
//*****************************************************************************/

// for development/debug purposes only
#define WORK_DEBUG


#include "qwerty.h"
#include <qapplication.h>
#include <qdesktopwidget.h>
#include <qdatetime.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qmessagebox.h>
#include <qfontdialog.h>
#include <qtextcodec.h>
#include "busycursor.h"
#include <qdir.h>

#ifdef USING_QT4
#include <q3filedialog.h>
#include <q3popupmenu.h>
#include <q3paintdevicemetrics.h>
#include <q3ptrlist.h>
//Added by qt3to4:
#include <Q3CString>
#include <QCloseEvent>
#include <QResizeEvent>
#include <Q3VBoxLayout>
//Added by qt3to4:
#include <QLabel>
#endif
#ifdef USING_QT3
#include <qfiledialog.h>
#include <qpaintdevicemetrics.h>
#endif
#include <qfileinfo.h>
#include <qinputdialog.h>
#include <qsplitter.h>
#include <qcombobox.h>
#include <qregexp.h>

#include "qtutils.h"
#include "spell.h"
#include "stats.h"
#include "dialogs.h"
#include "briefedit.h"
#include "lineandcolumn.h"

#include "latex.h"
#include "bufmgr.h"
#include "ctags.h"
#include "filevisitor.h"


//const bool no_writing = FALSE;

Editor::Editor( QWidget * parent , const char * name )
: Q3VBox(parent, name)
//: QWidget( parent, name, Qt::WDestructiveClose )
{
	LOG("before resize in constructor");
	resize(Global::instance()->xsize, Global::instance()->ysize);
	LOG("after resize in constructor");

	pdict_ = 0;
	search_ = "";
	default_format_ = FILE_UTF8;
	*word1_ = '\0';
	*word2_ = '\0';
	*word3_ = '\0';
	*word4_ = '\0';
	*word5_ = '\0';
	resize(Global::instance()->xsize, Global::instance()->ysize);

	//*****************************************************
	// accelerators
	//*****************************************************
	Q3Accel *a_ = new Q3Accel(this);
	// set bookmarks
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_1), this, SLOT(setBookmark1()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_2), this, SLOT(setBookmark2()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_3), this, SLOT(setBookmark3()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_4), this, SLOT(setBookmark4()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_5), this, SLOT(setBookmark5()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_6), this, SLOT(setBookmark6()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_7), this, SLOT(setBookmark7()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_8), this, SLOT(setBookmark8()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_9), this, SLOT(setBookmark9()));
	a_->connectItem(a_->insertItem(Qt::ALT+Qt::Key_0), this, SLOT(setBookmark0()));
	// user programable keys
	a_->connectItem(a_->insertItem(Qt::Key_F1), this, SLOT(userF1()));
	a_->connectItem(a_->insertItem(Qt::Key_F2), this, SLOT(userF2()));
	a_->connectItem(a_->insertItem(Qt::Key_F3), this, SLOT(userF3()));
	a_->connectItem(a_->insertItem(Qt::Key_F4), this, SLOT(userF4()));
	// debug
	a_->connectItem(a_->insertItem(Qt::Key_F10), this, SLOT(roDebug()));
#ifdef WORK_DEBUG
	a_->connectItem(a_->insertItem(Qt::Key_F11), this, SLOT(roDevel()));
#endif

	//*****************************************************
	// menubar
	//*****************************************************
	m_ = new QMenuBar( this, "menu" );
	// file
	Q3PopupMenu * file_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR( file_ );
	m_->insertItem( "&File", file_ );
	file_->insertItem( "&New Window",   this, SLOT(newDoc()));
	file_->insertItem( "New &Buffer",   this, SLOT(newDocBuffer()));
	file_->insertItem( "&Open...",  this, SLOT(load()), Qt::ALT+Qt::Key_E);
	file_->insertItem( "Open &Location",  this, SLOT(openWwwLocation()));
	file_->insertSeparator();
	file_->insertItem( "&Save...",  this, SLOT(save()), Qt::ALT+Qt::Key_W);
	file_->insertItem( "Sa&ve As",  this, SLOT(saveAs()));
	file_->insertItem( "Save &All",  this, SLOT(saveAll()));
	file_->insertSeparator();
	file_->insertItem( "&Close Buffer",  this, SLOT(closeBuffer()), Qt::ALT+Qt::Key_D );
	file_->insertItem( "E&xit", this, SLOT(close()), Qt::ALT+Qt::Key_X);
	file_->insertSeparator();
	file_->insertItem( "&Change format", this, SLOT(changeFormat()));
#ifndef ROSPELL_NO_PRINTER
	file_->insertItem( "&Print...", this, SLOT(print()), Qt::CTRL+Qt::Key_P );
#endif

	// edit
	Q3PopupMenu *edit_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR(edit_);
	m_->insertItem("Edit", edit_);
	edit_->insertItem( "&Undo", this, SLOT(undo()), Qt::ALT+Qt::Key_U);
	edit_->insertItem( "Toggle &Insert/Overwrite", this, SLOT(toggleOverwrite()));
	edit_->insertSeparator();
	edit_->insertItem( "Cut", this, SLOT(cut()), Qt::CTRL+Qt::Key_X);
	edit_->insertItem( "Copy", this, SLOT(copy()), Qt::CTRL+Qt::Key_C);
	edit_->insertItem( "Paste", this, SLOT(paste()), Qt::CTRL+Qt::Key_V);
	edit_->insertItem( "Delete", this, SLOT(e_delete()));
	edit_->insertItem( "Select All", this, SLOT(selectAll()));
	edit_->insertSeparator();
	edit_->insertItem( "&Find", this, SLOT(search()), Qt::Key_F5);
	edit_->insertItem( "Find Ne&xt", this, SLOT(searchAgain()), Qt::SHIFT+Qt::Key_F5);
	edit_->insertItem( "&Replace", this, SLOT(replace()), Qt::Key_F6);
	edit_->insertItem( "&Grep", this, SLOT(grep()), Qt::Key_F7);
	edit_->insertItem( "&TOC", this, SLOT(toc()), Qt::Key_F8);
	edit_->insertSeparator();
	edit_->insertItem( "Goto &Line", this, SLOT(gotoLine()), Qt::ALT+Qt::Key_G);
	edit_->insertItem( "&Start of File", this, SLOT(startOfBuffer()), Qt::CTRL+Qt::Key_PageUp);
	edit_->insertItem( "&End of File", this, SLOT(endOfBuffer()), Qt::CTRL+Qt::Key_PageDown);
	edit_->insertItem( "Set &Bookmark", this, SLOT(setBookmark()));
	edit_->insertItem( "Goto Boo&kmark", this, SLOT(gotoBookmark()), Qt::ALT+Qt::Key_J);
	edit_->insertSeparator();
	edit_->insertItem( "Insert Unicode Char", this, SLOT(insertHex()));

	// options
	Q3PopupMenu * config_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR( config_ );
	m_->insertItem( "Preferences", config_ );
	config_->insertItem("Select &Font", this, SLOT(font()));
	config_->insertItem("&Colors", this, SLOT(colors()));
	config_->insertItem("&Alias", this, SLOT(editAlias()));
	config_->insertItem("&Language", this, SLOT(configLanguage()));
	config_->insertItem("&WWW configuration", this, SLOT(configWWW()));
	Q3PopupMenu *user = new Q3PopupMenu(config_);
	user->insertItem( "Edit F&1 Script" ,   this, SLOT(editF1()));
	user->insertItem( "Edit F&2 Script" ,   this, SLOT(editF2()));
	user->insertItem( "Edit F&3 Script" ,   this, SLOT(editF3()));
	user->insertItem( "Edit F&4 Script" ,   this, SLOT(editF4()));
	config_->insertItem( "User &Scripts" ,  user);
	config_->insertSeparator();
	config_->insertItem("Cta&gs", this, SLOT(configCtags()));
	config_->insertItem("L&ock Ctags", this, SLOT(lockCtags()));
	config_->insertItem("&Unlock Ctags", this, SLOT(unlockCtags()));
	config_->insertSeparator();
	config_->insertItem("S&tatus", this, SLOT(status()));

	// buffers
	buffers_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR(buffers_);
	m_->insertItem("&Buffers", buffers_);
	connect(buffers_, SIGNAL(activated(int)), this, SLOT(pickBuffer(int)));
	buffers_->insertItem("&Next", this, SLOT(nextBuffer()), Qt::ALT+Qt::Key_N);
	buffers_->insertSeparator();

	// tools
	Q3PopupMenu * tools_ = new Q3PopupMenu(m_);
	Q_CHECK_PTR( tools_ );
	m_->insertItem( "T&ools", tools_ );
	tools_->insertItem( "&Record" , this, SLOT(recordAction()), Qt::ALT+Qt::Key_R);
	tools_->insertItem( "&Playback" , this, SLOT(playbackAction()), Qt::ALT+Qt::Key_P);
	tools_->insertSeparator();
	tools_->insertItem( "&Uppercase" ,   this, SLOT(uppercase()));
	tools_->insertItem( "&Lowercase" ,   this, SLOT(lowercase()));
	tools_->insertItem( "UNI&X CR/LF cleanup" ,   this, SLOT(unixCleanup()));
	tools_->insertSeparator();
	tools_->insertItem( "&dexonline" ,   this, SLOT(dexonline()));
	tools_->insertItem( "&Beautify" ,   this, SLOT(beautify()));
	tools_->insertItem( "&kloc" ,   this, SLOT(kloc()));
	tools_->insertItem( "m&ake" ,   this, SLOT(make()));
	tools_->insertSeparator();
	tools_->insertItem( "&Diff" ,   this, SLOT(diff()));
	tools_->insertItem( "&Session Diff" ,   this, SLOT(sessionDiff()));
	tools_->insertItem( "&Merge" ,   this, SLOT(merge()));

	// latex
	addLatextMenu();

	// help
	m_->insertSeparator();
	addHelpMenu();


	//*****************************************************
	// spelling menubar
	//*****************************************************
	Q3VBox *vbox = new Q3VBox(this);
	mwords_ = new QMenuBar(vbox, "words menu" );

	mwords_->insertItem( "Start Spelling",   this, SLOT(spellAdvance()));
	mwords_->insertItem( "Ignore",   this, SLOT(spellIgnore()));
	mwords_->insertItem( "Add",   this, SLOT(spellAdd()));
	if (Global::instance()->fixeddict)
		mwords_->insertItem( "Add na&me",   this, SLOT(spellName()),   Qt::ALT+Qt::Key_M );
	mw1_ = mwords_->insertItem( " ",   this, SLOT(replaceWord1()), 0 );
	mw2_ = mwords_->insertItem( " ",   this, SLOT(replaceWord2()), 0 );
	mw3_ = mwords_->insertItem( " ",   this, SLOT(replaceWord3()), 0 );
	mw4_ = mwords_->insertItem( " ",   this, SLOT(replaceWord4()), 0 );
	mw5_ = mwords_->insertItem( " ",   this, SLOT(replaceWord5()), 0 );



	
	// starting splitters!
	QSplitter *split1 = new QSplitter(vbox);
	split1->setOrientation(QSplitter::Vertical);
	QSplitter *split2 = new QSplitter(split1);
	
	//*****************************************************
	// directory window
	//*****************************************************
	Q3VBox *vdirbox = new Q3VBox(split2);
	vdirbox->setSpacing(1);
	dirfiles_size_ = 150;
	vdirbox->setMaximumWidth(dirfiles_size_);
	dircombo_ = new QComboBox(TRUE, vdirbox);
	dircombo_->insertItem("*");
	dircombo_->insertItem("*.asm,*.S");
	dircombo_->insertItem("*.c,*.cpp,*.h");
	dircombo_->insertItem("*.html");
	dircombo_->insertItem("*.java");
	dircombo_->insertItem("*.php");
	dircombo_->insertItem("*.pl");
	dircombo_->insertItem("*.py");
	dircombo_->insertItem("*.sh");
	dircombo_->insertItem("*.tex");
	
	dirfiles_ = new Q3MultiLineEdit(vdirbox);
	dirfiles_->setReadOnly(TRUE);
	dirfiles_path_ = "./";
	QString files = getDirfiles();
	dirfiles_->setText(files);
	connect(dirfiles_, SIGNAL(clicked(int, int)), this, SLOT(clickedDirfiles(int, int)));
	connect(dircombo_, SIGNAL(activated(int)), this, SLOT(dirboxChanged(int)));

	// colors
	dircombo_->setPaletteBackgroundColor(Global::instance()->c_background);
	dircombo_->setPaletteForegroundColor(Global::instance()->c_foreground);
	QPalette qp = dircombo_->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, Global::instance()->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, Global::instance()->c_highlight);
	dircombo_->setPalette(qp);
	dirfiles_->setPaletteBackgroundColor(Global::instance()->c_background);
	dirfiles_->setPaletteForegroundColor(Global::instance()->c_foreground);
	qp = dirfiles_->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, Global::instance()->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, Global::instance()->c_highlight);
	dirfiles_->setPalette(qp);

	
	//*****************************************************
	// first editor
	//*****************************************************
	QSplitter *split3 = new QSplitter(split2);
	split3->setOrientation(QSplitter::Vertical);
	emain_ = new BriefEdit(this, split3, "editor" );
	// get default XML syntax
	getSyntax("", emain_);
	connect( emain_, SIGNAL( textChanged() ), this, SLOT( textChanged() ) );
	connect( emain_, SIGNAL( clicked(int, int) ), this, SLOT( clicked(int, int) ) );
	emain_->setWordWrap(Q3TextEdit::NoWrap);
	b_main_ = new BufMgr();
	b_main_->dbgname_ = "main";
	changed_main_ = false;
	// active window setting
	using_aux_ = false;
	changed_main_ = false;
	e_ = emain_;
	b_ = b_main_;
	book_ = book_main_;
	
	// colors
	emain_->setPaletteBackgroundColor(Global::instance()->c_background);
	emain_->setPaletteForegroundColor(Global::instance()->c_foreground);
	qp = emain_->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, Global::instance()->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, Global::instance()->c_highlight);
	emain_->setPalette(qp);
	
	//*****************************************************
	// aux editor
	//*****************************************************
	eaux_ = new BriefEdit(this, split3, "editor_aux" );
	// get default XML syntax
	getSyntax("", eaux_);
	connect( eaux_, SIGNAL( textChanged() ), this, SLOT( textChanged() ) );
	connect( eaux_, SIGNAL( clicked(int, int) ), this, SLOT( clickedAux(int, int) ) );
	eaux_->setWordWrap(Q3TextEdit::NoWrap);

	// colors
	eaux_->setPaletteBackgroundColor(Global::instance()->c_background);
	eaux_->setPaletteForegroundColor(Global::instance()->c_foreground);
	qp = eaux_->palette();
	qp.setColor(QPalette::Active, QColorGroup::Highlight, Global::instance()->c_highlight);
	qp.setColor(QPalette::Inactive, QColorGroup::Highlight, Global::instance()->c_highlight);
	eaux_->setPalette(qp);
	changed_aux_ = false;
	b_aux_ = new BufMgr();
	b_aux_->dbgname_ = "aux";
	
	// the aux window is made with size 0
	QValueList<int> list = split3->sizes();
	QValueList<int>::Iterator it = list.begin();
	int split3size = 0;
	while( it != list.end() ) {
		split3size += *it;
		++it;
	}
	bool begin = true;
	it = list.begin();
	while( it != list.end() ) {
		if (begin) {
			*it = split3size;
			begin = false;
		}
		else {
			*it = 0;
		}
		++it;
	}
	split3->setSizes(list);
    

	//*****************************************************
	// user area
	//*****************************************************
	tw_ = new QTabWidget(split1);
	tw_size_ = 200;
	tw_->setMaximumHeight(tw_size_);
	// user
	tw_user_ = new Q3MultiLineEdit(split1);
	tw_user_->setReadOnly(TRUE);
	tw_user_->setPaletteBackgroundColor(Global::instance()->c_background);
	tw_user_->setPaletteForegroundColor(Global::instance()->c_foreground);
	tw_user_->setPalette(qp);
	// search
	tw_search_ = new Q3MultiLineEdit(split1);
	tw_search_->setReadOnly(TRUE);
	tw_search_->setPaletteBackgroundColor(Global::instance()->c_background);
	tw_search_->setPaletteForegroundColor(Global::instance()->c_foreground);
	tw_search_->setPalette(qp);
	// tags
	tw_tags_ = new Q3MultiLineEdit(split1);
	tw_tags_->setReadOnly(TRUE);
	tw_tags_->setPaletteBackgroundColor(Global::instance()->c_background);
	tw_tags_->setPaletteForegroundColor(Global::instance()->c_foreground);
	tw_tags_->setPalette(qp);
	getSyntax("file.cpp", tw_tags_);
	ctags_locked_ = false;
	// bookmarks
	tw_bookmarks_ = new Q3MultiLineEdit(split1);
	tw_bookmarks_->setReadOnly(TRUE);
	tw_bookmarks_->setPaletteBackgroundColor(Global::instance()->c_background);
	tw_bookmarks_->setPaletteForegroundColor(Global::instance()->c_foreground);
	tw_bookmarks_->setPalette(qp);
	// hex
	tw_hex_ = new Q3MultiLineEdit(split1);
	tw_hex_->setReadOnly(TRUE);
	tw_hex_->setPaletteBackgroundColor(Global::instance()->c_background);
	tw_hex_->setPaletteForegroundColor(Global::instance()->c_foreground);
	tw_hex_->setPalette(qp);
	QFont courier("Courier",12);
	tw_hex_->setFont(courier);

	
	connect(tw_tags_, SIGNAL(clicked(int, int)), this, SLOT(clickedTags(int, int)));
	connect(tw_search_, SIGNAL(clicked(int, int)), this, SLOT(clickedSearch(int, int)));
	connect(tw_user_, SIGNAL(clicked(int, int)), this, SLOT(clickedUser(int, int)));
	connect(tw_bookmarks_, SIGNAL(clicked(int, int)), this, SLOT(clickedBookmarks(int, int)));
	connect(tw_hex_, SIGNAL(clicked(int, int)), this, SLOT(clickedHex(int, int)));

	tw_->addTab(tw_tags_, "tags");
	tw_->addTab(tw_search_, "search");
	tw_->addTab(tw_user_, "user");
	tw_->addTab(tw_bookmarks_, "bookmarks");
	tw_->addTab(tw_hex_, "hex");
	tw_current_ = tw_tags_;
	connect(tw_, SIGNAL(currentChanged(QWidget*)), this, SLOT(twChanged(QWidget*)));

	//*****************************************************
	// status bar
	//*****************************************************
	status_ = new QStatusBar(vbox, "status bar");
	connect( emain_, SIGNAL(cursorPositionChanged(int, int)), this, SLOT(cursorPosChanged(int, int)));
	connect( eaux_, SIGNAL(cursorPositionChanged(int, int)), this, SLOT(cursorPosChanged(int, int)));
	ff_ = new FormatField(status_, ff_str(default_format_));
	status_->addWidget(ff_, 0, TRUE);
	iow_ = new InsertOverwrite(status_, "insert overwrite");
	status_->addWidget(iow_, 0, TRUE);
	lac_ = new LineAndColumn(status_, "line and column widget");
	status_->addWidget(lac_, 0, TRUE);
	// set keyboard focus in the editor window
	e_->setFocus();

	// connect ctags thread
//#ifdef USING_QT4
	thread_ = new CtagsThread(this);
	thread_->start(QThread::LowPriority);
#ifdef USING_QT4
	connect((const QObject *) thread_, SIGNAL(alldone()), this, SLOT(ctagsDone()));
#endif
}

Editor::~Editor()
{
	LOG("Editor::~Editor");
}

void Editor::releaseAll() {
//	if (tw_tags_syntax_ == 0) // in order not to release twice
//		return;
//printf("releaseAll\n");		
	LOG("releaseAll");
	
	// release syntax
	delete tw_tags_->syntaxHighlighter();
	delete emain_->syntaxHighlighter();
	delete eaux_->syntaxHighlighter();
	
	delete b_main_;
	delete b_aux_;
	
	if (pdict_ != 0)
		delete pdict_;
	delete m_;
	delete mwords_;
//	delete e_main_;
	// terminate the thread
	thread_->terminate();
	thread_->wait();
	delete thread_;
	delete status_;
}



void Editor::cursorPosChanged(int para, int pos) {
	char msg[500];
	if (e_->isRecording())
		sprintf(msg, "Recording: %d/%d", para + 1, pos + 1);
	else
		sprintf(msg, "%d/%d", para + 1, pos + 1);
	lac_->setText(msg);

	if (!thread_->isActive() || ctags_locked_)
		return;
		
	QString line = e_->textLine(para);
	QChar ch = line.at(pos);
	if (ch.isLetterOrNumber() || ch == '_') {
		// extract word
		int xstart = pos;
		while (xstart >= 0 && (line.at(xstart).isLetterOrNumber() || line.at(xstart) == '_'))
			xstart--;
		if (xstart != 0)
			xstart++;

		int xend = pos;
		while (line.at(xend).isLetterOrNumber() || line.at(xend) == '_')
			xend++;

		QString ctagword = line.mid(xstart, xend - xstart);
		if (ctagword.startsWith("\t")) // lucian: some problems with the first letter when is at the beginning of the line! Below is just a hack!
			ctagword = ctagword.mid(1);
		if (ctagword.startsWith("\\"))
			ctagword = ctagword.mid(1);
		thread_->startProcessing(ctagword);
	}
}

void Editor::pickBuffer(int id) {
	if (id < 0)
		return;
	saveCurentBuffer();

	int index = id - 1;
LOG("got alt key, id %d", id);

	// move to the specific buffer
	b_->setActiveIndex(index);
	
	// restore data
	if (using_aux_)
		changed_aux_ = true;
	else
		changed_main_ = true;
		
	QString fname = b_->getActiveFName();
	setFname(fname, b_->getActiveFormat(), false);
	e_->setText(b_->getActiveText());
	
	// some 50ms delay here
	QTime t;
	t.start();
	while (t.elapsed() < 50) {
		qApp->processEvents();
	}
	
	if (using_aux_)
		changed_aux_ = fname.startsWith("*");
	else
		changed_main_ = fname.startsWith("*");
	int line = b_->getActiveLine();
	int col = b_->getActiveCol();
//LOG("get line %d col %d", line, col);
	e_->setCursorPosition(line, col, false);
	redoBufMgrList();
}

void Editor::pickBuffer(BufMgr *b, BriefEdit *e, int id) {
	if (id < 0)
		return;
	saveCurentBuffer(b, e);

	int index = id - 1;
LOG("got alt key, id %d", id);

	// move to the specific buffer
	b->setActiveIndex(index);
	
	// restore data
	if (using_aux_)
		changed_aux_ = true;
	else
		changed_main_ = true;
		
	QString fname = b->getActiveFName();
//	setFname(fname, false);
	e->setText(b->getActiveText());
	
	// some 50ms delay here
	QTime t;
	t.start();
	while (t.elapsed() < 50) {
		qApp->processEvents();
	}
	
	if (using_aux_)
		changed_main_ = fname.startsWith("*");
	else
		changed_aux_ = fname.startsWith("*");
	int line = b->getActiveLine();
	int col = b->getActiveCol();
//LOG("get line %d col %d", line, col);
	e->setCursorPosition(line, col, false);
//	redoBufMgrList();
}

void Editor::saveCurentBuffer(BufMgr *b, BriefEdit *e) {
	QString fn = b->getActiveFName();
	if (fn.isNull() || fn.isEmpty()) // it will get here first time we open the editor
		return;

	b->setActiveText(e->text());
	bool isnew = b->setActiveFName(fn);
	if (isnew)
		b->add(caption(), default_format_);
	int line;
	int col;
	e->getCursorPosition(&line, &col);
//LOG("save line %d col %d", line, col);
	b->setActiveLine(line);
	b->setActiveCol(col);
}


void Editor::saveCurentBuffer() {
	QString fn = caption();
	if (fn.isNull() || fn.isEmpty()) // it will get here first time we open the editor
		return;

	b_->setActiveText(e_->text());
	bool isnew = b_->setActiveFName(caption());
	if (isnew)
		b_->add(caption(), default_format_);
	int line;
	int col;
	e_->getCursorPosition(&line, &col);
//LOG("save line %d col %d", line, col);
	b_->setActiveLine(line);
	b_->setActiveCol(col);
}

void Editor::nextBuffer() {
	// save curent text first
	saveCurentBuffer();
	
	// move to the next buffer
	b_->setActiveNext();

	
	// restore data
	if (using_aux_)
		changed_aux_ = true;
	else
		changed_main_ = true;
	QString fname = b_->getActiveFName();
	setFname(fname, b_->getActiveFormat(), false);
	e_->setText(b_->getActiveText());	
	if (using_aux_)
		changed_aux_ = fname.startsWith("*");
	else
		changed_main_ = fname.startsWith("*");

	int line = b_->getActiveLine();
	int col = b_->getActiveCol();
//LOG("get line %d col %d", line, col);
	e_->setCursorPosition(line, col, false);
	e_->syntaxHighlighter()->rehighlight();

//LOG("starting redo");	
	redoBufMgrList();
}

void Editor::saveBufMgrAll(BufMgr *bm, BriefEdit *e) {
	ASSERT(bm);
	ASSERT(e);
	// save current editing info in buffer mgr	
	saveCurentBuffer(bm, e);

	// save content of buffer mgr in files
	for (int i = 0; i < MAXBUFMGR; i++) {
		QString fname = bm->getFName(i);
		if (fname == "*") {
			QMessageBox::warning(this,"I/O Info", QString("Please save all new files first\n"));
			return;
		}

		if (fname.isNull())
			break;
		if (fname.startsWith("*")) {
			saveCommonBuffer(fname.mid(1), bm->getFormat(i), bm->getText(i));
			bm->setFName(fname.mid(1), i);
		}
	}
	
	// remove * from current editing name; by now we have no * in bufmgr
	if (bm == b_) {
		QString fn = bm->getActiveFName();
		setFname(fn, bm->getActiveFormat(), false);
	}
}

bool Editor::saveAll() {
	// save main buffer manager
	saveBufMgrAll(b_main_, emain_);
	changed_main_ = false;
	// save aux buffer manager
	saveBufMgrAll(b_aux_, eaux_);
	changed_aux_ = false;
	
	redoBufMgrList();
	return true;
}


void Editor::redoBufMgrList() {
	buffers_->clear();
	buffers_->insertItem( "&Next", this, SLOT(nextBuffer()), Qt::ALT+Qt::Key_N);
	buffers_->insertSeparator();
	for (int i = 0; i < MAXBUFMGR; i++) {
		QString fname = b_->getFName(i);
		if (fname.isNull())
			break;
//printf("fname %s\n", (const char *) fname);fflush(0);
		buffers_->insertItem(fname, i + 1);
	}
}




void Editor::setFname(QString fname, FILE_FORMAT format, bool modified) {
//lucian: implement format; if format is FILE_FORMAT_MAX do not change the stored format
	(void) format;
	if (format != FILE_FORMAT_MAX) {
		ASSERT(ff_ != 0);
		ff_->setText(ff_str(format));
	}
	
	if (modified) {
		QString new_fname = "*";
		new_fname.append(fname);
		setCaption(new_fname);
	}
	else {
		setCaption(fname);
	}
	
//	QSyntaxHighlighter *tmp1 = e_->syntaxHighlighter();
//printf("old %s syntax %p\n", (const char *) fname, tmp1);
	getSyntax(fname, e_);
//	e_->update();
	
// lucian: leaking memory here
//	QSyntaxHighlighter *tmp2 = e_->syntaxHighlighter();
//printf("new %s syntax %p\n", (const char *) fname, tmp2);
//	if (tmp1 != 0)
//		delete tmp1;
	
	// if necessary load latex keywords in dictionary
	if (fname.isNull() || fname.isEmpty())
		return;
	
	if (!fname.endsWith(".tex") && !fname.endsWith(".sty"))
		return;
	if (pdict_ == 0)
		initDictionary();
	loadLatexKeywords(fname, pdict_);
}

QString Editor::getFname() {
	QString retval = caption();

	if (retval.startsWith("*"))
		retval = retval.mid(1);
	
	return retval;
}






void Editor::newDoc()
{
	LOG("newDoc");
	Editor *ed = new Editor;
	if (qApp->desktop()->size().width() < 450
	    || qApp->desktop()->size().height() < 450) {
		ed->showMaximized();
	} else {
		ed->resize(Global::instance()->xsize, Global::instance()->ysize);
		LOG("Editor::newDoc sizes %d %d", Global::instance()->xsize, Global::instance()->ysize);
		ed->show();
	}
}

void Editor::newDocBuffer()
{
	LOG("newDocBuffer");
	// save curent text first
	saveCurentBuffer();

	// add new empty buffer
	if (using_aux_)
		changed_aux_ = true;
	else
		changed_main_ = true;
		
	b_->add("", default_format_);
	setFname("", default_format_);
	e_->setText("");
	e_->setCursorPosition(1, 1, false);
	redoBufMgrList();
	if (using_aux_)
		changed_aux_ = false;
	else
		changed_main_ = false;
}

// lucian: more work on format detection here
FILE_FORMAT Editor::detectWwwFormat(const char *str) {
	ASSERT(str != 0);
	FILE_FORMAT retval = FILE_UTF8;

	char *ptr = strcasestr(str, "<head>");
	if (ptr == 0)
		return retval;
		
	char *ptr1 = ptr;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-1")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_1;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-2")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_2;
	
	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-3")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_3;
	
	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-4")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_4;
	
	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-5")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_5;
	
	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-6")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_6;
	
	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-7")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_7;
	
	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-8")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_8;
	
	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-9")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_9;
	
	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-10")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_10;
	
	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-11")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_11;
	
	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-12")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_12;
	
	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-13")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_13;

	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-14")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_14;
	
	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-15")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_15;
	
	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=iso-8859-16")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_ISO8859_16;
	
	ptr = ptr1;
	if ((ptr = strcasestr(ptr, "charset=windows-1250")) != 0 && (ptr = strcasestr(ptr, "</head>")) != 0)
		return FILE_CP1250;

	return retval;
}


void Editor::openWwwLocation()
{
	if (using_aux_ && changed_aux_ || (!using_aux_) && changed_main_) { // the text has been changed
		switch (QMessageBox::warning( this, "Question",
								"Save changes to Document?",
								tr("&Yes"),
								tr("&No"),
								tr("Cancel"),
								0, 2)) {
			case 0: // yes
				save();
				break;
			case 1: // no
				break;
			default: // cancel
				return;
				break;
		}
	}

	// check for wget presence
	if (!Global::checkWhich("wget"))
		return;

	WwwDialog *wd = new WwwDialog();
	wd->setCaption("Enter document location");
	wd->exec();
	QString sword = wd->getWord();

	if (sword == "")
		return;
	Q3CString utext = sword.utf8();
	const char *cstr = utext;
	LOG("Editor::openWwwLocation  %s", cstr);
	delete wd;
	msgScreen("Loading...");
	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );

	char cmd[1000];
	snprintf(cmd, 1000, "wget --timeout=60 --tries=1 -q --output-document=- %s", cstr);
	char *www_content = UTILS::run_prg_malloc(cmd);
	QApplication::restoreOverrideCursor();
	if (www_content != 0) {
		FILE_FORMAT format = detectWwwFormat(www_content);
		QString txt = ff_toUtf8(www_content, format);
		e_->setText(txt);
		sword.replace(QChar('/'), "_");
		setFname(sword, format);
		msgScreen(ff_str(format), 5000);
		delete[] www_content;
		if (using_aux_)
			changed_aux_ = false;
		else
			changed_main_ = false;
		b_->add(sword, format);
		b_->setActiveText(txt);
		int line;
		int col;
		e_->getCursorPosition(&line, &col);
		b_->setActiveLine(line);
		b_->setActiveCol(col);
		e_->setCursorPosition(1, 1, false);

		redoBufMgrList();
		msgScreen("File loaded.", 5000);
	}
	else {
		msgScreen("Failed to load the file.", 5000);
	}
}

void Editor::msgScreen(const char *str, int mstime) {
	status_->message(str, mstime);
	qApp->processEvents();
}



void Editor::setDefaultFormat(FILE_FORMAT format) {
	default_format_ = format;
}

void Editor::load()
{
	msgScreen("loading file, please wait...");
	QString fn = Q3FileDialog::getOpenFileName( QString::null, QString::null, this );
	if (!fn.isEmpty())
		load(fn);
	else
		msgScreen("aborted", 5000);
}


void Editor::load(const QString& fileName, bool absolute)
{
	(void) absolute; // lucian???

	BCursor busy;
	msgScreen("loading file, please wait...");
	// save curent buffer first
	saveCurentBuffer();

	// move to an absolute file name
	QFileInfo qf(fileName);
	QString newfileName = qf.absFilePath();
	
	// update dirfiles_path_
	QFileInfo qf1(newfileName);
	QString fn = qf1.fileName();
	dirfiles_path_ = qf1.filePath();
	dirfiles_path_ = dirfiles_path_.left(dirfiles_path_.length() - fn.length());
	QString dfiles = getDirfiles();
	dirfiles_->setText(dfiles);

	// check if the file is already loaded
	int index = 0;
	bool foundit = b_->hasFile(newfileName, &index);
	if (foundit) {
		pickBuffer(index + 1);
		msgScreen("file already in the buffer list", 2000);
		return;
	}

	// check if the file is not loaded in the inactive buffer manager
	BufMgr *b1 = (using_aux_)? b_main_: b_aux_;
	BriefEdit *e1 = (using_aux_)? emain_: eaux_;
	foundit = b1->hasFile(newfileName, &index);
	if (foundit) {
		pickBuffer(b1, e1, index + 1);
		msgScreen("file already open in the other window");
		return;
	}
	
	if (using_aux_)
		changed_aux_ = true;
	else
		changed_main_ = true;
	Q3CString qfname = newfileName.utf8();
	const char *fname = qfname;
	LOG("Editor::load %s %d", fname, (int) default_format_);
	e_->setAutoUpdate( FALSE );
	// lucian: is the format correct?
	setFname(newfileName, default_format_, false);

	QFile f( fileName );
#ifdef USING_QT3
	if (f.open(IO_ReadOnly )) {
#endif
#ifdef USING_QT4
	if (f.open( QIODevice::ReadOnly )) {
#endif

		QTextStream t(&f);
		if (default_format_ == FILE_FORMAT_MAX) {
			ASSERT(0);
			msgScreen("encoding not found; file not loaded");
			return;
		}
		else if (default_format_ == FILE_UTF8) {
			t.setEncoding( QTextStream::UnicodeUTF8 );
			LOG("Editor::load utf8");
		}
		else {
			QTextCodec *codec = QTextCodec::codecForName(ff_str(default_format_));
			t.setCodec(codec);
			LOG("Editor::load %s", ff_str(default_format_));
		}
	
		e_->setText( t.read() );
		f.close();
	} // if file open
	else {
		e_->setText("");
	}
	
	// backup
	QString fname_only = qf.fileName();
	char *backup_fname = Global::getBackupFname((const char *) qf.fileName());
	bool noerr = saveCommon(QString(backup_fname), default_format_, true);
	if (noerr == FALSE) {
		// normally we should not get here!!!
		ASSERT(0);
		// error recovery: create backup directory
		char *pmkdir = UTILS::run_prg_malloc("mkdir -p ~/.rospell/backup");
		delete[] pmkdir;
		QMessageBox::warning(this,"I/O Info",
						 QString("Backup directory created\n"));
	}

	delete[] backup_fname;

	// if this is a latex file, convert it to utf8 for editing purposes
	if (fname_only.endsWith(".tex")) {
		QString tmp = latex2utf8(e_->text());
		e_->setText(tmp);
	}

	e_->setAutoUpdate( TRUE );
	e_->repaint();
	if (using_aux_)
		changed_aux_ = false;
	else
		changed_main_ = false;
	cleanMWords();
	
	// set buffer
	b_->add(newfileName, default_format_);
	b_->setActiveText(e_->text());
	e_->setCursorPosition(0, 0, false);
	redoBufMgrList();
	msgScreen("file loaded", 5000);
}


void Editor::save()
{
	msgScreen("saving file, please wait...");
	QString fname = getFname();
	if (fname.isNull())
		saveAs();
	else {
		BCursor busy;
		Q3CString utext = fname.utf8();
		const char *cstr = utext;
		LOG("Editor::save file %s", cstr);
		if (!fname.isEmpty())
			saveCommon(fname, default_format_);
		else
			msgScreen("aborted", 5000);
	}
	msgScreen("file saved", 5000);
	e_->update();
}

bool Editor::saveAs()
{
	msgScreen("saving file, please wait...");
	// name change in bufmgr is done in saveComon
	
	//storing filename (proper save) is left as an exercise...
	QString fn = Q3FileDialog::getSaveFileName( QString::null, QString::null, this );
	BCursor busy;
	if (!fn.isEmpty()) {
		bool retval = saveCommon(fn, default_format_);
		QString dirfiles = getDirfiles();
		dirfiles_->setText(dirfiles);
		return retval;
	}	

	msgScreen("aborted", 5000);
	return FALSE;
}

bool Editor::saveCommonBuffer(const QString& fileName, FILE_FORMAT format, QString text)
{
	QFile f( fileName );
	
#ifdef USING_QT3
	if (!f.open(IO_WriteOnly )) {
#endif
#ifdef USING_QT4
	if (!f.open( QIODevice::WriteOnly )) {
#endif

		QMessageBox::warning(this,"I/O Error",
						 QString("The file could not be opened.\n\n")
						 +fileName);
		msgScreen("file not saved", 5000);
		return FALSE;
	}
	QTextStream t(&f);
	LOG("save common format %d", format);


	if (format == FILE_FORMAT_MAX) {
		ASSERT(0);
		msgScreen("file not saved", 5000);
		return FALSE;
	}
	else if (format == FILE_UTF8) {
		t.setEncoding( QTextStream::UnicodeUTF8 );
		LOG("Editor::saveCommonBuffer utf8");
	}
	else {
		QTextCodec *codec = QTextCodec::codecForName(ff_str(default_format_));
		t.setCodec(codec);
		LOG("Editor::saveCommonBuffer %s", ff_str(default_format_));
	}

	QString tmp;
	// if this is a latex file save it in latex/ascii format
	if (fileName.endsWith(".tex"))
		tmp = utf82latex(text);
	else
		tmp = text;
	t << tmp;
	f.close();
	return TRUE;
}



bool Editor::saveCommon(const QString& fileName, FILE_FORMAT format, bool simple_save)
{
	QFile f( fileName );
	
#ifdef USING_QT3
	if (!f.open(IO_WriteOnly )) {
#endif
#ifdef USING_QT4
	if (!f.open( QIODevice::WriteOnly )) {
#endif

		QMessageBox::warning(this,"I/O Error",
						 QString("The file could not be opened.\n\n")
						 +fileName);
		msgScreen("file not saved", 5000);
		return FALSE;
	}
	QTextStream t(&f);
	LOG("save common format %d", format);

	if (format == FILE_FORMAT_MAX) {
		ASSERT(0);
		msgScreen("file not saved", 5000);
		return FALSE;
	}
	else if (format == FILE_UTF8) {
		t.setEncoding( QTextStream::UnicodeUTF8 );
		LOG("Editor::saveCommon utf8");
	}
	else {
		QTextCodec *codec = QTextCodec::codecForName(ff_str(default_format_));
		t.setCodec(codec);
		LOG("Editor::saveCommon %s", ff_str(default_format_));
	}

	QString tmp;
	
	// if this is a latex file save it in latex/ascii format
	if (fileName.endsWith(".tex"))
		tmp = utf82latex(e_->text());
	else
		tmp = e_->text();
	t << tmp;
	f.close();
	
	if (simple_save)
		return TRUE;
	setFname(fileName, format);
	if (using_aux_)
		changed_aux_ = false;
	else
		changed_main_ = false;
	cleanMWords();
	bool isnew = b_->setActiveFName(fileName);
	if (isnew)
		b_->add(fileName, format);
	b_->setActiveText(e_->text());
	int line;
	int col;
	e_->getCursorPosition(&line, &col);
	b_->setActiveLine(line);
	b_->setActiveCol(col);
	redoBufMgrList();
	msgScreen("file saved", 5000);
	e_->syntaxHighlighter()->rehighlight();
	e_->setFocus();
	return TRUE;
}



void Editor::print()
{
#ifndef ROSPELL_NO_PRINTER
	if (printer_.setup(this)) {			   // opens printer dialog
		printer_.setFullPage(TRUE);			 // we'll set our own margins
		QPainter p;
		p.begin( &printer_ );				 // paint on printer
		p.setFont( e_->font() );
		QFontMetrics fm = p.fontMetrics();
		Q3PaintDeviceMetrics metrics( &printer_ ); // need width/height
		// of printer surface
		const int MARGIN = metrics.logicalDpiX() / 2; // half-inch margin
		int yPos        = MARGIN;			// y position for each line

		for (int i = 0 ; i < e_->numLines() ; i++) {
			if (printer_.aborted())
				break;
			if (yPos + fm.lineSpacing() > metrics.height() - MARGIN) {
				// no more room on this page
				if (!printer_.newPage())		    // start new page
					break;					   // some error
				yPos = MARGIN;				   // back to top of page
			}
			p.drawText( MARGIN, yPos, metrics.width() - 2*MARGIN,
					  fm.lineSpacing(), ExpandTabs, e_->textLine( i ) );
			yPos += fm.lineSpacing();
		}
		p.end();							// send job to printer
	}
	cleanMWords();
#endif
}

void Editor::resizeEvent( QResizeEvent * )
{
	LOG("resizeEvent sizes %d %d", width(), height());
	Global::instance()->xsize = width();
	Global::instance()->ysize = height();
}

void Editor::closeEvent( QCloseEvent *event )
{
	LOG("closeEvent");
	event->accept();
	bool torelease = true;

	if (changed_main_ || b_main_->anyBufChanged() || changed_aux_ || b_aux_->anyBufChanged()) {
		QString r1 = b_main_->printUnsaved();
		QString r2 = b_aux_->printUnsaved();

		switch (QMessageBox::warning( this, "Question",
			"One or more buffers have not been saved:\n\n" + r1 + r2 + "Save all?",
			tr("&Yes"),
			tr("&No"),
			tr("Cancel"),
			0, 2)) {
			case 0: // yes
				if (saveAll())
					event->accept();
				else {
					event->ignore();
					torelease = false;
				}
				break;
			case 1: // no
				event->accept();
				break;
			default: // cancel
				event->ignore();
				torelease = false;
				break;
		}
	}
	
	if (torelease) {
		releaseAll();
	}
}

void Editor::closeBuffer()
{
	LOG("closeBuffer");
	// save curent buffer first
	saveCurentBuffer();

	if (using_aux_ && changed_aux_ || (!using_aux_) && changed_main_) { // the text has been changed
		switch (QMessageBox::warning( this, "Question",
								"Save changes to Document?",
								tr("&Yes"),
								tr("&No"),
								tr("Cancel"),
								0, 2)) {
			case 0: // yes
				save();
				break;
			case 1: // no
			default: // cancel
				break;
		}
	}
	
	// first move some active pointers
	e_->setText("");
	setFname("rospell", default_format_, false);

	// delete buffer
	int aindex = b_->getActiveIndex();
	b_->del(aindex);
		
	// restore data
	if (using_aux_)
		changed_aux_ = true;
	else
		changed_main_ = true;
	e_->setText(b_->getActiveText());	
	QString fname = b_->getActiveFName();
	setFname(fname, b_->getActiveFormat(), false);
	if (using_aux_)
		changed_aux_ = fname.startsWith("*");
	else
		changed_main_ = fname.startsWith("*");
	redoBufMgrList();
	int line = b_->getActiveLine();
	int col = b_->getActiveCol();
	e_->setCursorPosition(line, col, false);
	e_->syntaxHighlighter()->rehighlight();
}




void Editor::textChanged()
{
	//is coming here for each key press;
//LOG("Text changed");
	if (using_aux_) {
		if (changed_aux_ == false) {
			QString fname = getFname();
			// setFname will not change the format
			setFname(fname, FILE_FORMAT_MAX, true);
			fname = caption();
			bool isnew = b_->setActiveFName(fname);
			if (isnew)
				b_->add(caption(), default_format_);
			redoBufMgrList();
		}
		changed_aux_ = true;
	}
	else {
		if (changed_main_ == false) {
			QString fname = getFname();
			// setFname will not change the format
			setFname(fname, FILE_FORMAT_MAX, true);
			fname = caption();
			bool isnew = b_->setActiveFName(fname);
			if (isnew)
				b_->add(caption(), default_format_);
			redoBufMgrList();
		}
		changed_main_ = true;
	}
	
	cleanMWords();
}
void Editor::clicked(int para, int pos) {
	// coming here for each click in e_
	bool changing = false;
	LOG("clicked %d/%d", para, pos);
	if (using_aux_ == true) {
		saveCurentBuffer();
		changing = true;
	}
		
	using_aux_ = false;
	e_ = emain_;
	b_ = b_main_;
	book_ = book_main_;
	cleanMWords();
	
	if (changing) {
		redoBufMgrList();
		QString fname = b_->getActiveFName();
		setFname(fname, b_->getActiveFormat(), false);
		if (tw_bookmarks_->isVisible())
			regenBookmarks();
	}
}

void Editor::clickedAux(int para, int pos) {
	// coming here for each click in eaux_
	bool changing = false;
	LOG("clicked %d/%d", para, pos);
	if (using_aux_ == false) {
		saveCurentBuffer();
		changing = true;
	}

	using_aux_ = true;
	e_ = eaux_;
	b_ = b_aux_;
	book_ = book_aux_;
	cleanMWords();
	if (changing) {
		redoBufMgrList();
		if (b_->isEmpty()) {
			setFname("", default_format_, false);
		}
		else {
			QString fname = b_->getActiveFName();
			setFname(fname, b_->getActiveFormat(), false);
		}
		
		if (tw_bookmarks_->isVisible())
			regenBookmarks();
	}
}



bool Editor::extractFileAndLine(const QString &in, QString &filename, int &line) {
	filename = in.section("::", 0, 0);
	if (filename.isEmpty() || filename.isNull()) {
		LOG("filename section empty or null");
		return false;
	}
	LOG("extracted file: %s", (const char *) filename);

	QString linenum = in.section("::", 1, 1);
	LOG("linenum: %s", (const char *) linenum);
	if (linenum.isEmpty() || linenum.isNull()) {
		LOG("line section empty or null");
		return false;
	}
	line = atoi((const char *) linenum);
	if (line <= 0)
		return false;
	LOG("extracted line: %d", line);

	return true;
}

void Editor::clickedTags(int para, int pos)
{
	LOG("clickedTw %d/%d", para, pos);
	if (ctags_locked_)
		return;
		
	QString sline = tw_tags_->textLine(para);
	QString filename;
	int line = 0;
	if (extractFileAndLine(sline, filename, line) ||
	   extractFileAndLine(tw_tags_->textLine(0), filename, line)) {
		// we have a file and a line number
		// save curent text first
		saveCurentBuffer();
		// open the file in the editor window
		LOG("loading file from ctags %s", (const char *) filename);
		// load the file in current edit window; it is protected not to load already loaded files
		load(filename); 
		// move to an absolute file name
		QFileInfo qf(filename);
		filename = qf.absFilePath();
		int index;
		
		bool foundit = b_->hasFile(filename, &index);
		if (foundit) {
			e_->setCursorPosition(line - 1, 0, false);
			e_->setFocus();
		}
		else {
			BufMgr *b = (b_ == b_main_)? b_aux_: b_main_;
			BriefEdit *e = (e_ == emain_)? eaux_: emain_;
			foundit = b->hasFile(filename, &index);
			if (foundit) {
				e->setCursorPosition(line - 1, 0, false);
				e->setFocus();
			}
		}
	}
}


void Editor::customEvent( QCustomEvent * e ) {
	LOG("got customEvent");
	(void) e;
	ctagsDone();
}
void Editor::ctagsDone() {
	if (ctags_locked_)
		return;
		
	QString msg = thread_->getMsg();
	tw_tags_->setText(msg);
	QString sline = tw_tags_->textLine(0);
	if (sline.endsWith(" single ctag definition, see below...")) {
		QString filename;
		int line = 0;
		if (extractFileAndLine(sline, filename, line))
			tw_tags_->setCursorPosition(line, 0, false);
	}
}

void Editor::roDebug() {
	QString result = "";
	if (changed_main_)
		result += "main changed, ";
	if (changed_aux_)
		result += "aux changed";
	result += "\n";

	result += b_main_->roDebug();
	result += b_aux_->roDebug();
	result += thread_->roDebug();
	result += (ctags_locked_)? "ctags locked\n": "";
	QMessageBox::warning(this,"roDebug", result);
}

void Editor::roDevel() {
	int para;
	int index;
	e_->getCursorPosition(&para, &index);
	QString line = e_->textLine(para);
	int lines = e_->numLines();
	int len = line.length();
	QString line1 = line.mid(0, index);
	if (line1.isNull() || line1.isEmpty())
		line1 = "";
	QString line2 = line.mid(index);
	if (line2.isNull() || line2.isEmpty())
		return;
	
	const char *ptr = (const char *) line2;
	int i = 0;
	while (isalnum(ptr[i]) || ptr[i] == '_')
		i++;
	
	QString line3 = line2.mid(0, i);
	QString line4 = line2.mid(i);
	
	line = line1 + "'" + line3 + "'" + line4 + "\n";
	
	// select the line
	if (para == (lines - 1)) { 
		e_->setSelection(para, 0, para, len);
	}
	else {
		e_->setSelection(para, 0, para + 1, 0);
	}

	e_->cut();
	e_->insert(line);
	e_->setCursorPosition(para, index, false);
}


void Editor::userF1() {
	tw_user_->setText("");
	msgScreen("F1, please wait...");
	BCursor busy;
	
	QTUTILS::run_program(QString("~/.rospell/userF1.sh"), tw_user_);
	tw_->setCurrentPage(2); //lucian: hardcoded
	msgScreen("done", 2000);
}

void Editor::userF2() {
	tw_user_->setText("");
	msgScreen("F2, please wait...");
	BCursor busy;
	QTUTILS::run_program(QString("~/.rospell/userF2.sh"), tw_user_);
	tw_->setCurrentPage(2); //lucian: hardcoded
	msgScreen("done", 2000);
}

void Editor::userF3() {
	tw_user_->setText("");
	msgScreen("F3, please wait...");
	BCursor busy;
	QTUTILS::run_program(QString("~/.rospell/userF3.sh"), tw_user_);
	tw_->setCurrentPage(2); //lucian: hardcoded
	msgScreen("done", 2000);
}

void Editor::userF4() {
	tw_user_->setText("");
	msgScreen("F4, please wait...");
	BCursor busy;
	QTUTILS::run_program(QString("~/.rospell/userF4.sh"), tw_user_);
	tw_->setCurrentPage(2); //lucian: hardcoded
	msgScreen("done", 2000);
}


 

bool Editor::extractFileAndLine2(const QString &in, QString &filename, int &line) {
	filename = in.section(":", 0, 0);
	if (filename.isEmpty() || filename.isNull()) {
		LOG("filename section empty or null");
		return false;
	}
	LOG("extracted file: %s", (const char *) filename);

	QString linenum = in.section(":", 1, 1);
	LOG("linenum: %s", (const char *) linenum);
	if (linenum.isEmpty() || linenum.isNull()) {
		LOG("line section empty or null");
		return false;
	}
	line = atoi((const char *) linenum);
	if (line <= 0)
		return false;
	LOG("extracted line: %d", line);

	return true;
}

bool Editor::extractLine(const QString &in, int &line) {
	QString linenum = in.section(":", 0, 0);
	LOG("linenum: %s", (const char *) linenum);
	if (linenum.isEmpty() || linenum.isNull()) {
		LOG("line section empty or null");
		return false;
	}
	line = atoi((const char *) linenum);
	if (line <= 0)
		return false;
	LOG("extracted line: %d", line);

	return true;
}

void Editor::clickedUserSearch(QString sline, int para, int pos) {
	(void) para;
	(void) pos;
	QString filename;
	int line = 0;
	if (extractLine(sline, line)) { // only in the active window!!!
		// we have a line number in the current file
		e_->setCursorPosition(line - 1, 0, false);
		e_->setFocus();
	}
	else if (extractFileAndLine2(sline, filename, line)) {
		// we have a file and a line number
		// save curent text first
		saveCurentBuffer();
		// open the file in the editor window
		LOG("loading file from ctags %s", (const char *) filename);
		// load the file in current edit window; it is protected not to load already loaded files
		load(filename); 
		// move to an absolute file name
		QFileInfo qf(filename);
		filename = qf.absFilePath();
		int index;
		
		bool foundit = b_->hasFile(filename, &index);
		if (foundit) {
			e_->setCursorPosition(line - 1, 0, false);
			e_->setFocus();
		}
		else {
			BufMgr *b = (b_ == b_main_)? b_aux_: b_main_;
			BriefEdit *e = (e_ == emain_)? eaux_: emain_;
			foundit = b->hasFile(filename, &index);
			if (foundit) {
				e->setCursorPosition(line - 1, 0, false);
				e->setFocus();
			}
		}
	}	

}

void Editor::clickedSearch(int para, int pos)
{
	LOG("clickedSearch %d/%d", para, pos);
	QString sline = tw_search_->textLine(para);
	clickedUserSearch(sline, para, pos);
}

void Editor::clickedUser(int para, int pos)
{
	(void) pos;
	
	LOG("clickedUser %d/%d", para, pos);
	QString sline = tw_user_->textLine(para);
	clickedUserSearch(sline, para, pos);
}

void Editor::clickedBookmarks(int para, int pos) {
	(void) pos;
	int line = para / 2 + 1;
	bool found = false;
	int bookmark;
	for (bookmark = 0; bookmark < MAXBOOKMARKS; bookmark++) {
		if (book_[bookmark].fname_.isEmpty())
			continue;
		line--;
		if (line == 0) {
			found = true;
			break;
		}
	}
	if (found)
		gotoBookmarkCommon(bookmark);
}

void Editor::clickedHex(int para, int pos) {
	(void) para;
	(void) pos;
	
	QString hex = "";
	int lineno;
	int col;
	e_->getCursorPosition(&lineno, &col);
	QString line = e_->textLine(lineno);
	Q3CString uline = line.utf8();
	const char *ptr1 = uline;
	int len = strlen(ptr1);
	unsigned char *ptr = (unsigned char *) ptr1;

	char start[10000];
	char *hexline = start;
	int i;
	char str[17];
	memset(str, '\0', 17);
	
	for (i = 0; i < len; i++, ptr++) {
		if (i % 16 == 0) {
			sprintf(hexline, "%s\n", str);
			hex += start;
			hexline = start;
		}
		
		sprintf(hexline, "%02x ", *ptr);
		hexline += strlen(hexline);
		if (isprint(*ptr))
			str[i % 16] = *ptr;
		else
			str[i % 16] = '.';
	}
	str[i % 16] = '\0';
	for (i = 15 - (len % 16); i >= 0; i--) {
		sprintf(hexline, "   ");
		hexline += strlen(hexline);
	}
	sprintf(hexline, "%s\n", str);
	hex += start;

	tw_hex_->setText(hex);
}

QString Editor::getDirfiles() {
	FVBase fvb(dirfiles_path_, dircombo_->currentText(), "", false);
	FVisitorDirfiles visitor;
	return fvb.walk(&visitor);
}

void Editor::clickedDirfiles(int para, int pos)
{
	LOG("clickedDirfiles %d/%d", para, pos);
	BCursor busy;
	QString file = dirfiles_->textLine(para);
	QFileInfo qf(dirfiles_path_ + file);
	if (file == "..") {
		dirfiles_path_ += "../";
		QString files = getDirfiles();
		dirfiles_->setText(files);
	}
	else if (qf.isFile()) {
		load(dirfiles_path_ + file);
	}
	else if (qf.isDir()) {
		dirfiles_path_ += file + "/";
		QString files = getDirfiles();
		dirfiles_->setText(files);
	}
	else
		printf("??????\n");
	
	dirfiles_->setCursorPosition(para, pos, false); //clean any selection!!!
	e_->setFocus();
}

void Editor::twChanged(QWidget* qm) {
	tw_current_ = (Q3MultiLineEdit *) qm;
	if (qm == tw_hex_) {
//		sleep(2);
//		emit tw_hex_->clicked(0,0);
		clickedHex(0, 0);
	}
}

void Editor::dirboxChanged(int index) {
	(void) index;
	QString files = getDirfiles();
	dirfiles_->setText(files);
}


void Editor::initDictionary() {
	BCursor busy;
	pdict_ = new Dictionaries(true);
	readHomeDir(pdict_);
	ASSERT(pdict_ != 0);
}


void Editor::changeFormat() {
	LOG("Editor::changeFormat");
	cleanMWords();
	QStringList lst;
	ff_populateList(lst);
	bool ok;
	QString res = QInputDialog::getItem(
	        "Change Document Format", "Select format:", lst, 0, TRUE, &ok,
	        this );
	if ( ok ) {
		// user selected an item and pressed OK
		FILE_FORMAT format = ff_format((const char *) res);
		if (b_ == 0) {
			ASSERT(0);
			return;
		}
		default_format_ = format;
		ff_->setText(ff_str(format));
		b_->setActiveFormat(format);
	}
}
