// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************
//** $Id: qt/qwerty.h   3.3.4   edited May 27 2003 $
//**
//** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
//**
//** This file is part of an example program for Qt.  This example
//** program may be used, distributed and modified without limitation.
//**
//*****************************************************************************/

#ifndef QWERTY_H
#define QWERTY_H

#include "global.h"

#ifdef USING_QT4
//Added by qt3to4:
#include <QCloseEvent>
#include <QResizeEvent>
#include <Q3PopupMenu>
#endif


#include <qwidget.h>
#include <qmenubar.h>



#ifdef USING_QT3
#include <qlayout.h>
#include <qmultilineedit.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qaccel.h>
#include <qsyntaxhighlighter.h>
#endif
#ifdef USING_QT4
#include <Q3VBoxLayout>
#include <q3multilineedit.h>
#include <q3hbox.h>
#include <q3vbox.h>
#include <q3accel.h>
#include <q3syntaxhighlighter.h>
#endif

#include <qprinter.h>
#include <qdialog.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qfont.h>
#include <qlayout.h>
#include <qstatusbar.h>
#include <qtabwidget.h>


// forward declarations
class BufMgr;
class CtagsThread;
class Dictionaries;
class BriefEdit;
class LineAndColumn;
class InsertOverwrite;
class FormatField;
class QComboBox;

class Editor : public Q3VBox {
	Q_OBJECT

public:
	Editor( QWidget *parent=0, const char *name="qwerty" );
	~Editor();
	void load(const QString& fileName, bool absolute = false);
	void setDefaultFormat(FILE_FORMAT format);
	void setFname(QString fname, FILE_FORMAT format, bool modified = false);
	QString getFname();
	bool advanceSearch(QString tofind, bool ignore_case = false, bool whole = false);
	bool advanceSearchList(QString tofind, bool ignore_case = false, bool whole = false);
	bool advanceSearchRev(QString tofind, bool ignore_case = false, bool whole = false);
	bool advanceSearchListRev(QString tofind, bool ignore_case = false, bool whole = false);
	void msgScreen(const char *str, int mstime = 0);

public slots:
	void newDoc();
	void newDocBuffer();
	void load();
	void openWwwLocation();
	void save();
	bool saveAs();
	bool saveAll();
	void changeFormat();
	void print();
	void font();
	void spellAdvance();
	void spellIgnore();
	void spellAdd();
	void spellName();
	void undo();
	void cut();
	void copy();
	void paste();
	void e_delete();
	void search();
	void searchAgain();
	void replace();
	void toc();
	void gotoLine();
	void setBookmark();
	void gotoBookmark();
	void insertHex();
	void status();
	void uppercase();
	void lowercase();
	void dexonline();
	void unixCleanup();
	void nextBuffer();
	void closeBuffer();
	void diff();
	void sessionDiff();
	void merge();
	void beautify();
	void kloc();
	void make();
	void colors();
	void configCtags();
	void lockCtags();
	void unlockCtags();
	void configLanguage();
	void configWWW();
	void grep();
	
	// events coming here from ctag thread
	void ctagsDone();
	// customEvent is used by qt3 implementation and does a
	// function call to ctagsDone
	void customEvent( QCustomEvent * e );
	
	void selectAll();
	void toggleOverwrite();
	void recordAction();
	void playbackAction();
	void editAlias();

	void editF1();
	void editF2();
	void editF3();
	void editF4();
	
protected:
	void resizeEvent(QResizeEvent *);
	void closeEvent(QCloseEvent *);

private slots:
	void pickBuffer(int id);
	void pickBuffer(BufMgr *b, BriefEdit *e, int id);
	void textChanged();
	void clicked(int para, int pos);
	void clickedAux(int para, int pos);
	void clickedDirfiles(int para, int pos);
	void clickedTags(int para, int pos);
	void clickedSearch(int para, int pos);
	void clickedUser(int para, int pos);
	void clickedBookmarks(int para, int pos);
	void clickedHex(int para, int pos);
	void twChanged(QWidget* qm);
	void dirboxChanged(int index);
	// sepeller-sugested words
	void replaceWord1();
	void replaceWord2();
	void replaceWord3();
	void replaceWord4();
	void replaceWord5();
	void cursorPosChanged(int para, int pos);
	// bookmarks
	void setBookmark1();
	void setBookmark2();
	void setBookmark3();
	void setBookmark4();
	void setBookmark5();
	void setBookmark6();
	void setBookmark7();
	void setBookmark8();
	void setBookmark9();
	void setBookmark0();
	// user progamable keys
	void userF1();
	void userF2();
	void userF3();
	void userF4();
	// start/end of buffer
	void startOfBuffer();
	void endOfBuffer();
	// debug
	void roDebug();
	void roDevel();
	// latex templates
	void latexT1();
	void latexT2();
	void latexT3();
	void latexT4();
	// latex features
	void latexF1();
	void latexF2();
	void latexF3();
	void latexF4();
	void latexF5();
	void latexF6();
	void latexF7();
	void latexF8();
	void latexF9();
	void latexF10();
	void latexF11();
	void latexF12();
	// latex font sizes
	void latexFS1();
	void latexFS2();
	void latexFS3();
	void latexFS4();
	void latexFS5();
	void latexFS6();
	void latexFS7();
	void latexFS8();
	void latexFS9();
	void latexFS10();
	// latex font type
	void latexFT1();
	void latexFT2();
	void latexFT3();
	void latexFT4();
	void latexFT5();
	// help support
	void helpAbout();
	void helpAboutQt();
	void helpGettingStarted();
	void helpBasicOperation();
	void helpProgramming();
	void helpKeyboard();

private:
	void initDictionary();
	void saveBufMgrAll(BufMgr *bm, BriefEdit *e);
	void clickedUserSearch(QString sline, int para, int pos);
	void regenBookmarks();
	void setBookmarkCommon(int bookmark);
	void gotoBookmarkCommon(int bookmark);
	void addLatextMenu();
	void addHelpMenu();
	void releaseAll();
	bool saveCommon( const QString& fileName, FILE_FORMAT format, bool simple_save = false);
	bool saveCommonBuffer(const QString& fileName, FILE_FORMAT format, QString text);
	FILE_FORMAT detectWwwFormat(const char *str);
	void advanceStartWord();
	QString advanceEndWord();
	bool skeepXML();

	void cleanMWords();
	// lookalike words entry point
	void doLookalike(char *active_word);
	// first i to â lookalike
	bool lookalike1(char *active_word, char *dest);
	// the second i to â lookalike
	bool lookalike2(char *active_word, char *dest);
	// first î to â lookalike
	bool lookalike3(char *active_word, char *dest);
	// flip char lookalike
	bool lookalike4(char *active_word, char *dest);
	// fix teutzz spelling: tz -> ţ
	bool lookalike5(char *active_word, char *dest);
	// fix sh spelling: sh -> ş
	bool lookalike6(char *active_word, char *dest);
	// insert lookalike in empty storage word
	void insertLookalike(char *str);
	// save curent buffer in bufmgr
	void saveCurentBuffer();
	void saveCurentBuffer(BufMgr *b, BriefEdit *e);
	// get the names of the files to put in dirfiles window
	QString getDirfiles();

	void redoBufMgrList();
	bool extractFileAndLine(const QString &in, QString &filename, int &line);
	bool extractFileAndLine2(const QString &in, QString &filename, int &line);
	bool extractLine(const QString &in, int &line);

	Q3Accel *a_;
	Q3PopupMenu *file_;
	Q3PopupMenu *edit_;
	Q3PopupMenu *tools_;
//	int msgid_; // id for message in menubar
	QMenuBar       *m_;
	QMenuBar       *mwords_;
	int mw1_;	//id for the first word in mwords_ menu bar
	int mw2_;	//id for the second word in mwords_ menu bar
	int mw3_;	//id for the third word in mwords_ menu bar
	int mw4_;	//id for the fourth word in mwords_ menu bar
	int mw5_;	//id for the fifth word in mwords_ menu bar
	char word1_[100];
	char word2_[100];
	char word3_[100];
	char word4_[100];
	char word5_[100];

	// bookmarks
	struct Bookmark {
		int x_;
		int y_;
		QString fname_;
		QString line_;
		Bookmark(): x_(0), y_(0), fname_(""), line_("") {}
	};
	
	// active edit window
	BriefEdit *e_;
	bool using_aux_;
	BufMgr *b_;
	Bookmark* book_;

	// main edit window
	BriefEdit *emain_;
	bool changed_main_;
	BufMgr *b_main_;
	Bookmark book_main_[MAXBOOKMARKS];
	
	// aux edit window
	BriefEdit *eaux_;
	bool changed_aux_;
	BufMgr *b_aux_;
	Bookmark book_aux_[MAXBOOKMARKS];
	
	// dirfiles
	QComboBox *dircombo_;
	Q3MultiLineEdit *dirfiles_;
	QString dirfiles_path_;
	int dirfiles_size_;
#ifndef ROSPELL_NO_PRINTER
	QPrinter        printer_;
#endif
	Dictionaries *pdict_;
	QString search_;		// copy of the last search string
	bool ignore_case_;	// ignore case flag for the last search
	bool whole_;		// whole word flag for the last search
	bool backwards_;	// backwards search flag
	FILE_FORMAT default_format_;

	QString line_;
	int x_;
	int y_;
	int xword_;
	int yword_;

	// buffer manager
	Q3PopupMenu *buffers_;

	// user area - containing exclusively only Q3MultiLineEdit
	QTabWidget *tw_;
	int tw_size_;
	Q3MultiLineEdit *tw_current_;
	Q3MultiLineEdit *tw_user_;
	Q3MultiLineEdit *tw_search_;
	Q3MultiLineEdit *tw_tags_;
	Q3MultiLineEdit *tw_bookmarks_;
	Q3MultiLineEdit *tw_hex_;

	// status bar
	QStatusBar *status_;
	LineAndColumn *lac_;
	InsertOverwrite *iow_;
	FormatField *ff_;

	// ctags
//#ifdef USING_QT4
	CtagsThread *thread_;
//#endif
	// mouse press/release y location to resize tags/search/usr window or dirfiles window
	int mouse_press_y_;
	int mouse_press_x_;
	bool resize_dirfiles_;

	// ctags locking/unlocking
	bool ctags_locked_;

};

#endif // QWERTY_H








