//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#include "spell.h"

// extract word from position inside line
char *extract_word(char *ptrin) {
	PROFILE("extract_word");

	if (!UTF8::isutf8alnum(ptrin))
		return 0;

	char *ptr = ptrin;
	if (*ptr == '-')
		ptr++;
	while (UTF8::isutf8alnum(ptr) || *ptr == '-') {
		unsigned len = UTF8::utf8CharLen(ptr);
		ptr += len;
	}

	unsigned long len = (unsigned long) ptr - (unsigned long) ptrin;
	char *retval = new char[len + 1];
	memcpy(retval, ptrin, len);
	retval[len] = '\0';
	return retval;
}

// same as above, only the length is returned
unsigned extract_word_length(char *ptrin) {
	PROFILE("extract_word");

	if (!UTF8::isutf8alnum(ptrin))
		return 0;

	char *ptr = ptrin;
	if (*ptr == '-')
		ptr++;
	while (UTF8::isutf8alnum(ptr) || *ptr == '-') {
		unsigned len = UTF8::utf8CharLen(ptr);
		ptr += len;
	}

	unsigned retval = (unsigned) ((unsigned long) ptr - (unsigned long) ptrin);
	return retval;
}

bool hasnumeric(const char *str) {
	ASSERT(str != 0);
	const char *ptr = str;
	while (*ptr != '\0') {
		if (isdigit(*ptr))
			return true;
		ptr++;
	}
	return false;
}


static inline unsigned advance(char *ptrin) {
	PROFILE("advance");
	unsigned retval = 1; // normal case
	if (UTF8::isutf8Punctuation(ptrin)) {
		retval = UTF8::utf8CharLen(ptrin);
		LOG("UTF8 punctuation: %x, %x, %x, %x, retval %d\n",
		    (unsigned) *ptrin,
		    (unsigned) *(ptrin + 1),
		    (unsigned) *(ptrin + 2),
		    (unsigned) *(ptrin + 3),
		    retval);
		if (retval == 0)
			retval = 1;
	}
	return retval;
}



char *spell(Dictionaries *pdict, Action *aptr, char *fstr) {
	PROFILE("spell");
	ASSERT(pdict != 0);
	ASSERT(fstr != 0);
	ASSERT(aptr != 0);
	LOG("length %d\n", strlen(fstr));

	// get stats pointer
	Stats *pstats = Stats::instance();

	DbDict *hptr = pdict->getMaster();
	DbUser *uptr = pdict->getUser();
	DbUser *iptr = pdict->getIgnore();
	DbUser *fptr = pdict->getFixed();
	char *pwstart = fstr; // process word
	while (*pwstart != '\0') {
		{
			while (!UTF8::isutf8alnum(pwstart) && *pwstart != '\0') {
				pwstart += advance(pwstart);
			}
			if (*pwstart == '\0')
				break;
		}

		// look for word in dictionary
		char *active_word = extract_word(pwstart);
//LOG("%s", active_word);
		if (active_word == 0)
			break;

		if (!hptr->findString(active_word) &&	// if not in dictionary
		    !fptr->findString(active_word, true) &&	 // if not in fixed/names dictionary
		    !uptr->findString(active_word) &&	// if not in user dictionary
		    !iptr->findString(active_word) &&	// if not in ignore dictionary
		    !hasnumeric(active_word)) {		 // is not a number

			pstats->notFound();

			if ((*aptr)(pdict, active_word)) {
				delete[] active_word;
				break;
			}
		} else
			pstats->found();
		pwstart += strlen(active_word);
		delete[] active_word;
	}

	return fstr;
}

void readHomeDir(Dictionaries *pdict) {
	if (pdict == 0)
		return;
	// find user file
	// save new dictionary
	char *homedir = getenv("HOME");
	if (homedir == 0) {
		fprintf(stderr, "Error: HOME environment variable not found\n");
		exit(1);
	}
	unsigned len = strlen(homedir) + strlen("/.rospell/dictionary.fixed");
	char *fname = new char[len + 1];
	memset(fname, '\0', len + 1);
	strcpy(fname, homedir);
	strcat(fname, "/.rospell/dictionary.fixed");
	char *fstr = UTILS::read_file_malloc(fname);
	if (fstr != 0) {
		Action *aptr;
		aptr = new ActionAddAllFixed();
		spell(pdict, aptr, fstr);
		delete aptr;
	} else {
		char *pmkdir = UTILS::run_prg_malloc("mkdir ~/.rospell");
		delete[] pmkdir;
		pmkdir = UTILS::run_prg_malloc("mkdir ~/.rospell/backup");
		delete[] pmkdir;
	}

	delete[] fname;
	delete[] fstr;

	len = strlen(homedir) + strlen("/.rospell/dictionary.user");
	fname = new char[len + 1];
	memset(fname, '\0', len + 1);
	strcpy(fname, homedir);
	strcat(fname, "/.rospell/dictionary.user");
	fstr = UTILS::read_file_malloc(fname);
	if (fstr != 0) {
		Action *aptr;
		aptr = new ActionAddAllUser();
		spell(pdict, aptr, fstr);
		delete aptr;
	}
	delete[] fname;
	delete[] fstr;

	Stats::instance()->reset();
	LOG("HOME file was read\n");
}


