// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///****************************************************************************

#include "syntax.h"
#include <qtextedit.h>

//*******************************************************************
// LaTex syntax
//*******************************************************************
int LaTexSyntaxHighlighter::highlightParagraph(const QString &text, int endStateOfLastPara) {
//LOG("Highlight Paragraph");
	(void) endStateOfLastPara;
	Global *g = Global::instance();

	setFormat(0, text.length(), g->c_foreground);

	int x = 0;
	QChar ch = text.at(x);
	
	bool start_kwd = false;
	while (!ch.isNull()) {
		// cpp-style comment
		if (ch == '%'  && !start_kwd) {
			setFormat(x - 1, text.length() - x + 1, g->c_tag3); 
			return 0;
		}

		if (ch == '\\'  && !start_kwd) {
			int start = x;
			ch = text.at(++x);
			if (ch == '\\') { // we have here the \\ keyword
				setFormat(start, 2, g->c_tag4); 
			}
			else {			
				while(ch.isLetterOrNumber())
					ch = text.at(++x);
				if (x > start + 1) {
					setFormat(start, x - start, g->c_tag4); 
					x--;
					start_kwd = true;
				}
			}
		}				
		// look for '{'
//		else if (ch == '{' && start_kwd) {
//			int start = x;
//			ch = text.at(++x);
//			while (!ch.isNull() && ch != '}')
//				ch = text.at(++x);
//			if (x > start + 1) {
//				setFormat(start, x - start + 1, g->c_tag4);
//			}
//		}
//		// look for '['
		else if (ch == '[' && start_kwd) {
			int start = x;
			ch = text.at(++x);
			while (!ch.isNull() && ch != ']')
				ch = text.at(++x);
			if (x > start + 1) {
				setFormat(start, x - start + 1, g->c_tag1);
			}
		}
		else {
			start_kwd = false;
			if (ch == '{')
				setFormat(x, 1, g->c_tag2);
			if (ch == '}')
				setFormat(x, 1, g->c_tag2);
		}	
		ch = text.at(++x);
	}
	return 0;
}



//*******************************************************************
// general xml syntax
//*******************************************************************
int XMLSyntaxHighlighter::highlightParagraph(const QString &text, int endStateOfLastPara) {
	Global *g = Global::instance();
	int retval = 0;
	bool tagfound = false;
	
	setFormat(0, text.length(), g->c_foreground);

	int x = 0;
	QChar ch = text.at(x);
	int start = (endStateOfLastPara == 1)? 0:-1;
	while (!ch.isNull()) {
		if (ch == '<')
			start = x;
		if (ch == '>' && start != -1) {
			setFormat(start, x - start + 1, g->c_tag1);
			tagfound = true;
			start = -1;
		}
		ch = text.at(++x);
		// skeep "<<" which usually appears in C/C++ text
		if (ch == '<' && text.at(x + 1) == '<') {
			ch = text.at(++x);
			ch = text.at(++x);
		}
	}

	if (start != -1) {
		setFormat(start, x - start, g->c_tag1);
		tagfound = true;
		retval = 1;
	}
	
	if (tagfound) {
		// look for " in text
		x = 0;
		ch = text.at(x);
		while (!ch.isNull()) {
			// strings
			if (ch == '"') {
				int startx = x;
				ch = text.at(++x);
				// advance to the end of string
				while (!ch.isNull() && ch != '"')
					ch = text.at(++x);
				if (ch == '"')	
					setFormat(startx, x - startx + 1, g->c_tag2);
				else
					setFormat(startx, x - startx, g->c_tag2);
			}
			
			if (!ch.isNull())
				ch = text.at(++x);
		}
	}
	
	return retval;
}



//*******************************************************************
// C/C++/Java/Javascript/csharp/php
//*******************************************************************


// cpp keywords
static char *keyw_cpp[] = { 
	"__FILE__", "__LINE__", "and_eq", "and", "asm", "auto", "bitand", "bitor",
	"bool", "break", "case", "catch", "char", "class",
	"compl", "const_cast", "const", "continue", "default",
	"delete", "do", "double", "dynamic_cast",  "else",
	"enum", "explicit", "export", "extern", "false",
	"float", "for", "friend", "goto", "if", "inline",
	"int", "long", "mutable", "namespace", "new", "not_eq",
	"not", "operator", "or_eq", "or", "private",
	"protected", "public", "register", "reinterpret_cast",
	"return", "short", "signed", "sizeof", "static",
	"static_cast", "struct", "switch", "template", "this",
	"throw", "true", "try", "typedef", "typeid",
	"typename", "union", "unsigned", "using", "virtual",
	"void", "volatile", "wchar_t", "while", "xor_eq", "xor",
	0
};

// java
static char *keyw_java[] = { 
	"abstract","break","case","catch","class","const",
	"continue","default","do","else","extends","final","false",
	"for","if","implements","import","instanceof","interface",
	"native","new","null","package","private","protected","public",
	"register","return","super","switch","synchronized","this",
	"true","throws","throw","transient","try","while",
	"boolean","byte","char","double","float","int","long","short",
	"static","void","volatile",
	0
};

// javascript
static char *keyw_javascript[] = { 
	"break", "case", "continue", "default", "delete", "do",
	"else", "export", "for", "function", "if", "import", "in",
	"new", "return", "switch", "this", "typeof", "var", "void",
	"while", "with",
	0
};

// csharp
static char *keyw_csharp[] = { 
	"abstract","as","base","break","case","catch","checked",
	"class","const","continue","default","do","else","enum",
	"event","explicit","extern","false","finally","fixed",
	"for","foreach","get","goto","if","implicit","in","interface",
	"internal","is","lock","namespace","new","null","operator",
	"override","params","private","protected","public","return",
	"sealed","set","sizeof","stackalloc","struct","switch","this",
	"throw","true","try","typeof","unchecked","unsafe","using",
	"value","virtual","while",
	"bool","byte","char","decimal","delegate","double","float",
	"int","long","object","readonly","ref","sbyte","short","static",
	"string","uint","ulong","out","ushort","void","volatile",
	0
};

// php
static char *keyw_php[] = { 
	"and","array","break","case","class","continue","date","declare","die",
	"double","do","if","elseif","else","empty","endif","endfor","endforeach",
	"endswitch","endwhile","echo","exit","extends","false","fclose","fgets",
	"fopen","foreach","for","fputs","function","global","header","include",
	"int","list","new","NULL","object","print","require","return","static",
	"string","switch","this","ticks","true","unset","var","while",
	0
};


int CBasedSyntaxHighlighter::checkKeyword(const QString &text) {
	if (text.isNull() || text.isEmpty())
		return 0;
		
	for (int i = 0; keyw_[i] != 0; i++) {
		if (text.startsWith(keyw_[i])) {
			QChar ch = text.at(strlen(keyw_[i]));
			if (ch.isNull() || ch == ' ' || ch == '(' || ch == ')' || ch == ';' ||
			    ch == ':' || ch == '\t' || ch == '*' || ch == ',')
				return strlen(keyw_[i]);
		}
	}
	
	return 0;
}

int CBasedSyntaxHighlighter::highlightParagraph(const QString &text, int endStateOfLastPara) {
//LOG("Highlight Paragraph");
	Global *g = Global::instance();

	setFormat(0, text.length(), g->c_foreground);

	int x = 0;
	QChar ch = text.at(x);
	if (endStateOfLastPara == 1) {
				int found = text.find("*/", x);
				if (found == -1) {
					setFormat(0, text.length(), g->c_tag3); 
					return 1;
				}

				x = found;
				setFormat(0, x + 2, g->c_tag3); 
				ch = text.at(++x);
	}
	while (!ch.isNull()) {
		// strings
		if (ch == '"') {
			int startx = x;
			ch = text.at(++x);
			// advance to the end of string
			while (!ch.isNull() && ch != '"')
				ch = text.at(++x);
			if (ch == '"')	
				setFormat(startx, x - startx + 1, g->c_tag2);
			else
				setFormat(startx, x - startx, g->c_tag2);
				
//			LOG("string highlight %d/%d", startx, x);
		}
		
		// single chars
		if (ch == '\'') {
			int startx = x;
			ch = text.at(++x);
			// advance to the end of string
			while (!ch.isNull() && ch != '\'')
				ch = text.at(++x);
			if (ch == '\'')	
				setFormat(startx, x - startx + 1, g->c_tag2);
			else
				setFormat(startx, x - startx, g->c_tag2);
				
//			LOG("string highlight %d/%d", startx, x);
		}
		
		
		// macros
		if (ch == '#') {
			// advance to the end of #define...
			while (!ch.isNull() && ch != ' ' && ch != '\t')
				ch = text.at(++x);
				
			setFormat(0, x, g->c_tag1);
		}
		
		// cpp-style comment
		if (ch == '/') {
			ch = text.at(++x);
			if (ch == '/') {
				setFormat(x - 1, text.length() - x + 1, g->c_tag3); 
				return 0;
			}
			else
				ch = text.at(--x);
		}
		
		// c-style comment
		if (ch == '/') {
			int startx = x;
			ch = text.at(++x);
			if (ch == '*') {
				x++;
				int found = text.find("*/", x);
				if (found == -1) {
					setFormat(startx, text.length() - startx, g->c_tag3); 
					return 1;
				}

				x = found;
				setFormat(startx, x - startx + 2, g->c_tag3); 
				ch = text.at(++x);
			}
		}
		
		int len;
		if ((len = checkKeyword(text.right(text.length() - x))) != 0) {
			setFormat(x, len, g->c_tag4); 
			x += len - 1;
			ch = text.at(x);
		}

		bool goneover = false;		
		while(ch.isLetterOrNumber() || ch == '_') {
			ch = text.at(++x);
			goneover = true;
		}
		if (!ch.isNull() && !goneover)
			ch = text.at(++x);

	}
	return 0;
}

//*******************************************************************
// diff - highlighter for diff results
//*******************************************************************
int DiffSyntaxHighlighter::highlightParagraph(const QString &text, int endStateOfLastPara) {
	Global *g = Global::instance();
	(void) endStateOfLastPara;

	setFormat(0, text.length(), g->c_foreground);

//	if (text.at(64) != ' ' || text.at(65) != ' ' || text.at(66) != ' ')
	if (text.at(0) == (char) 0x02)
		setFormat(0, text.length(), g->c_tag1);
	else if (text.at(0) == (char) 0x01)
		setFormat(0, text.length(), g->c_tag2);

	return 0;
}


//*******************************************************************
// Assembler: .asm (NASM) and .S (GAS)
//*******************************************************************

// asm keywords
static char *keyw_asm[] = {
	"aam","aas","adc","add","and","align","assume","bne","bpl","br","call","cmp",
	"cbw","clc","cwd","daa","das","db","dd","dec","dq","dt","dup","dw","endp",
	"ends","end","equ","far","global","idiv","div","in","inc","int","jae","ja",
	"jb","jcxz","jmp","jne","jns","jle","je","jl","jg","jng","jnl","jns","jnz",
	"jz","lodsb","local","loop","leave","lea","main",
	"mov", "movb", "movl", "movw", "imul","mul","near",
	"neg","not","or","org","out","pusha","push","popa","pop","proc","iret","rep",
	"ret","rol","seg","section","segment","shld","shl","shr","stosd","stosw",
	"sbb","sub","test","xchg","xor",
	"ah","al","ax",
	"bh","bl","bx",
	"ch","cl","cx",
	"dh","dl","dx",
	"eax","ebx","ecx","edx",
	"ss","ds","cs","es",
	0
};    


int AsmSyntaxHighlighter::checkKeyword(const QString &text) {
//LOG("checkKeyword %s", (const char *) text);
	if (text.isNull() || text.isEmpty())
		return 0;
		
	for (int i = 0; keyw_asm[i] != 0; i++) {
		if (text.startsWith(keyw_asm[i])) {
			QChar ch = text.at(strlen(keyw_asm[i]));
			if (ch.isNull() || ch == ' ' || ch == '(' || ch == ')' || ch == ';' ||
			    ch == ':' || ch == '\t' || ch == '*' || ch == ',')
				return strlen(keyw_asm[i]);
		}
	}
	
	return 0;
}

int AsmSyntaxHighlighter::highlightParagraph(const QString &text, int endStateOfLastPara) {
//LOG("Highlight Paragraph");
	(void) endStateOfLastPara;
	Global *g = Global::instance();

	setFormat(0, text.length(), g->c_foreground);

	int x = 0;
	QChar ch = text.at(x);
	while (!ch.isNull()) {
		// strings
		if (ch == '"') {
			int startx = x;
			ch = text.at(++x);
			// advance to the end of string
			while (!ch.isNull() && ch != '"')
				ch = text.at(++x);
			if (ch == '"')	
				setFormat(startx, x - startx + 1, g->c_tag2);
			else
				setFormat(startx, x - startx, g->c_tag2);
				
			LOG("string highlight %d/%d", startx, x);
		}
		
		// single chars
		if (ch == '\'') {
			int startx = x;
			ch = text.at(++x);
			// advance to the end of string
			while (!ch.isNull() && ch != '\'')
				ch = text.at(++x);
			if (ch == '\'')	
				setFormat(startx, x - startx + 1, g->c_tag2);
			else
				setFormat(startx, x - startx, g->c_tag2);
				
			LOG("string highlight %d/%d", startx, x);
		}

		// NASM style comment
		if (ch == ';') {
			setFormat(x, text.length() - x, g->c_tag3);
			return 0;
		}
		
		// GAS style comment
		if (ch == '#') {
			setFormat(x, text.length() - x, g->c_tag3);
			return 0;
		}
		
		
		int len;
		if ((len = checkKeyword(text.right(text.length() - x))) != 0) {
			setFormat(x, len, g->c_tag4); 
			x += len - 1;
			ch = text.at(x);
		}

		bool goneover = false;		
		while(ch.isLetterOrNumber() || ch == '_') {
			ch = text.at(++x);
			goneover = true;
		}
		if (!ch.isNull() && !goneover)
			ch = text.at(++x);

	}
	return 0;
}




//*******************************************************************
// Trolltech qmake/Perl
//*******************************************************************

// perl keywords
static char *keyw_perl[] = {
	"and","bless","chdir","chomp","chop","chr","case","delete",
	"die","do","each","else","elsif","exit","foreach","for",
	"function","if","in","join","keys","last","local","my","next",
	"no","null","or","package","pack","printf","print","push",
	"read","redo","require","return","seek","select","shift",
	"splice","split","sub","tell","tied","then","undef","unless",
	"untie","until","use","vars","warn","while","xor"
	"close","closedir","flock","mkdir","open","opendir","readdir",
	"rewinddir","rmdir","unlink",
	0
};    

// Trolltech qmake keywords
static char *keyw_qmake[] = {
// only "Frequently Used System Variables" are implemented;
// see http://doc.trolltech.com/3.0/qmake-commandreference.html for more
	"CONFIG", "DEFINES", "DEF_FILE", "DESTDIR", "DLLDESTDIR",
	"HEADERS", "INCLUDEPATH", "FORMS", "LEXSOURCES", "LIBS",
	"MOC_DIR", "OBJECTS_DIR", "REQUIRES", "SOURCES", "SUBDIR",
	"TARGET", "TEMPLATE", "VERSION", "YACCSOURCES",
	0
};    


// bash/shell keywords
static char *keyw_shell[] = {
	"case","done","do","elif","else","esac","exit","for","fi",
	"function","if","in","then","until","while", "return", "local",
	"alias","ar","awk","bg","cat","cc","cd","chgrp","chmod","clear","cmp",
	"compress","cp","date","dialog","diff","echo","export","fg","find","finger",
	"ftp","gcc","grep","gunzip","gzip","jobs","killall","kill","ln","ls",
	"less","make","merge","mkdir","more","mv","ping","ps","pwd","read","rm",
	"rmdir","sed","set","sort","sleep","tail","tar","test","time","touch",
	"uncompress","wc","which","whoami","who",
	0
};    

// python
static char *keyw_python[] = {
	"and","assert","break","class","continue","def","del","elif","else","except",
	"exec","finally","for","from","global","if","import","input","in","is",
	"lambda","not","or","pass","print","raise","range","return","try","while",
	0
};    



int ScriptBasedSyntaxHighlighter::checkKeyword(const QString &text) {
//LOG("checkKeyword %s", (const char *) text);
	if (text.isNull() || text.isEmpty())
		return 0;
		
	for (int i = 0; keyw_[i] != 0; i++) {
		if (text.startsWith(keyw_[i])) {
			QChar ch = text.at(strlen(keyw_[i]));
			if (ch.isNull() || ch == ' ' || ch == '(' || ch == ')' || ch == ';' ||
			    ch == ':' || ch == '\t' || ch == '*' || ch == ',')
				return strlen(keyw_[i]);
		}
	}
	
	return 0;
}

int ScriptBasedSyntaxHighlighter::highlightParagraph(const QString &text, int endStateOfLastPara) {
//LOG("Highlight Paragraph");
	(void) endStateOfLastPara;
	Global *g = Global::instance();

	setFormat(0, text.length(), g->c_foreground);

	int x = 0;
	QChar ch = text.at(x);
	while (!ch.isNull()) {
		// comment
		if (ch == '#') {
			setFormat(x, text.length() - x, g->c_tag3);
			return 0;
		}
		
		// strings ""
		if (ch == '"') {
			int startx = x;
			ch = text.at(++x);
			// advance to the end of string
			while (!ch.isNull() && ch != '"')
				ch = text.at(++x);
			if (ch == '"')	
				setFormat(startx, x - startx + 1, g->c_tag2);
			else
				setFormat(startx, x - startx, g->c_tag2);
				
			LOG("string highlight %d/%d", startx, x);
		}
		
		// strings ``
		if (ch == '`') {
			int startx = x;
			ch = text.at(++x);
			// advance to the end of string
			while (!ch.isNull() && ch != '`')
				ch = text.at(++x);
			if (ch == '`')	
				setFormat(startx, x - startx + 1, g->c_tag2);
			else
				setFormat(startx, x - startx, g->c_tag2);
				
			LOG("string highlight %d/%d", startx, x);
		}
		
		// single chars or strings ''
		if (ch == '\'') {
			int startx = x;
			ch = text.at(++x);
			// advance to the end of string
			while (!ch.isNull() && ch != '\'')
				ch = text.at(++x);
			if (ch == '\'')	
				setFormat(startx, x - startx + 1, g->c_tag2);
			else
				setFormat(startx, x - startx, g->c_tag2);
				
			LOG("string highlight %d/%d", startx, x);
		}
		
		int len;
		if ((len = checkKeyword(text.right(text.length() - x))) != 0) {
			setFormat(x, len, g->c_tag4); 
			x += len - 1;
			ch = text.at(x);
		}

		bool goneover = false;		
		while(ch.isLetterOrNumber() || ch == '_') {
			ch = text.at(++x);
			goneover = true;
		}
		if (!ch.isNull() && !goneover)
			ch = text.at(++x);

	}
	return 0;
}


//*******************************************************************
// factory function
//*******************************************************************
Q3SyntaxHighlighter *getSyntax(QString fname, Q3TextEdit *textEdit) {
	LOG("sintax for file %s", (const char *) fname);
	RospellSyntaxHighlighter *retval = 0;
	RospellSyntaxHighlighter *old = (RospellSyntaxHighlighter *) textEdit->syntaxHighlighter();
	QString old_name = "";
	if (old != 0)
		old_name = old->getName();
	
	if (fname.isNull() || fname.isEmpty()) {
		LOG("XML syntax highlighter for null/empty file name");
		if (old_name == "xml")
			retval = old;
		else {
			retval = new XMLSyntaxHighlighter(textEdit);
			retval->setName("xml");
		}
	}
	
	// chose a highlighter based on the file extension
	else if (fname.endsWith(".c") ||
	    fname.endsWith(".cpp") ||
	    fname.endsWith(".h") ||
	    fname.endsWith(".cc")) {
		if (old_name == "c")
			retval = old;
		else {
			retval = new CBasedSyntaxHighlighter(textEdit, keyw_cpp);
			retval->setName("c");
		}
	}
	else if (fname.endsWith(".java")) {
		LOG("use java syntax highlighter");
		if (old_name == "java")
			retval = old;
		else {
			retval = new CBasedSyntaxHighlighter(textEdit, keyw_java);
			retval->setName("java");
		}
	}
	else if (fname.endsWith(".js")) {
		if (old_name == "js")
			retval = old;
		else {
			retval = new CBasedSyntaxHighlighter(textEdit, keyw_javascript);
			retval->setName("js");
		}
	}
	else if (fname.endsWith(".cs")) {
		if (old_name == "cs")
			retval = old;
		else {
			retval = new CBasedSyntaxHighlighter(textEdit, keyw_csharp);
			retval->setName("cs");
		}
	}
	else if (fname.endsWith(".ph3") ||
	         fname.endsWith(".ph4") ||
	         fname.endsWith(".php") ||
	         fname.endsWith(".phps")) {
		if (old_name == "php")
			retval = old;
		else {
			retval = new CBasedSyntaxHighlighter(textEdit, keyw_php);
			retval->setName("php");
		}
	}
	else if (fname.endsWith(".S") ||
	         fname.endsWith(".asm")) {
		if (old_name == "asm")
			retval = old;
		else {
			retval = new AsmSyntaxHighlighter(textEdit);
			retval->setName("asm");
		}
	}
	else if (fname.endsWith(".pro")) {
		if (old_name == "qmake")
			retval = old;
		else {
			retval = new ScriptBasedSyntaxHighlighter(textEdit, keyw_qmake);
			retval->setName("qmake");
		}
	}
	else if (fname.endsWith(".pl")) {
		if (old_name == "perl")
			retval = old;
		else {
			retval = new ScriptBasedSyntaxHighlighter(textEdit, keyw_perl);
			retval->setName("perl");
		}
	}
	else if (fname.endsWith(".py")) {
		if (old_name == "python")
			retval = old;
		else {
			retval = new ScriptBasedSyntaxHighlighter(textEdit, keyw_python);
			retval->setName("python");
		}
	}
	else if (fname.endsWith(".sh")) {
		if (old_name == "sh")
			retval = old;
		else {
			retval = new ScriptBasedSyntaxHighlighter(textEdit, keyw_shell);
			retval->setName("sh");
		}
	}
	else if (fname.endsWith(".diff")) {
		if (old_name == "diff")
			retval = old;
		else {
			retval = new DiffSyntaxHighlighter(textEdit);
			retval->setName("diff");
		}
	}
	else if (fname.endsWith(".tex") || fname.endsWith(".sty")) {
		if (old_name == "tex")
			retval = old;
		else {
			retval = new LaTexSyntaxHighlighter(textEdit);
			retval->setName("tex");
		}
	}
	else { // this is where html goes!
		if (old_name == "xml")
			retval = old;
		else {
			retval = new XMLSyntaxHighlighter(textEdit);
			retval->setName("xml");
		}
	}
	
	ASSERT(retval != 0);
//if (old != retval)
//printf("new syntax %p\n", retval);
	return (Q3SyntaxHighlighter *) retval;
}

