//****************************************************************************
// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//****************************************************************************

#ifndef SYNTAX_H
#define SYNTAX_H

#include "global.h"
#ifdef USING_QT3
#include <qsyntaxhighlighter.h>
#endif
#ifdef USING_QT4
#include <q3syntaxhighlighter.h>
#endif

class RospellSyntaxHighlighter: public Q3SyntaxHighlighter {
public:
	RospellSyntaxHighlighter(Q3TextEdit *textEdit): Q3SyntaxHighlighter(textEdit) {
		name_ = "";
	}
	virtual ~RospellSyntaxHighlighter(void) {}

	virtual int highlightParagraph(const QString &text, int endStateOfLastPara) {
		(void) text;
		(void) endStateOfLastPara;
		return 0;
	}
	QString getName() {
		return name_;
	}
	void setName(QString name) {
		name_ = name;
	}
protected:
	QString name_;
};


// generic xml
class XMLSyntaxHighlighter : public RospellSyntaxHighlighter {
public:
	XMLSyntaxHighlighter(Q3TextEdit *textEdit): RospellSyntaxHighlighter(textEdit) {}
	virtual ~XMLSyntaxHighlighter(void) {}

	virtual int highlightParagraph(const QString &text, int endStateOfLastPara);
};

// c/c++/java/javascript
class CBasedSyntaxHighlighter : public RospellSyntaxHighlighter {
public:
	CBasedSyntaxHighlighter(Q3TextEdit *textEdit, char **keyw): RospellSyntaxHighlighter(textEdit), keyw_(keyw) {}
	virtual ~CBasedSyntaxHighlighter(void) {}

	virtual int highlightParagraph(const QString &text, int endStateOfLastPara);
private:
	int checkKeyword(const QString &text);
	char **keyw_;
};

// the result of diff command
class DiffSyntaxHighlighter : public RospellSyntaxHighlighter {
public:
	DiffSyntaxHighlighter(Q3TextEdit *textEdit): RospellSyntaxHighlighter(textEdit) {}
	virtual ~DiffSyntaxHighlighter(void) {}

	virtual int highlightParagraph(const QString &text, int endStateOfLastPara);
private:
	int checkKeyword(const QString &text);
};


// NASM/GAS assembler
class AsmSyntaxHighlighter : public RospellSyntaxHighlighter {
public:
	AsmSyntaxHighlighter(Q3TextEdit *textEdit):  RospellSyntaxHighlighter(textEdit) {}
	virtual ~AsmSyntaxHighlighter(void) {}

	virtual int highlightParagraph(const QString &text, int endStateOfLastPara);
private:
	int checkKeyword(const QString &text);
};


// qmake/perl
class ScriptBasedSyntaxHighlighter : public RospellSyntaxHighlighter {
public:
	ScriptBasedSyntaxHighlighter(Q3TextEdit *textEdit, char **keyw):  RospellSyntaxHighlighter(textEdit), keyw_(keyw) {}
	virtual ~ScriptBasedSyntaxHighlighter(void) {}

	virtual int highlightParagraph(const QString &text, int endStateOfLastPara);
private:
	int checkKeyword(const QString &text);
	char **keyw_;
};

// LaTex
class LaTexSyntaxHighlighter : public RospellSyntaxHighlighter {
public:
	LaTexSyntaxHighlighter(Q3TextEdit *textEdit):  RospellSyntaxHighlighter(textEdit) {}
	virtual ~LaTexSyntaxHighlighter(void) {}

	virtual int highlightParagraph(const QString &text, int endStateOfLastPara);
};




#endif

