// Copyright (C) 2005,  Lucian Constantin <struct_bylighting@yahoo.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#ifndef UTILS_H
#define UTILS_H

#include <string.h>
#include <ctype.h>
#include "debug.h"

class UTILS {
	/* list definition for run_prg_malloc() */
	static const int PROG_BUF = 10000;

	struct MyList {
		char buf[PROG_BUF + 1];
		MyList *next;

		MyList(): next(0) {
			memset(buf, 0, sizeof(buf));
		}

		~MyList() {
			if (next != 0) {
				delete next;
			}
		}
	};

public:
	static char *run_prg_malloc(const char *program) {
		PROFILE("UTILS::run_prg_malloc");
		FILE *input;
		MyList *ptr = new MyList();
		MyList *head = ptr;
		char *retval;
		int len;



		if (0 == (input = popen(program, "r"))) {
			delete head;
			return 0;
		} else {
			while ((len = fread(ptr->buf, 1, PROG_BUF, input)) > 0) {
				printf("Read %d bytes\n", len);
				ptr->next = new MyList();
				ptr = ptr->next;
			}

			pclose(input);
		}
		/* calculate length of return string */
		len = 0;
		ptr = head;
		while (ptr != 0) {
			len += strlen(ptr->buf);
			ptr = ptr->next;
		}
		
		if (len == 0) {
			delete head;
			return 0;
		}
		
		/* allocate retval +10 just in case... */
		if ((retval = new char[len + 10]) == 0) {
			delete head;
			return 0;
		}
		memset(retval, 0, len + 10);

		/* copy retval */
		ptr = head;
		while (ptr != 0) {
			strcat(retval, ptr->buf);
			ptr = ptr->next;
		}

		delete head;
		return retval;
	}


	static char *read_file_malloc(const char *name) {
		PROFILE("UTILS::read_file_malloc");
		FILE *input;
		int len;
		char *inbuf;
		ASSERT(name != 0);


		if ((input = fopen(name, "r")) == 0) {
			return 0;
		}

		/* find the length of the file - fstat and stat are not exactely ANSI C */
		fseek(input, 0, SEEK_END);
		len = ftell(input);
		if (len == 0)
			return 0;

		/* alloc and clean */
		if ((inbuf = new char[len + 10/* just in case ... */])  == 0) {
			fclose(input);
			printf("Cannot alloc memory for %s\n", name);
			LOG("UTILS::read_file_malloc: Cannot alloc memory for %s\n", name);
			return 0;
		}
		memset(inbuf, 0, len + 10);

		/* rewind and read the file */
		rewind(input);

		fread(inbuf, 1, len, input);
		if (ferror(input)) {
			fclose(input);
			delete[] inbuf;
			printf("\nCannot read file %s\n", name);
			LOG("ferror %d", ferror(input));
			LOG("UTILS::read_file_malloc: Cannot read file %s", name);
			return 0;
		}

		fclose(input);
		return inbuf;
	}

	// save str string as file fname
	static bool write_file(const char *fname, const char *str) {
		PROFILE("UTILS::write_file");
		ASSERT(fname != 0);
		ASSERT(str != 0);

		FILE *fp = fopen(fname, "w");
		if (fp == 0) {
			LOG("Utils::write_file: cannot open  file %s", fname);
			return false;
		}

		fprintf(fp, "%s", str);
		fclose(fp);
		return true;
	}

	// replace src with dest in instr buffer; a new buffer is allocated
	static char *buffer_sed(char *instr, char *src, char *dest,
					    bool preserve_input = true) {

		PROFILE("UTILS::buffer_sed");

		int i = 0;
		char *ptr = instr;
		while ((ptr = strstr(ptr, src)) != 0 && *ptr != '\0') {
			ptr++;
			i++;
		}

		// save a copy of the input buffer
		char *incopy = 0;
		if (preserve_input) {
			incopy = new char[strlen(instr) + 1];
			strcpy(incopy, instr);
			// if no replacement to be done, just return
			if (i == 0)
				return incopy;
		}

		// allocate the output buffer
		int len = strlen(instr) + i * strlen(dest) + 10;
		char *retval = new char[len];
		memset(retval, 0, len);

		ptr = instr;
		char *oldptr = ptr;
		while ((ptr = strstr(ptr, src)) != 0 && *ptr != 0) {
			*ptr = '\0';
			strcat(retval, oldptr);
			strcat(retval, dest);
			oldptr = ptr + strlen(src);
			ptr = oldptr;
		}
		strcat(retval, oldptr);

		// put back instr
		if (preserve_input) {
			ASSERT(incopy != 0);
			strcpy(instr, incopy);
		}
		delete[] incopy;
		return retval;
	}
};

#endif //UTILS_H
