#ifdef _WIN32
/****************************************************************************
**
** Copyright (C) 2004-2006 Trolltech AS. All rights reserved.
**
** This file is part of the documentation of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/
#include "httpwindow.h"


#ifdef USING_QT4
//#include <QHttp>
#include <Q3VBoxLayout>
#include <q3hbox.h>
#include <q3accel.h>
#include <QLabel>
#endif
#ifdef USING_QT3
//#include <qhttp.h>
#include <qlayout.h>
#include <qhbox.h>
#include <qaccel.h>
#include <qlabel.h>
#endif

#include <qlineedit.h>
#include <qpushbutton.h>
#include <qprogressdialog.h>
#include <qurl.h>
#include <qhttp.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include "debug.h"



HttpWindow::HttpWindow(QWidget *parent, const char *some_url)
    : QDialog(parent), autoclose(false)
{

	Q3VBoxLayout *vly = new Q3VBoxLayout(this, 10);
	vly->setAutoAdd(true);
	Q3HBox *hbox = new Q3HBox(this);
	hbox->setSpacing(10);
	urlLabel = new QLabel("&URL:", hbox);
	urlLineEdit = new QLineEdit(hbox);
	urlLabel->setBuddy(urlLineEdit);

	Q3HBox *hbox1 = new Q3HBox(this);
	hbox->setSpacing(10);
	statusLabel = new QLabel("Please enter the URL of a file you want to download", hbox1);
	
	// accept/reject
	Q3HBox *hbox2 = new Q3HBox(this);
	hbox2->setSpacing(10);
	quitButton = new QPushButton("Quit", hbox2);
	downloadButton = new QPushButton("Download", hbox2);
	downloadButton->setDefault(true);

    progressDialog = new QProgressDialog(this);
    http = new QHttp(this);
    connect(urlLineEdit, SIGNAL(textChanged(const QString &)),
            this, SLOT(enableDownloadButton()));
    connect(http, SIGNAL(requestFinished(int, bool)),
            this, SLOT(httpRequestFinished(int, bool)));
    connect(http, SIGNAL(dataReadProgress(int, int)),
            this, SLOT(updateDataReadProgress(int, int)));
    connect(http, SIGNAL(responseHeaderReceived(const QHttpResponseHeader &)),
            this, SLOT(readResponseHeader(const QHttpResponseHeader &)));
    connect(progressDialog, SIGNAL(canceled()), this, SLOT(cancelDownload()));
    connect(downloadButton, SIGNAL(clicked()), this, SLOT(downloadFile()));
    connect(quitButton, SIGNAL(clicked()), this, SLOT(close()));

//    setWindowTitle(tr("HTTP"));
	urlLineEdit->setFocus();
	if (some_url != 0) {
		autoclose = true;
		urlLineEdit->setText(QString(some_url));
		downloadFile();
	}
}

void HttpWindow::downloadFile()
{
    QUrl url(urlLineEdit->text());
    QFileInfo fileInfo(url.path());
    QString fileName = fileInfo.fileName();
    if (fileName.isNull() || fileName.isEmpty())
    	fileName = "index.html";
    fname_ = fileName;
    

    if (QFile::exists(fileName)) {
        QMessageBox::information(this, tr("HTTP"),
                                 tr("There already exists a file called %1 in "
                                    "the current directory.")
                                 .arg(fileName));
        return;
    }
    
    LOG("httpwindow file name to be saved: #%s#", (const char *) fileName);
    file = new QFile(fileName);

	
#ifdef USING_QT3
     if (!file->open(IO_WriteOnly )) {
#endif
#ifdef USING_QT4
    if (!file->open( QIODevice::WriteOnly )) {
#endif
        QMessageBox::information(this, tr("HTTP"),
                                 tr("Unable to save the file %1: %2.")
                                 .arg(fileName).arg(file->errorString()));
        delete file;
        file = 0;
        return;
    }

    http->setHost(url.host(), url.port() != -1 ? url.port() : 80);
//    if (!url.userName().isEmpty())
//        http->setUser(url.userName(), url.password());
    httpRequestAborted = false;
    httpGetId = http->get(url.path(), file);

//    progressDialog->setWindowTitle(tr("HTTP"));
    progressDialog->setLabelText(tr("Downloading %1.").arg(fileName));
    downloadButton->setEnabled(false);
}

void HttpWindow::cancelDownload()
{
    statusLabel->setText(tr("Download canceled."));
    httpRequestAborted = true;
    http->abort();
    downloadButton->setEnabled(true);
}

void HttpWindow::httpRequestFinished(int requestId, bool error)
{
    if (httpRequestAborted) {
        if (file) {
            file->close();
            file->remove();
            delete file;
            file = 0;
        }
         progressDialog->hide();
        return;
    }
     if (requestId != httpGetId)
        return;
     progressDialog->hide();
    file->close();
     if (error) {
        file->remove();
        QMessageBox::information(this, tr("HTTP"),
                                 tr("Download failed: %1.")
                                 .arg(http->errorString()));
    } else {
        QString fileName = QFileInfo(QUrl(urlLineEdit->text()).path()).fileName();
        statusLabel->setText(tr("Downloaded %1 to current directory.").arg(fileName));
    }
     downloadButton->setEnabled(true);
    delete file;
    file = 0;
    if (autoclose)
    	close();
}

void HttpWindow::readResponseHeader(const QHttpResponseHeader &responseHeader)
{
    if (responseHeader.statusCode() != 200) {
        QMessageBox::information(this, tr("HTTP"),
                                 tr("Download failed: %1.")
                                 .arg(responseHeader.reasonPhrase()));
        httpRequestAborted = true;
        progressDialog->hide();
        http->abort();
        return;
    }
}

void HttpWindow::updateDataReadProgress(int bytesRead, int totalBytes)
{
    if (httpRequestAborted)
        return;
#ifdef USING_QT4
    progressDialog->setMaximum(totalBytes);
    progressDialog->setValue(bytesRead);
#else
LOG("progress: %d/%d", bytesRead, totalBytes);
    progressDialog->setProgress(bytesRead, totalBytes);
#endif
}

void HttpWindow::enableDownloadButton()
{
    downloadButton->setEnabled(!urlLineEdit->text().isEmpty());
}


#endif //#ifdef _WIN32
